/*
 * This is a crude program to build the Directory Peek Help
 * TEXT/styl resource.  The input is intended to be editable by a
 * normal (non-formatted) text editor (MPW Shell or Think C).
 * Also, the source should be printable on a non-formatting
 * printer.
 *
 * The input format has several parameter lines followed by
 * the help text itself.
 *
 * Parameter lines:						Example
 *		Output Resource file name		"DTS Directory Peek Help.rsrc"
 *		STR# id							4000
 *		TEXT base id					4000
 *		Plain font name or number		geneva
 *		Monospace font name or number	monaco
 *		Default text height				9
 * Text format:
 *	* In column 1 is a comment.
 *	\ Quote next character
 *		\r		"hard" return.
 *		\XX		(two hex bytes) becomes that character:
 *				\D2			open double quote
 *				\D3			close double quote
 *				\D4			open single quote
 *				\D5			close single quote
 *				\A5			bullet
 *				\C9			...
 *		\?		Other character becomes that character.
 *	~ Format introducer
 *		~P		Plain text (switch to geneva, no formatting).
 *		~B		Bold style.
 *		~I		Italic style.
 *		~U		Underline style.
 *		~Hnn	Font height
 *		~M		Monospace font (monaco)
 *		~S		Symbol (magic) font
 *	The Symbol font is output as font zero (systemFont).  DoHelp
 *	changes it to some other font number (maybe).
 *
 *	The program outputs a STR# resource with topic titles
 *	and a series of TEXT and styl resources, beginning with
 *	the indicated id. I.e., if there are three topics, the
 *	program will output STR# 4000, TEXT 4000, 4001, and 4002.
 */

#define INPUT_FILE		"DTS Catalog Peek Help Text"
#ifndef VERBOSE
#define VERBOSE		0
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#ifdef THINK_C
#include <console.h>
#include <pascal.h>
#else
#include <Strings.h>
#include <Processes.h>
#include <Errors.h>
#include <Finder.h>
#include <Folders.h>
#include <Script.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Packages.h>
#include <Traps.h>
#include <Lists.h>
#endif
#include <setjmp.h>
jmp_buf				gJumpBuf;
char				*gErrorText;
/*
 * Failure(status, "cString");
 * FailOSErr(status, "cString");
 */
#define Failure(status, msg) do {						\
		gErrorText = (msg);								\
		longjmp(gJumpBuf, status);						\
	} while (0)
#define FailOSErr(status, msg) do {						\
		if ((status) != noErr)							\
			Failure(status, msg);						\
	} while (0)
#define FailNIL(what) do {								\
		if ((what) == NULL)								\
			Failure(MemError(), "Out of memory");		\
	} while (0)
#ifndef FALSE
#define FALSE		0
#define TRUE		1
#endif
#if VERBOSE
#define NOTE(x)		fprintf x
#else
#define NOTE(x)		/* Nothing */
#endif

enum {
	AtEOF			= 0,
	ReadOK			= 1,
	Comment			= 2
};

#define	MAX_TEXT	32766
Ptr					gTextPtr = NULL;		/* Text work area			*/
unsigned long		gTextSize = 0;			/* -> end of text work area	*/
StScrpHandle		gScrapHandle = NULL;	/* Text style built here	*/
short				gTopicSTRCount = 0;
Handle				gTopicHandle = NULL;	/* STR# built here			*/				
short				nStyles = 0;
unsigned char		theLine[513];
Str255				gResourceFileName;
Str255				gPlainFontName;
Str255				gMonospaceFontName;
short				gPlainFont;
short				gMonospaceFont;
short				gSymbolFont = systemFont;
short				gTopicResourceID;
short				gTextResourceID;
short				gFontSize;
short				gCurrentSize;
short				gCurrentFont;
short				gLineHeight;
short				gLineAscent;
short				gCurrentStyle;
short				gResFileRefNum = 0;
short				gApplResFile;
short				gDefaultVolRefNum;
GrafPort			gMyGrafPort;

int					main(int argc, char *argv[]);
void				ReadPreamble(void);
void				StoreInitialResources(void);
void				CleanOutResourceFile(void);
void				CleanOutResource(OSType);
void				ReadResourceFileName(void);
void				ReadNumber(short *);
void				ReadFont(StringPtr, short *);
void				ReadMonospaceFont(void);
Boolean				ReadHelpText(void);
short				ReadLine(void);
void				FixBackslashes(void);
void				DoFormat(void);
void				SetStyle(unsigned long index, Style theStyle);
void				SetFont(unsigned long index, short font);
void				SetSize(unsigned long index, short size);
void				SetCurrentSize(short);
unsigned short		hex(unsigned char c);
void				AppendTopicString(StringPtr);
void				AddStyle(long startChar);
void				WriteText(short topicIndex);
void				WriteStyle(short topicIndex);
void				WriteTopicResource(void);
void				CloseResourceFile(void);
#ifndef pstrcpy
/*
 * Cheap 'n dirty pascal string copy routine.
 */
#define pstrcpy(dst, src) do {				\
		BlockMove(src, dst,					\
			((unsigned char) src[0]) + 1);	\
	} while (0)
#endif

int
main(
		int			argc,
		char		*argv[]
	)
{
		int					status;		/* C int	*/
		short				nTopics;
		char				*inputFile;	/* C string	*/
				
#ifdef THINK_C
		console_options.pause_atexit = 0;
#endif
		InitGraf(&qd.thePort);
		InitFonts();
		gApplResFile = CurResFile();
		if ((status = setjmp(gJumpBuf)) == 0) {
			GetVol(theLine, &gDefaultVolRefNum);
			if (argc > 1 && argv[1] != NULL && argv[1][0] != '\0')
				inputFile = argv[1];
			else {
				inputFile = INPUT_FILE;
			}
			if (freopen(inputFile, "r", stdin) == NULL) {
				perror(inputFile);
				Failure(errno, "Can't open input");
			}
			OpenPort(&gMyGrafPort);
			gTextPtr = NewPtr(MAX_TEXT);
			nTopics = 0;
			gTopicHandle = NewHandleClear(sizeof (short));
			gTopicSTRCount = 0;
			ReadPreamble();
			StoreInitialResources();
			CleanOutResourceFile();
			while (ReadHelpText()) {
				FixBackslashes();
				DoFormat();
				WriteText(nTopics);
				WriteStyle(nTopics);
				++nTopics;
			}
			WriteTopicResource();
			CloseResourceFile();
		}
		else {
			fprintf(stderr, "Failure %d: %s\n", status, gErrorText);
#ifdef THINK_C
			console_options.pause_atexit = 1;
#endif
			exit(EXIT_FAILURE);
		}
		exit(EXIT_SUCCESS);
}

/*
 * Read the five preamble lines
 */
void
ReadPreamble()
{
		ReadResourceFileName();						/* Output file name			*/
		ReadNumber(&gTopicResourceID);				/* Help STR# resource id	*/
		ReadNumber(&gTextResourceID);				/* Help TEXT resource base	*/
		ReadFont(gPlainFontName, &gPlainFont);		/* Primary (text) font name	*/
		ReadFont(gMonospaceFontName, &gMonospaceFont); /* Monospace font name	*/
		ReadNumber(&gFontSize);						/* Primary font size		*/
		gCurrentSize = gFontSize;
		NOTE((
			stderr,
			"Plain font %#s - %d\n",
			gPlainFontName, gPlainFont
		));
		NOTE((
			stderr,
			"Monospace font %#s - %d\n",
			gMonospaceFontName, gMonospaceFont
		));
		NOTE((
			stderr,
			"Topic Resource id %d, Text Resource id %d, font size %d\n",
			gTopicResourceID, gTextResourceID, gFontSize
		));
		NOTE((
			stderr,
			"Resource file refnum %d\n", gResFileRefNum
		));
}

/*
 * Store the parameters in the STR# index.
 */
void
StoreInitialResources(void)
{
		Str255				work;
		
		NumToString(gTextResourceID, work);
		AppendTopicString(work);
		AppendTopicString(gPlainFontName);
		AppendTopicString(gMonospaceFontName);
		NumToString(gFontSize, work);
		AppendTopicString(work);
}

void
CleanOutResourceFile()
{
		UseResFile(gResFileRefNum);
		CleanOutResource('STR#');
		CleanOutResource('TEXT');
		CleanOutResource('styl');
		UpdateResFile(gResFileRefNum);
		UseResFile(gApplResFile);
}

/*
 * This removes *all* resources of the indicated type from the
 * current resource file. Use with caution.
 */
void
CleanOutResource(
		OSType			resourceType
	)
{
		Handle			handle;
		short			rCount;
		short			i;
		
		rCount = Count1Resources(resourceType);
		for (i = rCount; i >= 1; --i) {
			handle = Get1IndResource(resourceType, i);
			if (handle == NULL)
				break;
			RmveResource(handle);
			DisposHandle(handle);
		}
}

void
ReadResourceFileName()
{
		short			readStatus;
		
		while ((readStatus = ReadLine()) == Comment)
			;
		if (readStatus == ReadOK) {
			c2pstr((char *) theLine);
			pstrcpy(gResourceFileName, theLine);
			gResFileRefNum = OpenResFile(gResourceFileName);
			if (gResFileRefNum == (-1)) {
				NOTE((
					stderr,
					"Resource file %#s not found\n",
					gResourceFileName
				));
				CreateResFile(gResourceFileName);
				gResFileRefNum = OpenResFile(gResourceFileName);
				if (gResFileRefNum == (-1))
					Failure(ResError(), "Couldn't create resource file");
				NOTE((
					stderr,
					"Created resource file %#s\n",
					gResourceFileName
				));
			}
		}
		else {
			Failure(eofErr, "Strange eof");
		}
}

void
ReadNumber(
		short			*resultPtr
	)
{
		short			readStatus;
		
		while ((readStatus = ReadLine()) == Comment)
			;
		if (readStatus == ReadOK)
			*resultPtr = atoi((char *) theLine);
		else {
			Failure(eofErr, "Strange eof");
		}
}

void
ReadFont(
		StringPtr		resultName,
		short			*fontID
	)
{
		short			funnyFontID;
		short			readStatus;
		
		while ((readStatus = ReadLine()) == Comment)
			;
		if (readStatus == ReadOK) {
			funnyFontID = atoi((char *) theLine);
			c2pstr((char *) theLine);
			pstrcpy(resultName, theLine);
			GetFNum(resultName, fontID);
			if (*fontID == 0) {
				*fontID = funnyFontID;
				NOTE((
					stderr,
					"No such font %#s, using font number %d\n",
					resultName,
					funnyFontID
				));
			}
		}
		else {
			Failure(eofErr, "Strange eof");
		}
}

void
AppendTopicString(
		StringPtr			theString
	)
{
		FailOSErr(
			PtrAndHand(theString, gTopicHandle, theString[0] + 1),
			"Can't add topic"
		);
		++gTopicSTRCount;
}

/*
 * Read the entire help text into the work area.
 */
Boolean
ReadHelpText()
{
		short			thisLength;
		short			readStatus;
		StringPtr		theTopic;
		
		while ((readStatus = ReadLine()) == Comment)
			;
		if (readStatus != ReadOK)
			return (FALSE);		
		/*
		 * Append this line to the topic resource.
		 */
		theTopic = CtoPstr((char *) theLine);
		AppendTopicString(theTopic);
		gTextSize = 0;
		while (ReadLine() == ReadOK) {
			thisLength = strlen((char *) theLine);
			if (gTextSize + thisLength >= MAX_TEXT)
				Failure(memFullErr, "Too much text");
			strcpy(&gTextPtr[gTextSize], (char *) theLine);
			gTextSize += thisLength;
		}
		NOTE((stderr, "%lu text bytes read\n", gTextSize));
		return (TRUE);
}

/*
 * Read one line, comments exit. Return:
 *		AtEOF			at end
 *		ReadOK			read real data
 *		Comment			read '*' in column 1.
 */
short
ReadLine()
{		
		if (gets((char *) theLine) != NULL) {
			return ((theLine[0] == '*') ? Comment : ReadOK);
		}
		return (AtEOF);
}

void
FixBackslashes()
{
		register unsigned long		in, out;
		
		in = out = 0;
		for (; in < gTextSize; in++) {
			if (gTextPtr[in] != '\\' || (in + 1) >= gTextSize)
				gTextPtr[out] = gTextPtr[in];
			else {
				if (gTextPtr[in + 1] == 'r') {
					gTextPtr[out] = '\r';
					in += 1;
				}
				else if ((in + 2) < gTextSize
					  && isxdigit(gTextPtr[in + 1])
					  && isxdigit(gTextPtr[in + 2])) {
					 gTextPtr[out] = hex(gTextPtr[in + 1]) << 4 | hex(gTextPtr[in + 2]);
					 in += 2;
				}
				else {
					gTextPtr[out] = gTextPtr[in + 1];
					in += 1;
				}
			}
			out++;
		}
		gTextSize = out;
		NOTE((stderr, "Fix backslash, %lu in, %lu out\n", in, out));
}


unsigned short
hex(
		unsigned char		c
	)
{
		if (isdigit(c))
			return (c - '0');
		else {
			return (toupper(c) - 'A' + 10);
		}
}

void
DoFormat()
{
		register unsigned long		in, out;
		short						newSize;
		
		in = out = 0;
		SetFont(0, gPlainFont);
		for (; in < gTextSize; in++) {
			if (gTextPtr[in] != '~' || (in + 1) >= gTextSize)
ignore:			gTextPtr[out++] = gTextPtr[in];
			else {
				switch (gTextPtr[in + 1]) {
				default:	goto ignore;
				case 'P': case 'p':	SetFont(out, gPlainFont);		break;
				case 'M': case 'm':	SetFont(out, gMonospaceFont);	break;
				case 'S': case 's': SetFont(out, gSymbolFont);		break;
				case 'I': case 'i':	SetStyle(out, italic);			break;
				case 'U': case 'u': SetStyle(out, underline);		break;
				case 'B': case 'b':	SetStyle(out, bold);			break;
				case 'H': case 'h':
						newSize = 0;
						while (isdigit(gTextPtr[in + 2])) {
							newSize *= 10;
							newSize += gTextPtr[in + 2] - '0';
							in += 1;
						}
						SetSize(out, newSize);
				}
				in += 1;
			}
		}
		gTextSize = out;
		NOTE((stderr, "Do Format, %lu in, %lu out\n", in, out));
}

void
SetStyle(
		unsigned long		index,
		Style				fontStyle
	)
{
		gCurrentStyle = fontStyle;
		AddStyle(index);
}

void
SetFont(
		unsigned long		index,
		short				fontNumber
	)
{
		gCurrentFont = fontNumber;
		gCurrentStyle = 0;				/* Plain		*/
		TextFont(gCurrentFont);
		TextFace(gCurrentStyle);
		SetCurrentSize(gFontSize);		/* Default		*/
		AddStyle(index);
}

void
SetSize(
		unsigned long	index,
		short			size
	)
{
		SetCurrentSize(size);
		AddStyle(index);
}

void
SetCurrentSize(
		short			size
	)
{
		FontInfo			info;
		GrafPtr				oldPort;
		
		GetPort(&oldPort);
		SetPort(&gMyGrafPort);
		TextSize(size);
		GetFontInfo(&info);
		gCurrentSize = size;
		gLineHeight = info.ascent + info.descent + info.leading;
		gLineAscent = info.ascent;
		SetPort(oldPort);
}

void
AddStyle(
		long		startChar
	)
{
		static ScrpSTElement	newStyle;
		
		newStyle.scrpStartChar = startChar;
		newStyle.scrpHeight = gLineHeight;
		newStyle.scrpAscent = gLineAscent;
		newStyle.scrpFont = gCurrentFont;
		newStyle.scrpFace = gCurrentStyle;
		newStyle.scrpSize = gCurrentSize;
		if (gScrapHandle == NULL) {
			gScrapHandle = (StScrpHandle)
				NewHandleClear(sizeof (short));
			FailNIL(gScrapHandle);
		}
		SetHandleSize(
			(Handle) gScrapHandle,
			sizeof (short)
			+ ((nStyles + 1) * sizeof (ScrpSTElement))
		);
		FailOSErr(MemError(), "No memory");
		(**gScrapHandle).scrpStyleTab[nStyles] = newStyle;
		++nStyles;
		(**gScrapHandle).scrpNStyles = nStyles;
}

void
WriteText(
		short				nTopic
	)
{
		Handle				textHandle;
		
		FailOSErr(
			PtrToHand(gTextPtr, &textHandle, gTextSize),
			"Out of memory"
		);
		UseResFile(gResFileRefNum);
		AddResource(
			textHandle,
			'TEXT',
			gTextResourceID + nTopic,
			"\p"
		);
		FailOSErr(ResError(), "Can't add text resource");
		WriteResource(textHandle);
		ReleaseResource(textHandle);
		UseResFile(gApplResFile);
		gTextSize = 0;
}

void
WriteStyle(
		short				nTopic
	)
{
		if (gScrapHandle != NULL) {
			UseResFile(gResFileRefNum);
			AddResource(
				(Handle) gScrapHandle,
				'styl',
				gTextResourceID + nTopic,
				"\p"
			);
			FailOSErr(ResError(), "Can't add style resource");
			WriteResource((Handle) gScrapHandle);
			ReleaseResource((Handle) gScrapHandle);
			gScrapHandle = NULL;
			nStyles = 0;
			UseResFile(gApplResFile);
		}
}

/*
 * Write the topic index to the resource file.
 */
void
WriteTopicResource(void)
{
		(*(short *) (*gTopicHandle)) = gTopicSTRCount;
		UseResFile(gResFileRefNum);
		AddResource(
			gTopicHandle,
			'STR#',
			gTopicResourceID,
			"\p"
		);
		FailOSErr(ResError(), "Can't add topic resource");
		ReleaseResource(gTopicHandle);
		gTopicHandle = NULL;
		UseResFile(gApplResFile);
}

void
CloseResourceFile()
{
		FInfo			finderInfo;
		
		CloseResFile(gResFileRefNum);	/* Update and close	*/
		FailOSErr(ResError(), "CloseResFile error");
		FailOSErr(
			GetFInfo(gResourceFileName, gDefaultVolRefNum, &finderInfo),
			"GetFInfo error"
		);
		finderInfo.fdCreator = 'RSED';
		finderInfo.fdType = 'rsrc';
		FailOSErr(
			SetFInfo(gResourceFileName, gDefaultVolRefNum, &finderInfo),
			"SetFInfo error"
		);
		FailOSErr(
			FlushVol("\p", gDefaultVolRefNum),
			"FlushVol error"
		);
}
