/*							BrowseSetupCatalog.c								*/
/*
 * DTS Catalog Peek
 * BrowseSetupCatalog.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

#define SETUP_LIST		(DBR.setupCatalogList)
#define SETUP_BOUNDS	((**SETUP_LIST).dataBounds)

Boolean								GetNthSetupCatalog(
		register CatalogBrowserPtr		dbp,
		short							cellIndex,
		Boolean							setSelect,
		Handle							*result
	);
void								ProcessCurrentSetupCatalog(
		register CatalogBrowserPtr		dbp
	);

void
OpenSetupCatalog(
		register CatalogBrowserPtr		dbp
	)
{
		Rect						viewRect;

		/*
		 * Create a list (it will be displayed where
		 * the SDP Directory Panel normally goes.)
		 * Note that the BrowserWindow module has
		 * previously hidden the SDP Directory Panel.
		 */
		viewRect = RECT(kInSDPPanel);
		InsetRect(&viewRect, 4, 4);	/* Room for focus rect	*/
		SETUP_LIST = CreateList(
			dbp,
			&viewRect,
			DBR.typeFont.fontNumber,
			DBR.typeFont.fontSize,
			(ProcPtr) SetupCatalogLDEF
		);
		FailNIL(SETUP_LIST);
		DBR.currentSetupCatalogIndex = (-1);
		LActivate(TRUE, SETUP_LIST);
		EnumerateSetupCatalog(dbp);
		SelectNthSetupCatalog(dbp, 0);
}

/*
 * CloseSetupCatalog
 * No longer browsing. Clear out the list and
 * toss it.
 */
void
CloseSetupCatalog(
		register CatalogBrowserPtr		dbp
	)
{
		Rect						viewRect;
		
		if (SETUP_LIST != NULL) {
			ClearSetupCatalogList(dbp);
			viewRect = (**SETUP_LIST).rView;
			InsetRect(&viewRect, -4, -4);
			if ((**SETUP_LIST).vScroll != NULL)
				viewRect.right += kScrollBarWidth;
			if ((**SETUP_LIST).hScroll != NULL)
				viewRect.bottom += kScrollBarWidth;
			EraseRect(&viewRect);
			ValidRect(&viewRect);
			DisposeList(&SETUP_LIST);
		}
		ClearRecordDisplay(dbp);
}

/*
 * ClearSetupCatalogList
 * Clean out the list, but don't dispose the list handle.
 */
void
ClearSetupCatalogList(
		register CatalogBrowserPtr		dbp
	)
{
		short						i;
		Handle						infoHdl;
		
		FailNIL(SETUP_LIST);
		/*
		 * Delete the data handles that are stored in the
		 * attribute value list item, then delete the list
		 * contents.
		 */
		for (i = 0; i < SETUP_BOUNDS.bottom; i++) {
			GetNthSetupCatalog(dbp, i, FALSE, &infoHdl);
			ForgetHandle(infoHdl);
		}
		LDelRow(0, 0, SETUP_LIST);
		DBR.currentSetupCatalogIndex = (-1);
}

/*
 * AddSetupCatalogEntry
 * Add one entry (stored in the argument handle)
 * to the list.
 */
void
AddSetupCatalogEntry(
		register CatalogBrowserPtr		dbp,
		Handle							dirEnumSpecHdl
	)
{
		Cell				theCell;
		
		theCell.v = LAddRow(1, SETUP_BOUNDS.bottom, SETUP_LIST);
		FailMemError();
		theCell.h = 0;
		LSetCell(
			(Ptr) &dirEnumSpecHdl,
			sizeof (Handle),
			theCell,
			SETUP_LIST
		);
		FailMemError();
}

/*
 * DoClickInSetupCatalogList
 * The user clicked in the Setup directory list. Get the
 * new selection, ignoring double-clicks. If the selection
 * changed, TBS.
 */
void
DoClickInSetupCatalogList(
		register CatalogBrowserPtr		dbp
	)
{
		Cell				theCell;
		
		if (SETUP_LIST != NULL) {
			LClick(DBR.penPt, EVENT.modifiers, SETUP_LIST);
			theCell = LLastClick(SETUP_LIST);
			if (PtInRect(theCell, &SETUP_BOUNDS)
			 && theCell.v != DBR.currentSetupCatalogIndex) {
				SelectNthSetupCatalog(dbp, theCell.v);
			}
		}
}

/*
 * SelectNthSetupCatalog
 * Select a directory and (maybe) enumerate it.
 *
 * Globals
 *	DBR.dirEnumSpecHdl
 *			Set to the spec (handle format) if found.
 *	DBR.currentSetupCatalogIndex
 *			The index of the new spec.
 */
void
SelectNthSetupCatalog(
		register CatalogBrowserPtr		dbp,
		short							index
	)
{
		Boolean						foundIt;
		
		if (DBR.dirEnumSpecHdl != NULL)
			HSetState(DBR.dirEnumSpecHdl, DBR.enumSpecLockState);
		foundIt = GetNthSetupCatalog(
				dbp,
				index,
				TRUE,
				&DBR.dirEnumSpecHdl
			);
		if (foundIt && DBR.dirEnumSpecHdl != NULL) {
			DBR.currentSetupCatalogIndex = index;
			DBR.enumSpecLockState = GetDirEnumSpecFromHandle(
					DBR.dirEnumSpecHdl,
					&DBR.currentEnumSpec
				);
			ProcessCurrentSetupCatalog(dbp);
		}
}

/*
 * GetNthSetupCatalog
 * Get selected directory's handle, return TRUE if there was one.
 */
Boolean
GetNthSetupCatalog(
		register CatalogBrowserPtr		dbp,
		short							cellIndex,
		Boolean							setSelect,
		Handle							*result
	)
{
		short							dataLen;
		Cell							theCell;
		Boolean							gotOne;
		
		SetPt(&theCell, 0, cellIndex);
		if (PtInRect(theCell, &SETUP_BOUNDS) == FALSE) {
		 	*result = NULL;
		 	gotOne = FALSE;
		}
		else {
			dataLen = sizeof (Handle);
			LGetCell(
				result,
				&dataLen,
				theCell,
				SETUP_LIST
			);
			if (setSelect)
				LSetSelect(TRUE, theCell, SETUP_LIST);
			gotOne = TRUE;
		}
		return (gotOne);
}
		
/*
 * ProcessCurrentSetupCatalog
 * If it's a record, try to evaluate it. If it's a directory,
 * expand it (someday). The current setup catalog is identified
 * by its index value.
 */
void
ProcessCurrentSetupCatalog(
		register CatalogBrowserPtr		dbp
	)
{
#define SPEC	(DBR.currentEnumSpec)

		switch (SPEC.enumFlag) {
		case kEnumDistinguishedNameMask:
		case kEnumAliasMask:
		case kEnumPseudonymMask:
			/*
			 * We have a record of some sort. Try to
			 * enumerate it like a normal application.
			 */
			SETUP_DIR_RecordID.local = SPEC.u.recordIdentifier;
			SETUP_DIR_RecordID.rli = NULL;
			SETUP_DIR_PDRefNum = DBR.setupCatalogRefNum;
			EnumerateAttributeTypes(dbp, &DBR.setupCatalogInfo);
			break;
		case kEnumDNodeBit:
			ClearRecordDisplay(dbp);
			/* TBS: display the DNode contents */
			break;
		}
}


