/*									BrowserWindow.c								*/
/*
 * DTS Catalog Peek
 * BrowserWindow.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * This file contains the "main" browser window routines. This includes functions
 & to create, dispose, and draw the browser window. The window contents are also
 * defined here. This module is responsible for the visual appearance of the
 * Catalog Browser, but has very little AOCE-specific code (just a few calls to
 * the Standard Directory Panel Manager).
 *
 * Note: this module offers a clear example, if examples were needed, of why you
 * should design your programs before you write them. My apologies for the mess.
 *
 * Globals relevant to the Browser Window itself.
 *	gOffsetWindow
 *			This is a crude "tile window" variable that creates new windows so
 *			they don't obscure the previous window. It will not be satisfactory
 *			for more than one or two additional windows.
 * gUpdateMenusNeeded
 *			Set TRUE to update menu enable state.
 * gOpenWindowCount
 *			The number of windows currently open. When the last window is closed,
 *			the application exits.
 *	DBR.window (also WINDOW)
 *			This is the window that displays the current directory browser data.
 *	DBR.panelHandle
 *			This is the SDP Directory Panel displayed in this window.
 *	DBR.pdPanelHandle
 *			This is a SDP Directory Panel directed to the user's personal directory.
 *	DBR.iconPopupControl
 *			This is a popup menu control (see Inside Mac VI) that lets the user
 *			select the icon type that will be displayed by the low-level Directory
 *			Manager call.
 *	DBR.browsePopupControl
 *	DBR.browserChoice
 *			The popup lets you specify what will be displayed. It sets the state
 *			of the DBR.browserChoice selector.
 *	DBR.showEntityRadioControl
 *	DBR.showDSSpecRadioControl
 *	DBR.showCatalog
 *			These two radio buttons specify the content of the data display panel.
 *			The choice is needed as there isn't enough room to show everything on
 *			a "Classic" screen. The radio buttons set the state of the
 *			DBR.showCatalog Boolean.
 *	DBR.vampSICNIndex
 *	DBR.nextAnimation
 *			These two variables are needed to drive the animated icon that twirls
 *			while the SDP Panel is busy interrogating the network.
 *	DBR.windowFocus
 *			The index (into RECT) of the currently-active window object.
 *	DBR.penPt
 *			The current pen location in local coordinates
 *	DBR.labelFont	(Geneva   9)
 *	DBR.typeFont	(Times   10)
 *	DBR.valueFont	(Geneva  10)
 *	DBR.dataFont	(Courier 10)
 *			These contain the font information (FontInfo, fontNumber, fontSize,
 *			and lineHeight) needed to draw information. Note that that typeFont,
 *			valueFont, and dataFont must have identical fontSize and fontAscent.
 *	DBR.errorLog
 *			This ListHandle stores current error and status messages.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode


/*
 * This is a crude "offset window" offsetter. You can do better.
 */
static short gOffsetValue			= 0;

/*
 * Local function prototypes
 */
static void							MakeControls(
		register CatalogBrowserPtr		dbp
	);
static ControlHandle				MakeRadioButton(
		register CatalogBrowserPtr		dbp,
		const Rect						*viewRect,
		ConstStr255Param				titleString,
		Boolean							selected
	);
void								UpdateSDPPanel(
		register CatalogBrowserPtr		dbp
	);
void								UpdateAttributeTypeList(
		register CatalogBrowserPtr		dbp
	);
void								UpdateAttributeValueList(
		register CatalogBrowserPtr		dbp
	);
void								UpdateDataDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								UpdateSetupCatalogList(
		register CatalogBrowserPtr		dbp
	);
static void							ClearDataDisplay(
		register CatalogBrowserPtr		dbp
	);
void								UpdateCatalogIcons(
		register CatalogBrowserPtr		dbp
	);
static WindowFocus					DoControlClick(
		register CatalogBrowserPtr		dbp,
		ControlHandle					theControl
	);
static WindowFocus					SelectBrowser(
		register CatalogBrowserPtr		dbp,
		BrowserChoice					newBrowseMethod
	);
static void							SelectDataDisplay(
		register CatalogBrowserPtr		dbp,
		Boolean							newShowCatalog
	);
static void							SetWindowFocus(
		register CatalogBrowserPtr		dbp,
		WindowFocus						newWindowFocus
	);
static void							GetIconPosition(
		register CatalogBrowserPtr		dbp,
		short							index,
		Rect							*iconRect
	);

/*
 * MakeCatalogBrowser
 * Create a window that holds the SDP Browser Window and its content.
 */
void
MakeCatalogBrowser(void)
{
		register CatalogBrowserPtr		dbp;
		WindowPtr						theWindow;
		Rect							windowShape;
		Rect							viewRect;
		
		dbp = (CatalogBrowserPtr)
			NewPtrClear(sizeof (CatalogBrowserRecord));
		FailNIL(dbp);
		/*
		 * We need a working data buffer, too.
		 */
		DBR.buffer = NewPtr(kMyBufferSize);
		FailNIL(DBR.buffer);
		SetRect(&windowShape, 0, 0, kMinWindowWidth, kMinWindowHeight);
#if 0	/* Normal: position the window in the center of the display				*/
		PositionRect(&windowShape);
#else	/* Debug: force the window to the upper-left corner						*/
		OffsetRect(&windowShape, 4, GetMBarHeight() * 2);
#endif
		OffsetRect(&windowShape, gOffsetValue, gOffsetValue);
		gOffsetValue = (gOffsetValue > 100) ? 0 : (GetMBarHeight() * 2);
		if (gHasColorQuickDraw) {
			theWindow = GetNewCWindow(WIND_Browser, (Ptr) dbp, (WindowPtr) -1L);
		}
		else {
			theWindow = GetNewWindow(WIND_Browser, (Ptr) dbp, (WindowPtr) -1L);
		}
		FailNIL(theWindow);
		SetPort(theWindow);						/* Do this first, or else		*/
		MoveWindow(theWindow, windowShape.left, windowShape.top, TRUE);
		/*
		 * Get some font info to position text items in the lists and
		 * descriptive frames. Type font, value font, and data font
		 * must be the same size.
		 */
		GetFontData(&DBR.labelFont,	kLabelFontName,		kLabelFontSize);
		GetFontData(&DBR.typeFont,	kRStringFontName,	kDataFontSize);
		GetFontData(&DBR.valueFont,	kValueFontName,		kDataFontSize);
		GetFontData(&DBR.dataFont,	kDataFontName,		kDataFontSize);
		DecorateDisplay(dbp, FALSE);			/* Set initial item shape		*/
		FailNIL(DBR.recordName = (RStringHandle) NewHandle(0));
		FailNIL(DBR.recordType = (RStringHandle) NewHandle(0));
		DBR.windowFocus = kUnknownFocus;		/* Preset focus rect			*/
		DBR.browserChoice = kUnknownBrowser;	/* Preset Browse popup			*/
		DBR.showCatalog = FALSE;				/* Preset radio button			*/
		DBR.errorLog = CreateLog(
				&RECT(kInErrorLog),
				DBR.labelFont.fontNumber,
				DBR.labelFont.fontSize,
				0								/* Use default nLines			*/
			);
		SetUserIdentity(dbp);					/* Before creating SDP Panel	*/
		InitializeVampDisplay(dbp);				/* Before creating SDP Panel	*/
		DBR.panelHandle = CreateSDPDirectoryPanel(
				dbp,
				&RECT(kInSDPPanel),
				NULL
			);
		DBR.pdPanelHandle = CreateSDPDirectoryPanel(
				dbp,
				&RECT(kInSDPPanel),
				(PackedRLIPtr) -1
			);
		ValidRect(&RECT(kInSDPPanel));
		/*
		 * Build the lists
		 */
		DBR.attributeTypeList = CreateList(
			dbp,
			&RECT(kInAttributeTypeList),
			DBR.typeFont.fontNumber,
			DBR.typeFont.fontSize,
			(ProcPtr) RStringLDEF
		);
		DBR.attributeValueList = CreateList(
			dbp,
			&RECT(kInAttributeValueList),
			DBR.valueFont.fontNumber,
			DBR.valueFont.fontSize,
			(ProcPtr) AttributeValueLDEF
		);
		DBR.dataDisplayList = CreateList(
			dbp,
			&RECT(kInDataDisplayList),
			DBR.valueFont.fontNumber,
			DBR.valueFont.fontSize,
			(ProcPtr) DataDisplayLDEF
		);
		/*
		 * Since DBR.showCatalog is FALSE at start, create the twist-down
		 * (directory) list offscreen. Note that the twist-down list rectangle
		 * includes the scroll bar.
		 */
		viewRect = RECT(kInDataDisplayList);
		OffsetRect(&viewRect, 4096, 4096);
		DBR.directoryDisplayList = NewTwistDownList(
					&viewRect,
					DrawCatalogListCell,
					CharWidth('M'),
					FALSE							/* No hilite in the list	*/
				);
		FailNIL(DBR.directoryDisplayList);
		(**DBR.directoryDisplayList).refCon = (long) dbp;
		HideList(DBR.directoryDisplayList);
		ShowList(DBR.dataDisplayList);
		FailNIL(DBR.directoryDisplayList);
		InitializeIconSelector(dbp);
		MakeControls(dbp);
		SelectBrowser(dbp, kBrowseSDPPanel);
		SetWindowFocus(dbp, kInSDPPanel);
		ShowWindow(theWindow);
		SelectWindow(theWindow);
		gUpdateMenusNeeded = TRUE;
		++gOpenWindowCount;
}

/*
 * MakeControls
 * Create the various buttons. They should be in resources.
 */
static void
MakeControls(
		register CatalogBrowserPtr		dbp
	)
{
		DBR.showEntityRadioControl = MakeRadioButton(
				dbp,
				&RECT(kInShowEntityRadioButton),
				"\pShow Entity Data",
				!(DBR.showCatalog)
			);
		DBR.showDSSpecRadioControl = MakeRadioButton(
				dbp,
				&RECT(kInShowCatalogRadioButton),
				"\pShow Expanded Catalog",
				DBR.showCatalog
			);
		TextFont(DBR.labelFont.fontNumber);
		TextSize(DBR.labelFont.fontSize);
		DBR.browsePopupControl = NewControl(
				WINDOW,						/* Display window		*/
				&RECT(kInBrowsePopup),
				"\p", 						/* No Title				*/
				TRUE,						/* Visible				*/
				popupTitleCenterJust,		/* value: title adjust	*/
				MENU_Browser,				/* min:   menu resource	*/
				0,							/* max:   popup width	*/
				popupMenuProc				/* Popup control proc	*/
				 | popupUseWFont,			/*  Draw in window font	*/
				0							/* RefCon (unused)		*/
			);
		FailNIL(DBR.browsePopupControl); 	/* Oops?				*/
}

/*
 * MakeRadioButton
 * Create one radio button. Fail if we're out of memory
 */
static ControlHandle
MakeRadioButton(
		register CatalogBrowserPtr		dbp,
		const Rect						*viewRect,
		ConstStr255Param				titleString,
		Boolean							selected
	)
{
		ControlHandle				result;
		
		result = NewControl(
				WINDOW,					/* Display window		*/
				viewRect,
				titleString,		 	/* Title				*/
				TRUE,					/* Visible				*/
				(selected) ? 1 : 0,		/* Initial value		*/
				0,						/* min					*/
				1,						/* max					*/
				radioButProc			/* Radio button			*/
				 |popupUseWFont,		/*  Draw in window font	*/
					0					/* RefCon (unused)		*/
			);
		FailNIL(result);
		return (result);
}

/*
 * DisposeCatalogBrowser
 * The user clicked on the closer. Dispose of the window
 * and its contents.
 */
void
DisposeCatalogBrowser(
		register CatalogBrowserPtr		dbp
	)
{
		DisposeSDPDirectoryPanel(dbp, &DBR.panelHandle);
		DisposeSDPDirectoryPanel(dbp, &DBR.pdPanelHandle);
		ClearRecordDisplay(dbp);
		DisposeList(&DBR.attributeTypeList);
		DisposeList(&DBR.attributeValueList);
		DisposeList(&DBR.dataDisplayList);
		DisposeList(&DBR.directoryDisplayList);
		ForgetHandle(DBR.recordName);
		ForgetHandle(DBR.recordType);
		if (DBR.directoryListHead != NULL) {
			DisposeTwistDownHdl(DBR.directoryListHead);
			DBR.directoryListHead = NULL;
		}
		ForgetPtr(DBR.buffer);
		DisposeIconSuiteHandle(dbp, &DBR.SDPIconSuite);
		DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
		ForgetPtr(DBR.partsPtr);
		DisposeLog(DBR.errorLog);
		CloseWindow(WINDOW);
		DisposePtr((Ptr) dbp);
		gUpdateMenusNeeded = TRUE;
		--gOpenWindowCount;
}

/*
 * ActivateCatalogBrowser
 * This is called on activate and deactivate (or suspend/resume)
 * events to enable/disable the window content. The SDP
 * Panel manages its own activation.
 */
void
ActivateCatalogBrowser(
		register CatalogBrowserPtr		dbp,
		Boolean							activating
	)
{
		LActivate(activating, DBR.attributeTypeList);
		LActivate(activating, DBR.attributeValueList);
		if (DBR.showCatalog) {
			LActivate(activating, DBR.directoryDisplayList);
		}
		else {
			LActivate(activating, DBR.dataDisplayList);
		}
		if (DBR.setupCatalogList != NULL)
			LActivate(activating, DBR.setupCatalogList);
		ActivateLog(DBR.errorLog, activating);
}

/*
 * DoWindowKeyDown
 * This is called when a key is hit in the window and
 * no list-specific routine absorbs it. It only looks
 * for tabs, which switch the window focus around.
 */
void
DoWindowKeyDown(
		register CatalogBrowserPtr		dbp
	)
{		
		if ((EVENT.message & charCodeMask) == '\t')
			SelectNextWindowObject(dbp);
}

/*
 * DoContentClick
 * Click in the window. This function decides which element
 * received the click and calls the requisite click handler.
 * Note that there is no click handler for the SDP Directory
 * Panel as that is handled by the SDPPanelEvent function
 * which is called each time through the event loop.
 */
void
DoContentClick(
		register CatalogBrowserPtr		dbp
	)
{
		WindowFocus				newWindowFocus;
		ControlHandle			theControl;
		short					partCode;
		
		DBR.penPt = EVENT.where;
		GlobalToLocal(&DBR.penPt);
		newWindowFocus = kUnknownFocus;
		if (PtInRect(DBR.penPt, &RECT(kInSDPPanel))
		 && (DBR.browserChoice == kBrowseSDPPanel
		  || DBR.browserChoice == kBrowseSetupCatalog))
			newWindowFocus = kInSDPPanel;
		else if (PtInRect(DBR.penPt, &RECT(kInAttributeTypeList)))
			newWindowFocus = kInAttributeTypeList;
		else if (PtInRect(DBR.penPt, &RECT(kInAttributeValueList)))
			newWindowFocus = kInAttributeValueList;
		else if (PtInRect(DBR.penPt, &RECT(kInDataDisplayList)))
			newWindowFocus = kInDataDisplayList;
		else if (PtInRect(DBR.penPt, &RECT(kInErrorLog))) {
			DoClickInLog(DBR.errorLog, &EVENT);
			/* Note that the focus doesn't change */
		}
		else {
			partCode = FindControl(DBR.penPt, WINDOW, &theControl);
			if (partCode != 0)
				newWindowFocus = DoControlClick(dbp, theControl);
			else {
				/* Random click: ignored */
			}
		}
		if (newWindowFocus != kUnknownFocus) {
			SetWindowFocus(dbp, newWindowFocus);
			switch (DBR.windowFocus) {
			case kInSDPPanel:
				switch (DBR.browserChoice) {
				case kBrowseSDPPanel:
					break;	/* SDPPanelEvent will handle this */
				case kBrowseSetupCatalog:
					DoClickInSetupCatalogList(dbp);
					break;
				case kBrowseSetupRecord:
					newWindowFocus = kUnknownFocus;
					break;
				}
				break;
			case kInAttributeTypeList:
				DoClickInAttributeTypeList(dbp);
				break;
			case kInAttributeValueList:
				DoClickInAttributeValueList(dbp);
				break;
			case kInDataDisplayList:
				DoClickInDataDisplayList(dbp);
				break;
			}
		}
}

/*
 * DoControlClick
 * The user clicked in a control. Handle the control (setting
 * radio buttons and redrawing parts of the window as needed)
 * and return a new window focus, if necessary. It would be
 * reasonable to break this into several sub-functions, each
 * handling a different control.
 */
static WindowFocus
DoControlClick(
		register CatalogBrowserPtr		dbp,
		ControlHandle					theControl
	)
{
		WindowFocus				newWindowFocus;
		short					partCode;
		Boolean					newShowCatalog;

		newWindowFocus = kUnknownFocus;
		/*
		 * The controls are in the window font. Ensure that the font
		 * is correct before calling TrackControl.
		 */
		TextFont(DBR.labelFont.fontNumber);
		TextSize(DBR.labelFont.fontSize);
		TextFace(normal);
		if (theControl == DBR.iconPopupControl) {
			/*
			 * You must pass (ProcPtr) -1 to make the popup work.
			 * Don't ask me, I only work here.
			 */
			partCode = TrackControl(theControl, DBR.penPt, (ProcPtr) -1);
			if (partCode != 0)
				GetSelectedCatalogIcon(dbp);
		}
		else if (theControl == DBR.browsePopupControl) {
			partCode = TrackControl(theControl, DBR.penPt, (ProcPtr) -1);
			if (partCode != 0) {
				newWindowFocus = SelectBrowser(
						dbp,
						GetCtlValue(DBR.browsePopupControl)
					);
			}
		}
		else if (theControl == DBR.showEntityRadioControl
			  || theControl == DBR.showDSSpecRadioControl) {
			/*
			 * Manage the Show Entity radio button choice
			 */
			partCode = TrackControl(theControl, DBR.penPt, NULL);
			if (partCode != 0) {
				newShowCatalog = (theControl == DBR.showDSSpecRadioControl);
				SelectDataDisplay(dbp, newShowCatalog);
				newWindowFocus = kInDataDisplayList;
			}
		}
		else {
			/* Clicked on a random piece of the window -- ignored			*/
		}
		return (newWindowFocus);
}

/*
 * SelectBrowser
 * Select the browser method. There is some error recovery here to force
 * the selection back to the SDP panel in case OpenSetupRecord fails.
 */
static WindowFocus
SelectBrowser(
		register CatalogBrowserPtr		dbp,
		BrowserChoice					newBrowseMethod
	)
{
		Boolean						tryAgainFlag;
		WindowFocus					newWindowFocus;
		OSErr						status;
		Boolean						oldPanelBrowser;
		Boolean						newPanelBrowser;
		
		tryAgainFlag = FALSE;
		newWindowFocus = kUnknownFocus;
		do {
			/*
			 * Did things change?
			 */
			if (newBrowseMethod != DBR.browserChoice) {
				ClearRecordDisplay(dbp);
				/*
				 * Start with a clean slate (using the old format)
				 */
				switch (DBR.browserChoice) {
				case kBrowseSDPPanel:
					oldPanelBrowser = TRUE;
					DisposeSDPSelection(dbp);
					status = SDPHidePanel(DBR.panelHandle);
					LOG(status, "\pSDPHidePanel panelHandle");
					break;
				case kBrowsePDPanel:
					oldPanelBrowser = TRUE;
					DisposeSDPSelection(dbp);
					status = SDPHidePanel(DBR.pdPanelHandle);
					LOG(status, "\pSDPHidePanel pdPanelHandle");
					break;
				case kBrowseSetupCatalog:
					oldPanelBrowser = FALSE;
					CloseSetupCatalog(dbp);
					break;
				case kBrowseSetupRecord:
					oldPanelBrowser = FALSE;
					CloseSetupRecord(dbp);
					break;
				}
				DBR.browserChoice = newBrowseMethod;
				switch (DBR.browserChoice) {
				default:
					DBR.browserChoice = kBrowseSDPPanel;
					/* Fall through */
				case kBrowseSDPPanel:
					newPanelBrowser = TRUE;
					newWindowFocus = kInSDPPanel;
					DisposeSDPSelection(dbp);
					SDP_INFO_PDRefNum = 0;
					status = SDPShowPanel(DBR.panelHandle);
					LOG(status, "\pSDPShowPanel");
					break;
				case kBrowsePDPanel:
					newPanelBrowser = TRUE;
					newWindowFocus = kInSDPPanel;
					DisposeSDPSelection(dbp);
					SDP_INFO_PDRefNum = 0;
					status = SDPShowPanel(DBR.pdPanelHandle);
					LOG(status, "\pSDPShowPanel");
					break;
				case kBrowseSetupCatalog:
					newPanelBrowser = FALSE;
					SetWindowFocus(dbp, kUnknownFocus);
					TRY {
						OpenSetupCatalog(dbp);
						newWindowFocus = kInSDPPanel;
					}
					CATCH {
						/*
						 * Oops. Force the selection back to the SDP Panel.
						 */
						LOG(STATUS, "\pOpenSetupCatalog recovery");
						tryAgainFlag = TRUE;
						newBrowseMethod = kBrowseSDPPanel;
						NO_PROPAGATE;
					}
					ENDTRY;
					newWindowFocus = kInSDPPanel;
					break;					
				case kBrowseSetupRecord:
					newPanelBrowser = FALSE;
					SetWindowFocus(dbp, kUnknownFocus);
					TRY {
						newWindowFocus = kInAttributeTypeList;
						OpenSetupRecord(dbp);
					}
					CATCH {
						/*
						 * Oops. Force the selection back to the SDP Panel.
						 */
						LOG(STATUS, "\pOpenSetupRecord recovery");
						tryAgainFlag = TRUE;
						newBrowseMethod = kBrowseSDPPanel;
						newWindowFocus = kInSDPPanel;
						NO_PROPAGATE;
					}
					ENDTRY;
					break;
				}
			}
		} while (tryAgainFlag);
		SetCtlValue(DBR.browsePopupControl, DBR.browserChoice);
		if (oldPanelBrowser != newPanelBrowser) {
			/*
			 * If we switch from one of the panels to one of the built-in
			 * lists, disable the radio buttons and force their value to
			 * "display catalog data". Else, if we switch from one of the
			 * built-in lists to one of the catalogs, re-enable the buttons
			 * and restore their value. Otherwise, do nothing.
			 */
			TextFont(DBR.labelFont.fontNumber);
			TextSize(DBR.labelFont.fontSize);
			TextFace(normal);
			if (newPanelBrowser) {
				/*
				 * Re-enable the radio buttons.
				 */
				SelectDataDisplay(dbp, DBR.oldShowCatalog);
				HiliteControl(DBR.showEntityRadioControl, kActiveButton);
				HiliteControl(DBR.showDSSpecRadioControl, kActiveButton);
			}
			else {
				/*
				 * Disable the radio buttons and force the display to
				 * "display catalog record value"
				 */
				DBR.oldShowCatalog = DBR.showCatalog;
				SelectDataDisplay(dbp, FALSE);
				HiliteControl(DBR.showEntityRadioControl, kDisabledButton);
				HiliteControl(DBR.showDSSpecRadioControl, kDisabledButton);
			}
		}
		return (newWindowFocus);
}


void
SelectDataDisplay(
		register CatalogBrowserPtr		dbp,
		Boolean							newShowCatalog
	)
{
		Rect							viewRect;
		
		if (newShowCatalog != DBR.showCatalog) {
			ClearDataDisplay(dbp);			/* Clear old data		*/
			DBR.showCatalog = newShowCatalog;
			TextFont(DBR.labelFont.fontNumber);
			TextSize(DBR.labelFont.fontSize);
			TextFace(normal);
			if (DBR.showCatalog) {
				SetCtlValue(DBR.showEntityRadioControl, 0);
				SetCtlValue(DBR.showDSSpecRadioControl, 1);
				HideList(DBR.dataDisplayList);
				ShowList(DBR.directoryDisplayList);
			}
			else {
				SetCtlValue(DBR.showEntityRadioControl, 1);
				SetCtlValue(DBR.showDSSpecRadioControl, 0);
				HideList(DBR.directoryDisplayList);
				ShowList(DBR.dataDisplayList);
			}
			SetWindowFocus(dbp, kInDataDisplayList);
			InvalRect(&RECT(kInRecordLengthText));
			InvalRect(&RECT(kInDataDisplayList));
			if (DBR.showCatalog)
				MakeCatalogDisplayList(dbp);
			else {
				SelectNthAttributeValue(dbp, DBR.currentAttributeValueIndex);
				viewRect = (**DBR.dataDisplayList).rView;
				ValidRect(&viewRect);
			}
		}
}

/*
 * ClearRecordDisplay
 * Clear out the entire record display.
 */
void
ClearRecordDisplay(
		register CatalogBrowserPtr		dbp
	)
{
		DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
		DisposeIconSuiteHandle(dbp, &DBR.SDPIconSuite);
		RefreshCatalogIcons(dbp);
		ClearRecordNameAndType(dbp);
		ClearAttributeTypeList(dbp);
		ClearDataDisplay(dbp);
}

/*
 * ClearRecordNameAndType
 * Clear the data and force redraw.
 */
void
ClearRecordNameAndType(
		register CatalogBrowserPtr		dbp
	)
{
		SetHandleSize((Handle) DBR.recordName, 0);
		SetHandleSize((Handle) DBR.recordType, 0);
		UpdateRecordNameAndType(dbp);
}

/*
 * SelectNextWindowObject
 * The user tabbed to a new window section. Reset the focus.
 */
void
SelectNextWindowObject(
		register CatalogBrowserPtr		dbp
	)
{
		WindowFocus				newWindowFocus;
		
		if ((newWindowFocus = DBR.windowFocus + 1) > kLastFocus)
			newWindowFocus = kFirstFocus;
		SetWindowFocus(dbp, newWindowFocus);
}

/*
 * UpdateBrowserWinow
 * Process an update event in this window.
 */
void
UpdateBrowserWindow(
		register CatalogBrowserPtr		dbp
	)
{
		TextMode(srcCopy);
		if (DBR.panelHandle != NULL
		 && (**DBR.panelHandle).visible) {
			FailOSErr(
				SDPUpdatePanel(DBR.panelHandle, WINDOW->visRgn)
			);
		}
		if (DBR.pdPanelHandle != NULL
		 && (**DBR.pdPanelHandle).visible) {
			FailOSErr(
				SDPUpdatePanel(DBR.pdPanelHandle, WINDOW->visRgn)
			);
		}
		TextFont(DBR.labelFont.fontNumber);
		TextSize(DBR.labelFont.fontSize);
		TextFace(normal);
		MyDrawGrowIcon(WINDOW);
		UpdateVampDisplay(dbp);
		UpdateRecordNameAndType(dbp);
		UpdateAttributeTypeList(dbp);
		UpdateAttributeValueList(dbp);
		UpdateDataDisplayList(dbp);
		UpdateSetupCatalogList(dbp);
		UpdateLog(DBR.errorLog);
		UpdateCatalogIcons(dbp);
		UpdateVampDisplay(dbp);
		DrawCurrentCatalogIcon(dbp);
		TextFace(normal);
		TextFont(DBR.labelFont.fontNumber);
		TextSize(DBR.labelFont.fontSize);
		UpdateControls(WINDOW, WINDOW->visRgn);
}

/*
 * UpdateRecordNameAndType
 * Redraw the record name and type strings.
 */
void
UpdateRecordNameAndType(
		register CatalogBrowserPtr		dbp
	)
{
		TextFont(DBR.typeFont.fontNumber);
		TextSize(DBR.typeFont.fontSize);
		DrawRStringInFrame(DBR.recordName, &RECT(kInLocalRecordName));
		DrawRStringInFrame(DBR.recordType, &RECT(kInLocalRecordType));
}

/*
 * UpdateAttributeTypeList
 * Redraw the attribute type list and its focus rectangle
 */
void
UpdateAttributeTypeList(
		register CatalogBrowserPtr		dbp
	)
{
		UpdateList(
			DBR.attributeTypeList,
			(DBR.windowFocus == kInAttributeTypeList)
		);
}

/*
 * UpdateAttributeValueList
 * Redraw the attribute value list and its focus rectangle
 */
void
UpdateAttributeValueList(
		register CatalogBrowserPtr		dbp
	)
{
		UpdateList(
			DBR.attributeValueList,
			(DBR.windowFocus == kInAttributeValueList)
		);
}

/*
 * UpdateDataDisplayList
 * Redraw the attribute value data display list and its focus rectangle.
 * Also, redraw the length of the current data
 */
void
UpdateDataDisplayList(
		register CatalogBrowserPtr		dbp
	)
{
		UpdateList(
			(DBR.showCatalog)
				? DBR.directoryDisplayList
				: DBR.dataDisplayList,
			(DBR.windowFocus == kInDataDisplayList)
		);
		DrawDataRecordLength(dbp);
}

/*
 * UpdateSetupCatalogList
 * Update the setup directory list, if it's present.
 */
void
UpdateSetupCatalogList(
		register CatalogBrowserPtr		dbp
	)
{
		if (DBR.browserChoice == kBrowseSetupCatalog
		 && DBR.setupCatalogList != NULL) {
			UpdateList(
				DBR.setupCatalogList,
				(DBR.windowFocus == kInSDPPanel)
			);
		}
}

/*
 * ClearDataDisplay
 * Erase the data display area. Clear both display areas,
 * even though one isn't on screen.
 */
static void
ClearDataDisplay(
		register CatalogBrowserPtr		dbp
	)
{
		ClearCatalogDataDisplay(dbp);
		ClearAttributeDataDisplay(dbp);
}

/*
 * RefreshCatalogIcons
 * Redraw the SDP directory icons.
 */
void
RefreshCatalogIcons(
		register CatalogBrowserPtr		dbp
	)
{
		Rect							iconRect;
		short							i;
		
		for (i = 0; i < 2; i++) {
			GetIconPosition(dbp, i, &iconRect);
			InvalRect(&iconRect);
		}
		RefreshGetCatalogIcons(dbp);
}		

/*
 * UpdateCatalogIcons
 * Plot the SDP directory icons, if any. If none are currently defined,
 * just frame the icon rectangle.
 */
void
UpdateCatalogIcons(
		register CatalogBrowserPtr		dbp
	)
{
		Rect							iconRect;
		short							i;
		
		for (i = 0; i < 2; i++) {
			GetIconPosition(dbp, i, &iconRect);
			if (DBR.SDPIconSuite == NULL)
				FrameRect(&iconRect);
			else {
				FailOSErr(
					PlotIconSuite(
						&iconRect,			/* Plot it here, please	*/
						atNone,				/* No alignment			*/
						ttNone,				/* No transformation	*/
						DBR.SDPIconSuite	/* The icon to plot		*/
					)
				);
			}
		}
}

/*
 * GetIconPosition
 * Given an icon index, return the position of the icon in
 * the display window. Icon 0 is "large", Icon 1 is "small"
 */
static void
GetIconPosition(
		register CatalogBrowserPtr		dbp,
		short							index,
		Rect							*iconRect
	)
{
#pragma unused (dbp)
		short				iconSize;
		
		iconRect->top = RECT(kInSDPCatalogIcon).top;
		iconRect->left = RECT(kInSDPCatalogIcon).left;
		if (index == 0)
			iconSize = 32;
		else {
			iconSize = 16;
			iconRect->top += 4;
			iconRect->left += 38;
		}
		iconRect->right = iconRect->left + iconSize;
		iconRect->bottom = iconRect->top + iconSize;
}

/*
 * GetFontData
 * This is called when the window is created. It retrieves the
 * information we need to draw in this font. This function follows
 * the rules for locating missing fonts, but this ought not to be
 * a problem (and the code paths haven't been tested).
 */
void
GetFontData(
		register FontDataPtr			fontDataPtr,
		short							fontNameIndex,
		short							fontSize
	)
{
#define FONT		(*fontDataPtr)
		Str255						fontName;
		Str255						systemFontName;

		GetIndString(fontName, STRN_Parameters, fontNameIndex);
		GetFNum(fontName, &FONT.fontNumber);
		FONT.fontSize = fontSize;
		if (FONT.fontNumber == 0) {
			GetFontName(0, systemFontName);
			if (EqualString(
						fontName, systemFontName, FALSE, FALSE
					) == FALSE) {
				/*
				 * No such font. Use default.
				 */
				FONT.fontNumber = applFont;
			}
		}
		TextFont(FONT.fontNumber);
		TextSize(FONT.fontSize);
		GetFontInfo(&FONT.info);
		FONT.lineHeight = FONT.info.ascent
						+ FONT.info.descent
						+ FONT.info.leading;
#undef FONT
}

/*
 * SaveCatalogBrowserInfo
 * Write the information to a text file. This is a placeholder
 * for future expansion.
 */
void
SaveCatalogBrowserInfo(
		register CatalogBrowserPtr		dbp
	)
{
		LOG(userCanceledErr, "\pTBS");
}

/*
 * SetWindowFocus
 * This is called when the window focus changes. It causes
 * the focus rectangle to be drawn (or erased) around the
 * appropriate list.
 */
static void
SetWindowFocus(
		register CatalogBrowserPtr		dbp,
		WindowFocus						newWindowFocus
	)
{
		ListHandle					listHandle;
		
		if (newWindowFocus != DBR.windowFocus) {
			switch (DBR.windowFocus) {		/* Erase old focus	*/
			case kInSDPPanel:
				switch (DBR.browserChoice) {
				case kBrowseSDPPanel:
					if (DBR.panelHandle != NULL) {
						FailOSErr(SDPSetFocus(DBR.panelHandle, FALSE));
					}
					break;
				case kBrowsePDPanel:
					if (DBR.pdPanelHandle != NULL) {
						FailOSErr(SDPSetFocus(DBR.pdPanelHandle, FALSE));
					}
					break;
				case kBrowseSetupCatalog:
					listHandle = DBR.setupCatalogList;
					goto eraseFocusRect;
				case kBrowseSetupRecord:
					/* Nothing happens here */
					break;
				}
				break;
			case kInAttributeTypeList:
				listHandle = DBR.attributeTypeList;
				goto eraseFocusRect;
			case kInAttributeValueList:
				listHandle = DBR.attributeValueList;
				goto eraseFocusRect;
			case kInDataDisplayList:
				if (DBR.showCatalog)
					listHandle = DBR.directoryDisplayList;
				else {
					listHandle = DBR.dataDisplayList;
				}
				/* goto eraseFocusRect; */
eraseFocusRect:	if (listHandle != NULL)
					DrawFocusRect(listHandle, patBic);
			}
			DBR.windowFocus = newWindowFocus;
			switch (DBR.windowFocus) {		/* Set new focus	*/
			case kInSDPPanel:
				switch (DBR.browserChoice) {
				case kBrowseSDPPanel:
					if (DBR.panelHandle != NULL) {
						FailOSErr(SDPSetFocus(DBR.panelHandle, TRUE));
					}
					break;
				case kBrowsePDPanel:
					if (DBR.pdPanelHandle != NULL) {
						FailOSErr(SDPSetFocus(DBR.pdPanelHandle, TRUE));
					}
					break;
				case kBrowseSetupCatalog:
					listHandle = DBR.setupCatalogList;
					goto drawFocusRect;
				case kBrowseSetupRecord:
					/* Nothing happens here */
					break;
				}
				break;
			case kInAttributeTypeList:
				listHandle = DBR.attributeTypeList;
				goto drawFocusRect;
			case kInAttributeValueList:
				listHandle = DBR.attributeValueList;
				goto drawFocusRect;
			case kInDataDisplayList:
				if (DBR.showCatalog)
					listHandle = DBR.directoryDisplayList;
				else {
					listHandle = DBR.dataDisplayList;
				}
drawFocusRect:	if (listHandle != NULL)
					DrawFocusRect(listHandle, patOr);
				break;
			}
		}
}

