/*									DTSCatalogPeek.h							*/
/*
 * DTS Catalog Peek
 * DTSCatalogPeek.h
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 */
#ifndef _DTSCatalogPeek_
#define _DTSCatalogPeek_
/*
 * These definitions are common to ReZ and code sources.
 */
#define kApplicationCreator	'DPEK'		/* Registered Creator Code				*/
#define kApplicationType	'APPL'
#define STRN_Parameters		(128)
#define ICON_Application	(128)
#define ALRT_NonFatal		(9000)
#define ALRT_Fatal			(9010)
#define ALRT_Continue		(9020)
#define ALRT_Info			(9030)
#define DLOG_Help			(4000)
#define STRn_Help			(9000)
#define HELP_Browser		(1024)
#define WIND_Browser		(1024)

#define MBAR_MenuBar		(1)
#define MENU_Apple			(1)
#define MENU_File			(256)
#define MENU_Edit			(257)
#define MENU_Icons			(258)
#define MENU_Browser		(259)
#define kMinWindowWidth		500
#define kMinWindowHeight	310

#ifndef REZ
#ifndef THINK_C				/* MPW includes			*/
#include <Processes.h>
#include <Errors.h>
#include <Finder.h>
#include <Folders.h>
#include <Script.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Packages.h>
#include <Traps.h>
#include <Lists.h>
#endif
#include <GestaltEqu.h>
#include "Failure.h"
#include "LogManager.h"
#include "TwistDownList.h"
/*
 * IconSuite (in this folder) contains the System 7 icon plotting definitions.
 * If your version of the MPW or Think headers contains a correct Icon.h, you
 * should use that, rather than this temporary expedient. The Icon Suite
 * utilities are described in Tech Note M.IM.IconDrawingTheSystem7Way.
 */
#if 0
#include <Icons.h>
#else
#include "IconSuite.h"
#endif
#include <OCE.h>
#include <OCEAuthDir.h>
#include <OCEStandardDirectory.h>
#include <OCEErrors.h>

#ifndef EXTERN
#define EXTERN				extern
#endif
#ifndef TRUE
#define TRUE				1
#define FALSE				0
#endif
#ifndef NULL
#define NULL				0
#endif
#ifndef topLeft
#define topLeft(rect)	(((Point *) &(rect))[0])
#define botRight(rect)	(((Point *) &(rect))[1])
#endif
#define SYNC				0
#define ASYNC				1
#define kMyBufferSize		(33L * 1024L)	/* Working data buffer	*/
enum {										/* Display characters	*/
	NUL					= 0,
	kOpenDoubleQuote	= 0xD2,
	kCloseDoubleQuote	= 0xD3,
	kOpenSingleQuote	= 0xD4,
	kCloseSingleQuote	= 0xD5
};
/*
 * These are indices into the parameter strings used to configure the display.
 * They are stored in STR# STRN_Parameters.
 */
enum {
	kPanelFontName	= 1,		/* Palatino			*/
	kLabelFontName,				/* Geneva			*/
	kRStringFontName,			/* Times			*/
	kValueFontName,				/* Geneva			*/
	kDataFontName,				/* Courier			*/
	kPanelFontSize				/* 12				*/
};
/*
 * Because the rest of the window layout depends on the size of various pieces
 * of text, we don't put these values into the parameter resource file.
 */
#define kLabelFontSize			(9)
#define kDataFontSize			(10)
#define kBytesPerLine			(16)
#ifndef __CharCodesDefined__
#define __CharCodesDefined__
enum {										/* Keyboard commands	*/
	charEnter			= 0x03,				/* Enter key			*/
	charEscape			= 0x1B,
	charHome			= 0x01,				/* Extended keyboard	*/
	charEnd				= 0x04,
	charArrowLeft		= 0x1C,
	charArrowRight		= 0x1D,
	charArrowUp			= 0x1E,
	charArrowDown		= 0x1F,
	charPageUp			= 0x0B,
	charPageDown		= 0x0C,
	charHelp			= 0x05
};
enum {										/* Scroll bar constants				*/
	kScrollBarWidth 	= 16,
	kScrollBarOffset	= kScrollBarWidth - 1,
	kInvisibleCtrl		= 0,				/* contrlVis values					*/
	kVisibleCtrl		= 0xFF,
	kActiveButton		= 0,				/* HiliteControl: normal button		*/
	kDisabledButton		= 255				/* HiliteControl: disabled button	*/
};
#endif
#define MakeControlVisible(ctrl)	((**ctrl).contrlVis = kVisibleCtrl)
#define MakeControlInvisible(ctrl)	((**ctrl).contrlVis = kInvisibleCtrl)
/*
 * This is the information returned by DirGetDirectoryInfo and the
 * DirEnumerateDirectoriesParse callback, formatted into a convenient package.
 */
typedef struct CatalogInfoRecord {
	DirectoryName		directoryName;
	DirDiscriminator	discriminator;
	DirGestalt			features;
} CatalogInfoRecord, *CatalogInfoPtr, **CatalogInfoHandle;

/*
 * We store attributes in a list by unpacking the fixed-length stuff into a
 * private structure, and stuffing the variable-length data into a handle.
 * GetHandleSize gives us the size of the variable-length data. We could do
 * the entire thing in one handle, of course.
 */
typedef struct AttributeHeaderInfo {
	AttributeType		attributeType;
	AttributeCreationID	cid;
	AttributeTag		tag;
	Handle				dataHandle;
} AttributeHeaderInfo, *AttributeHeaderInfoPtr;

/*
 * WindowFocus (in DBR.windowFocus) identifies the part of the window that the
 * user last clicked in. This is needed to draw the thick-line focus rectangle,
 * as well as to enable the Edit/Copy menu option. This enum is also used to
 * organize the visual display: every distinct piece of the visual puzzle is
 * listed here. The RECT(windowFocus) macro access the item rectangle for
 * the current window in its current shape and size.
 */
typedef enum {
	kUnknownFocus = 0,
	kWindowShape = 0,					/* The window shape (in itemRect)		*/
	kInSDPPanel,
	kInAttributeTypeList,
	kInAttributeValueList,
	kInDataDisplayList,
	kInRecordLengthText,
	kInErrorLog,
	kInSDPCatalogIcon,
	kInGetCatalogIconPopup,
	kInGetCatalogIconItself,
	kInLocalRecordName,
	kInLocalRecordType,
	kInBrowsePopup,
	kInShowEntityRadioButton,
	kInShowCatalogRadioButton,
	kInSpinnerRect,
	nItemRect,							/* Number of visual items				*/
	kFirstFocus = kInSDPPanel,
	kLastFocus = kInDataDisplayList,
	kFirstItemRect = kInSDPPanel,
	kLastItemRect = kInSpinnerRect
} WindowFocus;

typedef enum {
	kUnknownBrowser = 0,
	kBrowseSDPPanel,
	kBrowsePDPanel,
	kBrowseSetupCatalog,
	kBrowseSetupRecord
} BrowserChoice;

/*
 * This collects everything we need to know about a font
 * into one convenient place.
 */
typedef struct FontData {
	FontInfo			info;
	short				fontNumber;
	short				fontSize;
	short				lineHeight;
} FontData, *FontDataPtr;

/*
 * This is the information needed to enumerate/display records. The Record ID
 * identifies the record, and the dsRefNum the directory containing it. The three
 * (modal) paths into the enumerator (SDP Panel, Browse Setup Catalog, and
 * Browse Setup Record) create these and pass a pointer to the current record to
 * the enumeration routines. Note that the record ID contains pointers to other
 * data.
 */
typedef struct RecordInfo {
	RecordID			recordID;				/* Record to browse				*/
	unsigned short		dsRefNum;				/* Which personal directory		*/
} RecordInfo, *RecordInfoPtr;

/*
 * This is the information needed to manage the directory display list. It is
 * stored in the "data" portion of a TwistDownList.
 */
typedef enum {
	kElementString,
	kElementMessage,
	kElementRString,
	kElementCode
} ElementFormat;

typedef struct ElementRecord {
	union {
		short			filler;
		ElementFormat	format;					/* Format of the data			*/
	} header;
	unsigned char		data[1];				/* String, RString or whatever	*/
} ElementRecord, *ElementPtr;
#define ELEM_HEADER_SIZE	(sizeof (ElementRecord) - sizeof (unsigned char))

typedef struct CatalogBrowserRecord {
	WindowRecord		window;					/* WINDOW -- must be first		*/
	AuthIdentity		userIdentity;
	SDPPanelHandle		panelHandle;			/* Browser panel				*/
	SDPPanelHandle		pdPanelHandle;			/* Personal directory			*/
	ListHandle			attributeTypeList;		/* Record's attribute types		*/
	ListHandle			attributeValueList;		/* Attribute's values			*/
	ListHandle			dataDisplayList;		/* Data for this value			*/
	ListHandle			directoryDisplayList;	/* Twist-down directory display	*/
	ControlHandle		iconPopupControl;		/* Popup menu: which icon		*/
	ControlHandle		browsePopupControl;		/* Popup menu: what to browse	*/
	ControlHandle		showEntityRadioControl;	/* Showing catalog data			*/
	ControlHandle		showDSSpecRadioControl;	/* Showing raw DSSpec data		*/
	short				vampSICNIndex;			/* Current animation SICN		*/
	unsigned long		nextAnimation;			/* When to update animation		*/
	WindowFocus			windowFocus;			/* Active window list			*/
	Point				penPt;					/* Local coordinates			*/
	/*
	 * Font information
	 */
	FontData			labelFont;				/* Geneva   9 (small text)		*/
	FontData			typeFont;				/* Times   10 (RStrings)		*/
	FontData			valueFont;				/* Geneva  10 (labels)			*/
	FontData			dataFont;				/* Courier 10 (Hex data)		*/
	/*
	 * Various display stuff
	 */
	ListHandle			errorLog;
	BrowserChoice		browserChoice;			/* SDP Panel or Setup stuff		*/
	Boolean				showCatalog;			/* Data display mode flag		*/
	Boolean				oldShowCatalog;			/* Saved value of showCatalog	*/
	RStringHandle		recordName;				/* Current record name			*/
	RStringHandle		recordType;				/* Current record type			*/
	TwistDownHdl		directoryListHead;		/* For directoryDisplayList		*/
	/*
	 * These contain the current SDP Panel selection data.
	 */
	unsigned short		selectedDSSpecSize;		/* SDPGetPanelSelectionSize()	*/
	PackedDSSpecPtr		selectedDSSpec;			/* SDPGetPanelSelection()		*/
	DSSpec				theDSSpec;				/* OCEUnpackDSSpec()			*/
	RecordInfo			panelRecordInfo;
	RLI					theDSSpecRLI;			/* Unpacked RLI					*/
	OSType				specType;				/* OCEGetDSSpecInfo()			*/
	/*
	 * These Booleans are set by UnpackSDPPanelSelection()
	 */
	Boolean				isValidDSSpec;			/* OCEValidPackedDSSpec();		*/
	Boolean				isNonNullRLI;			/* NULL RLI is possible			*/
	Boolean				isValidPackedRLI;		/* OCEValidPackedRLI()			*/
	Boolean				isValidPackedPathName;	/* OCEValidPackedPathName()		*/
	Boolean				isValidNodeNameCount;	/* nodeNameCount < 64			*/
	Boolean				isValidUnpackedCount;
	unsigned short		nodeNameCount;			/* OCEDNodeNameCount()			*/
	unsigned short		unpackedNodeNameCount;	/* OCEUnpackPathName()			*/
	RStringPtr			*partsPtr;				/* NewPtr(nodeNameCount)		*/
	/*
	 * These "globals" are for the Enumeration of Attribute Types and values.
	 */
	RecordInfoPtr		currentRecordInfoPtr;	/* INFO							*/
	AttributeType		currentAttrType;		/* For value enumerator			*/
	AttributeHeaderInfo	currentAttrValue;		/* For value enumerator			*/
	short				currentAttributeTypeIndex;	/* Attribute type browser	*/
	short				currentAttributeValueIndex;	/* Attribute value browser	*/
	/*
	 * Stuff needed to browse the Setup Catalog. Nothing valid if
	 * DBR.browserChoice is not kBrowseSetupCatalog.
	 */
	ListHandle			setupCatalogList;
	short				currentSetupCatalogIndex;
	DirEnumSpec			currentEnumSpec;		/* Return from DirEnumParse		*/
	Handle				dirEnumSpecHdl;			/* As stored in the list		*/
	unsigned short		enumSpecLockState;
	RecordInfo			setupCatalogInfo;
	/*
	 * Stuff needed to browse the Setup Record.
	 */
	RecordInfo			setupRecordInfo;
	/*
	 * This needed for all setup directory/record browsing
	 */
	unsigned short		setupCatalogRefNum;	/* DSRefNum of setup directory	*/
	/*
	 * These are needed to fish around in the directory manager.
	 */
	DirParamBlock		dirParamBlock;			/* PB -- parameter block		*/
	short				currentIcon;			/* Selected icon				*/
	Handle				SDPIconSuite;			/* SDPGetDSSpecIcon result		*/
	Handle				DirIconSuite;			/* DirGetDirectoryIcon suite	*/
	Ptr					buffer;					/* Dynamically allocated		*/
	Rect				itemRect[nItemRect];	/* The visual elements			*/
} CatalogBrowserRecord, *CatalogBrowserPtr	;
/*
 * The current directory browser record is passed as a
 * parameter named dbp
 */
#define DBR		(*dbp)							/* CatalogBrowserRecord			*/
#define WINDOW	(&DBR.window.port)				/* WindowPtr, GrafPtr			*/
#define PB		(DBR.dirParamBlock)				/* DirParamBlock				*/
#define INFO	(*DBR.currentRecordInfoPtr)		/* Current record to browse		*/
#define RECT(i)	(DBR.itemRect[i])				/* One visual item				*/
#define SDP_INFO_PDRefNum	(DBR.panelRecordInfo.dsRefNum)
#define SDP_INFO_RecordID	(DBR.panelRecordInfo.recordID)
#define SETUP_REC_PDRefNum	(DBR.setupRecordInfo.dsRefNum)
#define SETUP_REC_RecordID	(DBR.setupRecordInfo.recordID)
#define SETUP_DIR_PDRefNum	(DBR.setupCatalogInfo.dsRefNum)
#define SETUP_DIR_RecordID	(DBR.setupCatalogInfo.recordID)
#define LOG(status, text)	(LogStatus(DBR.errorLog, (status), (text)))
#define NOTE(text)			(DisplayLogString(DBR.errorLog, (text)))

/*
 * Note: these must be macros. The arguments must not have side-effects.
 */
#define ForgetPtr(/* Ptr */ memPtr) do {	\
		Ptr		__memPtr = (Ptr) memPtr;	\
		if (__memPtr != NULL) {				\
			memPtr = NULL;					\
			DisposPtr(__memPtr);			\
			FailMemError();					\
		}									\
	} while (0)
#define ForgetHandle(/* Handle */ hdl) do {	\
		Handle		__hdl = (Handle) hdl;	\
		if (__hdl != NULL) {				\
			__hdl = NULL;					\
			DisposHandle(__hdl);			\
			FailMemError();					\
		}									\
	} while (0)


/*
 * CatalogBrowser functions.
 */
void								MakeCatalogBrowser(void);
void								DisposeCatalogBrowser(
		register CatalogBrowserPtr		dbp
	);
void								DoContentClick(
		register CatalogBrowserPtr		dbp
	);
void								DoWindowKeyDown(
		register CatalogBrowserPtr		dbp
	);
Boolean								IsOurWindow(
		WindowPtr						theWindow
	);
void								ActivateCatalogBrowser(
		register CatalogBrowserPtr		dbp,
		Boolean							activating
	);
void								UpdateBrowserWindow(
		register CatalogBrowserPtr		dbp
	);
void								UpdateRecordNameAndType(
		register CatalogBrowserPtr		dbp
	);
void								RefreshCatalogIcons(
		register CatalogBrowserPtr		dbp
	);
void								SelectNextWindowObject(
		register CatalogBrowserPtr		dbp
	);
void								InitializeBalloonHelp(void);
void								BalloonHelp(
		register CatalogBrowserPtr		dbp
	);
short								GetHelpItemIndex(
		register CatalogBrowserPtr		dbp,
		short							helpItem
	);
void								DecorateDisplay(
		register CatalogBrowserPtr		dbp,
		Boolean							redrawDisplay
	);
void								DoZoomWindow(
		WindowPtr						theWindow,
		short							whichPart
	);
Boolean								DoGrowWindow(
		WindowPtr						theWindow,
		short							minimumWidth,
		short							minimumHeight
	);
void								MyDrawGrowIcon(
		WindowPtr						theWindow
	);
/*
 * SDP Catalog Panel management
 */
SDPPanelHandle						CreateSDPDirectoryPanel(
		register CatalogBrowserPtr		dbp,
		const Rect						*viewRect,
		const PackedRLIPtr				initialRLI
	);
void								DisposeSDPDirectoryPanel(
		register CatalogBrowserPtr		dbp,
		SDPPanelHandle					*thePanel
	);
void								SaveCatalogBrowserInfo(
		register CatalogBrowserPtr		dbp
	);
void								DoSDPPanelEvent(
		register CatalogBrowserPtr		dbp
	);
void								DoChangedSDPSelection(
		register CatalogBrowserPtr		dbp,
		SDPPanelHandle					thePanel
	);
void								GetSDPPanelSelection(
		register CatalogBrowserPtr		dbp
	);
void								UnpackSDPPanelSelection(
		register CatalogBrowserPtr		dbp,
		SDPPanelHandle					thePanel
	);
void								GetSDPIcon(
		register CatalogBrowserPtr		dbp
	);
void								DisposeSDPSelection(
		register CatalogBrowserPtr		dbp
	);
void								ChangeRecordSelection(
		register CatalogBrowserPtr		dbp
	);
void								DisposeIconSuiteHandle(
		register CatalogBrowserPtr		dbp,
		Handle							*iconSuiteHdlPtr
	);
/*
 * ListManagement.c
 */
ListHandle							CreateList(
		register CatalogBrowserPtr		dbp,
		const Rect						*viewRect,
		short							listFontNumber,
		short							listFontSize,
		ProcPtr							listDefProc
	);
void								UpdateList(
		ListHandle						listHandle,
		Boolean							hasFocus
	);
void								DrawFocusRect(
		ListHandle						listHandle,
		short							penMode
	);
void								DisposeList(
		ListHandle						*listHandlePtr
	);
void								HideList(
		ListHandle						listHandle
	);
void								ShowList(
		ListHandle						listHandle
	);
/*
 * Personal Address Book management
 */
void								OpenPersonalAddressBook(
		register CatalogBrowserPtr		dbp
	);
void								ClosePersonalAddressBook(
		register CatalogBrowserPtr		dbp
	);
/*
 * Setup Record Browser
 */
void								OpenSetupRecord(
		register CatalogBrowserPtr		dbp
	);
void								CloseSetupRecord(
		register CatalogBrowserPtr		dbp
	);
/*
 * Setup Catalog info
 */
void								GetSetupCatalogInfo(
		register CatalogBrowserPtr		dbp
	);
/*
 * Setup Catalog Browser
 */
void								OpenSetupCatalog(
		register CatalogBrowserPtr		dbp
	);
void								CloseSetupCatalog(
		register CatalogBrowserPtr		dbp
	);
void								DoClickInSetupCatalogList(
		register CatalogBrowserPtr		dbp
	);
void								ClearSetupCatalogList(
		register CatalogBrowserPtr		dbp
	);
void								AddSetupCatalogEntry(
		register CatalogBrowserPtr		dbp,
		Handle							dirEnumSpecHdl
	);
void								SelectNthSetupCatalog(
		register CatalogBrowserPtr		dbp,
		short							index
	);


/*
 * Setup Catalog Enumerator and friends.
 */
void								EnumerateSetupCatalog(
		register CatalogBrowserPtr		dbp
	);

/*
 * Clear out everything
 */
void								ClearRecordDisplay(
		register CatalogBrowserPtr		dbp
	);
void								ClearRecordNameAndType(
		register CatalogBrowserPtr		dbp
	);

/*
 * Attribute Type, Value, and Data management
 */
void								ClearAttributeTypeList(
		register CatalogBrowserPtr		dbp
	);
void								ClearAttributeValueList(
		register CatalogBrowserPtr		dbp
	);
void								ClearAttributeData(
		register CatalogBrowserPtr		dbp
	);
void								DoClickInAttributeTypeList(
		register CatalogBrowserPtr		dbp
	);
void								DoClickInAttributeValueList(
		register CatalogBrowserPtr		dbp
	);
void								DoClickInDataDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								AddAttributeType(
		register CatalogBrowserPtr		dbp
	);
void								SelectNthAttributeType(
		register CatalogBrowserPtr		dbp,
		short							index
	);
void								AddAttributeValue(
		register CatalogBrowserPtr		dbp,
		const Attribute					*theAttribute
	);
void								SelectNthAttributeValue(
		register CatalogBrowserPtr		dbp,
		short							index
	);
void								AddAttributeData(
		register CatalogBrowserPtr		dbp
	);
/*
 * LDEF's
 */
#define DefineLDEF(what)				\
	pascal void			what(			\
		short		listMessage,		\
		Boolean		listSelect,			\
		Rect		*listRect,			\
		Cell		listCell,			\
		short		listDataOffset,		\
		short		listDataLen,		\
		ListHandle	listHandle			\
	)
DefineLDEF(RStringLDEF);
DefineLDEF(AttributeValueLDEF);
DefineLDEF(DataDisplayLDEF);
DefineLDEF(CatalogDisplayLDEF);
DefineLDEF(SetupCatalogLDEF);
#undef DefineLDEF


/*
 * Enumerators and List Management
 */
void								EnumerateAttributeTypes(
		register CatalogBrowserPtr		dbp,
		RecordInfoPtr					recordInfoPtr
	);
void								EnumerateAttributeValues(
		register CatalogBrowserPtr		dbp
	);
void								CreateDataDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								UpdateDataDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								DoClickInDataDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								ClearAttributeDataDisplay(
		register CatalogBrowserPtr		dbp
	);
void								DisplayCurrentAttributeData(
		register CatalogBrowserPtr		dbp
	);
void								DrawDataRecordLength(
		register CatalogBrowserPtr		dbp
	);
void								MakeCatalogDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								DeleteCatalogDisplayList(
		register CatalogBrowserPtr		dbp
	);
void								ClearCatalogDataDisplay(
		register CatalogBrowserPtr		dbp
	);
pascal void							DrawCatalogListCell(
		ListHandle						twistDownListHdl,
		const Ptr						dataPtr,
		unsigned short					dataLength,
		const Rect						*viewRect
	);
TwistDownHdl						DisplayDataBlock(
		register CatalogBrowserPtr		dbp,
		TwistDownHdl					previousElement,	/* Linkage				*/
		short							indentLevel,
		Ptr								dataPtr,
		unsigned short					dataLength
	);
TwistDownHdl						StoreElementRString(
		TwistDownHdl					previousElement,
		short							indentLevel,
		const RStringPtr				srcRStringPtr
	);
TwistDownHdl						StoreElementMessage(
		TwistDownHdl					previousElement,
		short							indentLevel,
		ConstStr255Param				work
	);
TwistDownHdl						StoreElementString(
		TwistDownHdl					previousElement,
		short							indentLevel,
		ConstStr255Param				work
	);
TwistDownHdl						StoreElementFormattedString(
		TwistDownHdl					previousElement,
		short							indentLevel,
		ElementFormat					elementFormat,
		ConstStr255Param				work
	);
TwistDownHdl						StoreElementDatum(
		TwistDownHdl					previousElement,
		short							indentLevel,
		ElementFormat					elementFormat,
		unsigned short					dataLength,
		const Ptr						dataPtr
	);
TwistDownHdl						DisplayDSSpecType(
		register CatalogBrowserPtr		dbp,
		TwistDownHdl					previousElement,
		short							indentLevel
	);
TwistDownHdl						DisplayRecordID(
		register CatalogBrowserPtr		dbp,
		TwistDownHdl					previousElement,
		short							indentLevel
	);
TwistDownHdl						DisplayExtension(
		register CatalogBrowserPtr		dbp,
		TwistDownHdl					previousElement,
		short							indentLevel
	);
TwistDownHdl						DisplayRawDSSpecPtr(
		register CatalogBrowserPtr		dbp,
		TwistDownHdl					previousElement,
		short							indentLevel
	);

/*
 * Busy-feedback management
 */
void								InitializeVampDisplay(
		register CatalogBrowserPtr		dbp
	);
void								UpdateVampDisplay(
		register CatalogBrowserPtr		dbp
	);
pascal void							AnimateVampDisplay(
		SDPPanelHandle					panelHandle,
		Boolean							busy
	);

/*
 * Icon management
 */
void								InitializeIconSelector(
		register CatalogBrowserPtr		dbp
	);
void								RefreshGetCatalogIcons(
		register CatalogBrowserPtr		dbp
	);
void								GetSelectedCatalogIcon(
		register CatalogBrowserPtr		dbp
	);
void								DrawCurrentCatalogIcon(
		register CatalogBrowserPtr		dbp
	);

/*
 * User identity and AOCE utilities.
 */
void								SetUserIdentity(
		register CatalogBrowserPtr		dbp
	);
void								SetSpecificIdentity(
		register CatalogBrowserPtr		dbp
	);
OSErr								SystemSupportsAOCE(void);

void								DoHelp(
		short					dialogID,	/* -> DLOG/DITL				*/
		short					topicID		/* -> STR# with topics		*/
	);

/*
 * Utility functions
 */
void								GetFontData(
		register FontDataPtr			fontDataPtr,
		short							fontNameIndex,	/* In STRN_Parameters	*/
		short							fontSize
	);
void								ClearMemory(
		register void					*recordPtr,
		register Size					recordSize
	);
#define CLEAR(what)	ClearMemory(&what, sizeof what)
void								ErrorAlert(
		short							alrtResId,
		OSErr							errorValue,
		ConstStr255Param				errorText
	);
/*
 * Manage HandleDirEnumSpec
 *	MakeHandleFromDirEnumSpec takes a DirEnumSpec and packages
 *		into a private handle-based structure. It returns
 *		NULL on memory errors.
 *	GetDirEnumSpecFromHandle takes a handle as returned
 *		from MakeHandleFromDirEnumSpec and stores the necessary
 *		data into the argument DirEnumSpec. It ensures that
 *		the handle is locked (the argument DirEnumSpec contains
 *		pointers to the handle contents) and returns a value
 *		that may be passed to HSetState to restore the
 *		handle locked status:
 *			oldState = GetDirEnumSpecFromHandle(myHandle, &myEnumSpec);
 *			DrawRString(...);
 *			HSetState(myHandle, oldState);
 */
Handle								MakeHandleFromDirEnumSpec(
		const DirEnumSpec				*dirEnumSpecPtr
	);
unsigned short						GetDirEnumSpecFromHandle(
		Handle							handleDirEnumSpec,
		DirEnumSpec						*dirEnumSpecPtr
	);

/*
 * RString utilities
 */
/*
 * TRUE if this is a non-null, non zero-length RString
 */
#define IsRStringHandle(rString) \
	((rString) != NULL && GetHandleSize((Handle) (rString)) !=  0)
/*
 * Return the number of bytes needed to store the RString
 */
#define RStringLength(ptr)	(sizeof (ProtoRString) + (ptr)->dataLength)

/*
 * Store the RString in a handle, return the handle.
 * Failure on memory errors. Invalid RStrings return
 * zero-length handles.
 */
RStringHandle						MakeRStringHandle(
		const RStringPtr				rStringPtr
	);
/*
 * Store the RString in the handle, adjusting its
 * size as needed. Failure on memory errors. Invalid
 * RStrings return zero-lenght handles.
 */
void								SetRStringHandle(
		RStringHandle					rStringHandle,
		const RStringPtr				rStringPtr
	);
void								DrawRStringInFrame(
		const RStringHandle				rStringHandle,
		const Rect						*viewRect
	);
void								DrawRStringHandle(
		const RStringHandle				rStringHandle,
		const Rect						*boundsRect,
		short							textFace
	);
void								DrawRString(
		const RStringPtr				rStringPtr,
		const Rect						*boundsRect,
		short							textFace
	);
unsigned short						RStringWidth(
		const RStringPtr				rStringPtr
	);
void								DrawWideString(
		const StringPtr					stringPtr,
		const Rect						*boundsRect,
		short							textFace
	);
void								DrawWideText(
		Ptr								textPtr,
		unsigned short					offset,
		unsigned short					textLength,
		const Rect						*boundsRect,
		short							textFace
	);

/*
 * Cheap 'n dirty pascal string copy routine.
 */
#define pstrcpy(dst, src) do {							\
		StringPtr	_src = (src);						\
		BlockMove(_src, dst, _src[0] + 1);				\
	} while (0)
/*
 * Cheap 'n dirty pascal string concat.
 */
#define pstrcat(dst, src) do {							\
		StringPtr		_dst = (dst);					\
		StringPtr		_src = (src);					\
		short			_len;							\
		_len = 255 - _dst[0];							\
		if (_len > _src[0]) _len = _src[0];				\
		BlockMove(&_src[1], &_dst[1] + _dst[0], _len);	\
		_dst[0] += _len;								\
	} while (0)
#define width(r)	((r).right - (r).left)
#define height(r)	((r).bottom - (r).top)

/*
 * Dialog Management
 */
void						PositionDialog(
		ResType					dialogType,
		short					dialogID
	);
void						PositionRect(
		Rect					*boundsPtr
	);

/*
 * Numeric formatting functions in StringFormat.c
 */
#define AppendChar(result, c) (result[++result[0]] = (c))
void						AppendUnsigned(
		StringPtr				result,
		unsigned long			value
	);
void						AppendSigned(
		StringPtr				result,
		signed long				value
	);
void						AppendUnsignedLeadingZeros(
		StringPtr				result,
		unsigned long			value,
		short					fieldWidth,
		short					terminatorChar
	);
void						AppendHexLeadingZeros(
		StringPtr				result,
		unsigned long			value,
		short					fieldWidth
	);
void						AppendPascalString(
		StringPtr				result,
		const StringPtr			value
	);
void						AppendOSType(
		StringPtr				result,
		OSType					value
	);

/*
 * Global definitions
 */
EXTERN MenuHandle					gAppleMenu;
EXTERN MenuHandle					gFileMenu;
EXTERN MenuHandle					gEditMenu;
EXTERN MenuHandle					gHelpMenu;
EXTERN short						gHelpMenuItem;
EXTERN Boolean						gQuitNow;
EXTERN Boolean						gUpdateMenusNeeded;
EXTERN EventRecord					gEventRecord;
#define EVENT						(gEventRecord)
EXTERN short						gOpenWindowCount;
extern const Rect					gItemShapeRect[];
EXTERN const AddrBlock				gNullAddrBlock;		/* All zero server hint	*/
EXTERN const DSSpec					gNullDSSpec;
EXTERN const RecordID				gNullRecordID;
EXTERN const CatalogInfoRecord		gNullCatalogInfoRecord;
EXTERN const RLI					gNullRLI;
EXTERN const CreationID				gNullCID;
EXTERN const DirEnumSpec			gNullDirEnumSpec;
EXTERN CatalogBrowserPtr			gCurrentCatalogBrowserPtr;	/* DEBUG	*/
#define gDBR						(*gCurrentCatalogBrowserPtr)
/*
 * Gestalt responses
 */
EXTERN Boolean						gSystemHasBalloonHelp;
EXTERN Boolean						gHasColorQuickDraw;
EXTERN Boolean						gHasPopupCDEF;


#endif /* Not REZ */
/*
 * The following information is common to the application
 * and the Rez source.
 */
#endif	/* _DTSCatalogPeek_	*/
