/*								DrawCatalogIcons.c								*/
/*
 * DTS Catalog Peek
 * DrawCatalogIcons.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * The user has selected a new directory specification.
 * Grab the icon and enumerate its attribute types.
 * This should be extended to grab other pertinent information,
 * such as the fully-resolved directory path name.
 *
 * Globals
 *	DBR.currentIcon
 *			The index (from the popup) of the current icon type.
 *	DBR.DirIconSuite
 *			The current icon suite handle.
 *	INFO.recordID.rli
 *			The record whose icon we want to display
 */
#include "DTSCatalogPeek.h"
#include <Controls.h>
#pragma segment MainCode

#ifdef kSDPInstallPanelBusyProc	/* Temp	for B1/B3 compatibility	*/
#define B3_HEADERS	1
#else
#define B3_HEADERS	0
#endif

/*
 * This table must track the MENU_Icon resource.
 *	iconType	The type of this icon (what the user sees)
 *	iconSize	The size of the icon handle (by ResEdit inspection)
 *	offset		OffsetRect by this amount to draw icon
 *	size		Icon height/width
 *	bitDepth	The depth of the screen (1 == black/white, zero
 *				means don't choose this one).
 *	maskType	The icon suite must have both an icon and a mask.
 *				If the icon type is not 'ICN#' or 'ics#', the
 *				mask icon type is specified here.
 */
typedef struct {
	ResType				iconType;	/* 'icl8', etc.				*/
	short				iconSize;	/* Handle size (temp)		*/
	short				offset;		/* Offset top/left in Rect	*/
	short				size;		/* Rect height/width		*/
	short				bitDepth;	/* Screen color pixel depth	*/
	ResType				maskType;	/* 'ICN#' or 'ics#' or zero	*/
} IconSizeTable, *IconSizeTablePtr;

#define kLargeMaskSize	(256)
#define kSmallMaskSize	(64)

IconSizeTable			gIconSizeTable[] = {
/*		 Icon	    Size Offset  Rect Depth   Mask				*/
	{	'icl8',		1024,	  0,   32,    8, 'ICN#'	},
	{	'icl4',		 512,	  0,   32,    4, 'ICN#'	},
	{	'ICN#',		 256,	  0,   32,    1,     0	},
	{	'ics8',		 256,	  8,   16,    0, 'ics#'	},
	{	'ics4',		 128,	  8,   16,    0, 'ics#'	},
	{	'ics#',		  64,	  8,   16,    0,     0	}
};
#define nIcons	((sizeof gIconSizeTable) / sizeof gIconSizeTable[0])
void								GetCurrentCatalogIcon(
		register CatalogBrowserPtr		dbp
	);
void								SetInitialCatalogIconPopupValue(
		register CatalogBrowserPtr		dbp
	);
Boolean								AddCatalogIcon(
		register CatalogBrowserPtr		dbp,
		ResType							iconType,
		short							iconSize
	);

/*
 * InitializeIconSelector
 * Create a popup menu using the system 7 popup control definition.
 */
void
InitializeIconSelector(
		register CatalogBrowserPtr		dbp
	)
{
		SetPort(WINDOW);
		TextFont(DBR.labelFont.fontNumber);
		TextSize(DBR.labelFont.fontSize);
		DBR.iconPopupControl = NewControl(
				WINDOW,					/* Display window		*/
				&RECT(kInGetCatalogIconPopup),
				"\p", 					/* No Title				*/
				TRUE,					/* Visible				*/
				popupTitleLeftJust,		/* value: title adjust	*/
				MENU_Icons,				/* min:   menu resource	*/
				0,						/* max:   popup width	*/
				popupMenuProc			/* Popup control proc	*/
				 | popupUseWFont,		/*  Draw in window font	*/
				0						/* RefCon (unused)		*/
			);
		FailNIL(DBR.iconPopupControl);	/* Oops?				*/
		SetInitialCatalogIconPopupValue(dbp);
		DBR.currentIcon = GetCtlValue(DBR.iconPopupControl);
}

void
SetInitialCatalogIconPopupValue(
		register CatalogBrowserPtr		dbp
	)
{
		short					screenDepth;
		short					i;
		register CGrafPtr		cGrafPtr;
		PixMapHandle			pixMapHandle;
		
		/*
		 * Set screenDepth to 1, 4, or 8 depending on the display
		 * screen depth.
		 */
		cGrafPtr = (CGrafPtr) WINDOW;	/* For convenience			*/
		if ((*cGrafPtr).portVersion != 0xC000)
			screenDepth = 1;
		else {
			pixMapHandle = (*cGrafPtr).portPixMap;
			screenDepth = (**pixMapHandle).pixelSize;
		}
		/*
		 * Normalize (crude, but it works), then look for this
		 * depth icon in the icon table. The menu item number
		 * is the icon index + 1.
		 */
		if (screenDepth >= 8)
			screenDepth = 8;
		else if (screenDepth >= 4)
			screenDepth = 4;
		else {
			screenDepth = 1;
		}
		for (i = 0; i < nIcons; i++) {
			if (gIconSizeTable[i].bitDepth == screenDepth) {
				SetCtlValue(DBR.iconPopupControl, i + 1);
				break;
			}
		}
}

/*
 * GetSelectedCatalogIcon
 * This is called to fetch the icon for display.
 */
void
GetSelectedCatalogIcon(
		register CatalogBrowserPtr		dbp
	)
{
		short				whichIcon;

		whichIcon = GetCtlValue(DBR.iconPopupControl);
		DBR.currentIcon = whichIcon;
		RefreshGetCatalogIcons(dbp);
}

/*
 * RefreshGetCatalogIcons
 * Redraw the current directory icon.
 */
void
RefreshGetCatalogIcons(
		register CatalogBrowserPtr		dbp
	)
{
		GetCurrentCatalogIcon(dbp);
		DrawCurrentCatalogIcon(dbp);
		ValidRect(&RECT(kInGetCatalogIconItself));
}

/*
 * DisposeIconSuiteHandle
 * This is called for any icon suite to dispose the suite
 * and its contents.
 */
void
DisposeIconSuiteHandle(
		register CatalogBrowserPtr		dbp,
		Handle							*iconSuiteHdlPtr
	)
{
		Handle				iconSuiteHdl;
		OSErr				status;
		
		iconSuiteHdl = *iconSuiteHdlPtr;
		if (iconSuiteHdl != NULL) {
			*iconSuiteHdlPtr = NULL;
			status = DisposeIconSuite(iconSuiteHdl, TRUE);
			LOG(status, "\pDisposeIconSuite");
		}
}

/*
 * GetCurrentCatalogIcon
 * Get the current directory icon information. This is made
 * slightly more complicated by the need to fetch the mask
 * for color icons.
 */
void
GetCurrentCatalogIcon(
		register CatalogBrowserPtr		dbp
	)
{
		register IconSizeTablePtr	iconSizeTablePtr;
		Boolean						hasIcon;
#define ICON	(*iconSizeTablePtr)

		DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
		if (DBR.currentIcon != 0
		 && DBR.currentRecordInfoPtr != NULL
		 && OCEValidPackedRLI(INFO.recordID.rli)) {
			FailOSErr(NewIconSuite(&DBR.DirIconSuite));
		 	iconSizeTablePtr = &gIconSizeTable[DBR.currentIcon - 1];
			TRY {
			 	hasIcon = AddCatalogIcon(
			 		dbp,
			 		ICON.iconType,
			 		ICON.iconSize
			 	);
			 	if (hasIcon && ICON.maskType != 0) {
			 		hasIcon = AddCatalogIcon(
			 			dbp,
			 			ICON.maskType,
			 			(ICON.maskType == 'ICN#')
			 				? kLargeMaskSize : kSmallMaskSize
			 		);
			 	}
			 	if (hasIcon == FALSE)
			 		DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
			}
			CATCH {
				DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
			}
			ENDTRY;
		}
#undef ICON
}

/*
 * DrawCurrentCatalogIcon
 * Locate and plot the current icon.
 */
void
DrawCurrentCatalogIcon(
		register CatalogBrowserPtr		dbp
	)
{
		OSErr						status;
		register IconSizeTablePtr	iconSizeTablePtr;
		Rect						viewRect;
#define ICON	(*iconSizeTablePtr)
		
		EraseRect(&RECT(kInGetCatalogIconItself));
		if (DBR.DirIconSuite == NULL || DBR.currentIcon == 0)
			FrameRect(&RECT(kInGetCatalogIconItself));
		else {
		 	iconSizeTablePtr = &gIconSizeTable[DBR.currentIcon - 1];
		 	viewRect = RECT(kInGetCatalogIconItself);
		 	OffsetRect(&viewRect, ICON.offset, ICON.offset);
		 	viewRect.right = viewRect.left + ICON.size;
		 	viewRect.bottom = viewRect.top + ICON.size;
			status = PlotIconSuite(
						&viewRect,
						atAbsoluteCenter,		/* Alignment		*/
						ttNone,					/* No transform		*/
						DBR.DirIconSuite
					);
			if (status != noErr)
				DisposeIconSuiteHandle(dbp, &DBR.DirIconSuite);
			LOG(status, "\pPlotIconSuite");
		}
}

/*
 * AddCatalogIcon
 * Fetch the icon from the directory. Return TRUE on success,
 * FALSE if the icon type was not found, and Fail on memory errors.
 * We "hack" the directory icon size if it was too small.
 */
Boolean
AddCatalogIcon(
		register CatalogBrowserPtr		dbp,
		ResType							iconType,
		short							iconSize	/* Presumed size	*/
	)
{
		OSErr				status;
		Handle				iconBufferHdl;
		Boolean				result;
#define GET		(PB.getDirectoryIconPB)

	 	iconBufferHdl = NewHandle(iconSize);
		FailNIL(iconBufferHdl);
		TRY {
		 	do {
				CLEAR(PB);
				GET.ioCompletion = NULL;
				GET.serverHint = gNullAddrBlock;	/* Clear addr block	*/
				GET.dsRefNum = INFO.dsRefNum;
				GET.identity = DBR.userIdentity;
				GET.pRLI = INFO.recordID.rli;
				GET.iconType = iconType;
				HLock(iconBufferHdl);				/* Not really needed	*/
				GET.iconBuffer = *iconBufferHdl;
#if B3_HEADERS
				GET.bufferSize = GetHandleSize(iconBufferHdl);
#endif
				status = DirGetDirectoryIcon(&PB, SYNC);
				HUnlock(iconBufferHdl);
#if B3_HEADERS
				if (status == kOCEBufferTooSmall) {
					{
						Str255				work;
						NumToString(GET.bufferSize, work);
						pstrcat(work, "\p: DirGetDirectoryIcon new size");
						NOTE(work);
					}	
					SetHandleSize(iconBufferHdl, GET.bufferSize);
					FailMemError();
				}
			} while (status == kOCEBufferTooSmall);
#else
			} while (0);
#endif
			FailOSErrNeverDebug(status);
			/*
			 * Build an icon suite that contains this icon.
			 * An error from the above indicates "no icon for this
			 * selector type (or something more serious)."
			 * The handle will be deleted by DisposeIconSuite.
			 */
			FailOSErr(
				AddIconToSuite(
						iconBufferHdl,
						DBR.DirIconSuite,
						iconType
					)
			);
			result = TRUE;
		}
		CATCH {
			LOG(STATUS, "\pDrawCatalogIcons");
			DisposeHandle(iconBufferHdl);
			result = FALSE;
			/*
			 * NO_PROPAGATE is probably wrong on memory
			 * and other non-AOCE errors.
			 */
			NO_PROPAGATE;
		}
		ENDTRY;
		return (result);
#undef GET
}
