/*							EnumerateAttributeTypes.c							*/
/*
 * DTS Catalog Peek
 * EnumerateAttributeTypes.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * Enumerate all attribute types for the currently-selected
 * directory. Attribute types are added to the type list
 * as they are found.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

static pascal Boolean				ForEachAttributeType(
		long							clientData,
		const AttributeType				*aType
	);


/*
 * EnumerateAttributeTypes
 * This is called when a new directory is selected.
 */
void
EnumerateAttributeTypes(
		register CatalogBrowserPtr		dbp,
		RecordInfoPtr					recordInfoPtr
	)
{
		OSErr					status;
		Boolean					first;
#define GET	(PB.enumerateAttributeTypesGetPB)
#define PARSE	(PB.enumerateAttributeTypesParsePB)

		/*
		 * Make sure we're starting from a clean slate.
		 * This is mostly redundant, but harmless.
		 */
		ClearRecordDisplay(dbp);
		/*
		 * Set our "globals"
		 */
		DBR.currentRecordInfoPtr = recordInfoPtr;
		if (&INFO.recordID == NULL) {
			NOTE("\pEnum: NULL RecordID");
			Failure(abortErr);
		}
		if (INFO.recordID.rli != NULL
		 && OCEValidPackedRLI(INFO.recordID.rli) == FALSE) {
		 	NOTE("\pEnum: Invliad Packed RLI");
		 	Failure(abortErr);
		}
		/*
		 * Store the record name and type.
		 */
		SetRStringHandle(
			DBR.recordName,
			INFO.recordID.local.recordName
		);
		SetRStringHandle(
			DBR.recordType,
			INFO.recordID.local.recordType
		);
		UpdateRecordNameAndType(dbp);
		/*
		 * Enumerate the record.
		 */
		CLEAR(PB);
		first = TRUE;
		do {
			GET.identity = DBR.userIdentity;
			GET.ioCompletion = NULL;
			GET.serverHint = gNullAddrBlock;	/* Clear addr block		*/
			GET.dsRefNum = INFO.dsRefNum;
			GET.clientData = (long) dbp;
			GET.aRecord = &INFO.recordID;
			if (first) {
				GET.startingAttrType = NULL;
				first = FALSE;
			}
			else {
				/*
				 * This is the last attribute type that was fetched
				 * by the parser callback.
				 */
				GET.startingAttrType = &DBR.currentAttrType;
			}
			GET.includeStartingPoint = FALSE;
			GET.getBuffer = DBR.buffer;
			GET.getBufferSize = kMyBufferSize;
			status = DirEnumerateAttributeTypesGet(&PB, SYNC);
			if (status != kOCEMoreData)
				LOG(status, "\pDirEnumerateAttributeTypesGet");
			if (status == noErr || status == kOCEMoreData) {
				/*
				 * We have a record, or we have a record and more
				 * data to read. Parse the data: this will call
				 * our callback function.
				 */
				PARSE.eachAttrType = ForEachAttributeType;
				status = DirEnumerateAttributeTypesParse(&PB, SYNC);
				if (status != kOCEMoreData)
					LOG(status, "\pDirEnumerateAttributeTypesParse");
			}
		} while (status == kOCEMoreData);
		SelectNthAttributeType(dbp, 0);		/* Display first attribute type		*/
#undef GET
#undef PARSE
}

/*
 * ForEachAttributeType
 * This is called by the DirEnumerateAttributeTypesParse function.
 * The attribute type is an RString: we put it into our list.
 */
static pascal Boolean
ForEachAttributeType(
		long							clientData,
		const AttributeType				*aType
	)
{
		register CatalogBrowserPtr		dbp;
		Boolean							stopParse;
		
		dbp = (CatalogBrowserPtr) clientData;
		stopParse = FALSE;
		TRY {
			/*
			 * Grab a copy of the attribute type
			 * for the "more data" get call.
			 */
			OCECopyRString(
				(const RStringPtr) aType,
				(RStringPtr) &DBR.currentAttrType,
				kAttributeTypeMaxBytes
			);
			AddAttributeType(dbp);
		}
		CATCH {
			/*
			 * Memory trouble
			 */
			ErrorAlert(ALRT_Continue, STATUS, "\pCan't add attribute type");
			NO_PROPAGATE;
			stopParse = TRUE;
		}
		ENDTRY;
		return (stopParse);
}
