/*							EnumerateAttributeValues.c							*/
/*
 * DTS Catalog Peek
 * EnumerateAttributeValues.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * Enumerate all attribute types for the currently-selected
 * directory. Attribute types are added to the type list
 * as they are found.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

static pascal Boolean				ForEachAttributeValue(
		long							clientData,
		const Attribute					*theAttribute
	);


/*
 * EnumerateAttributeValues
 * This is called when a new attribute type is selected.
 *
 * Globals
 *	INFO.dsRefNum
 *			Current personal directory RefNum
 *	INFO.recordID
 *			Current record to examine
 *	DBR.currentAttributeType
 *			Current attribute type to examine
 */
void
EnumerateAttributeValues(
		register CatalogBrowserPtr		dbp
	)
{
		OSErr					status;
		AttributeTypePtr		attrTypeList[1];
		RecordIDPtr				recordIDList[1];
#define GET		(PB.lookupGetPB)
#define PARSE	(PB.lookupParsePB)

		/*
		 * Make sure we're starting from a clean slate.
		 */
		ClearAttributeValueList(dbp);
		CLEAR(PB);
		recordIDList[0] = &INFO.recordID;
		attrTypeList[0] = &DBR.currentAttrType;
		GET.identity = DBR.userIdentity;
		GET.ioCompletion = NULL;
		GET.serverHint = gNullAddrBlock;	/* Clear addr block	*/
		GET.dsRefNum = INFO.dsRefNum;
		GET.clientData = (long) dbp;
		GET.aRecordList = recordIDList;
		GET.attrTypeList = attrTypeList;
		GET.recordIDCount = 1;
		GET.attrTypeCount = 1;
		GET.includeStartingPoint = FALSE;
		GET.getBuffer = DBR.buffer;
		GET.getBufferSize = kMyBufferSize;
		GET.startingRecordIndex = 1;
		GET.startingAttrTypeIndex = 1;
		CLEAR(GET.startingAttribute);
		do {
			status = DirLookupGet(&PB, SYNC);
			if (status == noErr || status == kOCEMoreData) {
				PARSE.eachRecordID = NULL;
				PARSE.eachAttrType = NULL;
				PARSE.eachAttrValue = ForEachAttributeValue;
				status = DirLookupParse(&PB, SYNC);
			}
		} while (status == kOCEMoreData);
		LOG(status, "\pDirEnumerateAttributeValuesGet");
		SelectNthAttributeValue(dbp, 0);
#undef GET
#undef PARSE
}

/*
 * ForEachAttributeValue
 * This is called by the DirLookupParse function.
 * The attribute value is an RString: we put it into our list.
 */
static pascal Boolean
ForEachAttributeValue(
		long							clientData,
		const Attribute					*theAttribute
	)
{
		register CatalogBrowserPtr		dbp;
		Boolean							stopParse;
		
		dbp = (CatalogBrowserPtr) clientData;
		stopParse = FALSE;
		TRY {
			AddAttributeValue(dbp, theAttribute);
		}
		CATCH {
			ErrorAlert(ALRT_Continue, STATUS, "\pCan't add attribute value");
			NO_PROPAGATE;
			stopParse = TRUE;
		}
		ENDTRY;
		return (stopParse);
}
