/*							EnumerateSetupCatalog.c								*/
/*
 * DTS Catalog Peek
 * EnumerateSetupCatalog.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * This module calls DirEnumerateGet and DirEnumerateParse
 * to return the contents of the Setup Catalog. Note
 * that we assume that everything will fit in memory,
 * which is -- in the real world of AOCE directories --
 * quite unrealistic.  I.e., this code may offer an
 * example of examining the contents of a directory,
 * but it lacks many of the "large directory" features
 * of the SDP Directory Panel.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

#define SETUP_LIST		(DBR.setupCatalogList)
#define SETUP_BOUNDS	((**SETUP_LIST).dataBounds)
static pascal Boolean				MyForEachEnumSpec(
		long							clientData,
		const DirEnumSpec				*enumSpec
	);

/*
 * EnumerateSetupCatalog
 * Look at all directories in the AOCE Setup Catalog
 */
void
EnumerateSetupCatalog(
		register CatalogBrowserPtr		dbp
	)
{
		OSErr					status;
		Boolean					first;
#define GET	(PB.enumerateGetPB)
#define PARSE	(PB.enumerateParsePB)

		/*
		 * Make sure we're starting from a clean slate.
		 */
		ClearSetupCatalogList(dbp);
		GetSetupCatalogInfo(dbp);
		CLEAR(PB);
		first = TRUE;
		do {
			GET.identity = DBR.userIdentity;
			GET.ioCompletion = NULL;
			GET.serverHint = gNullAddrBlock;	/* Clear addr block		*/
			GET.clientData = (long) dbp;
			GET.dsRefNum = DBR.setupCatalogRefNum;
			GET.aRLI = NULL;
			if (first) {
				GET.startingPoint = NULL;
				first = FALSE;
			}
			else {
				GET.startingPoint = &DBR.currentEnumSpec;
			}
			GET.includeStartingPoint = FALSE;
			GET.sortBy = kSortByName;			/* User choice?			*/
			GET.sortDirection = kSortForwards;
			GET.nameMatchString = NULL;			/* Anything				*/
			GET.matchNameHow = kMatchAll;		/* Worth				*/
			GET.typesList = NULL;				/* Doing				*/
			GET.typeCount = 0;					/* Is					*/
			GET.matchTypeHow = kMatchAll;		/* Worth				*/
			GET.enumFlags = kEnumAllMask;		/* Overdoing			*/
			GET.getBuffer = DBR.buffer;
			GET.getBufferSize = kMyBufferSize;
			status = DirEnumerateGet(&PB, SYNC);
			if (status != noErr && status != kOCEMoreData)
				LOG(status, "\pDirEnumerateGet");
			if (status == noErr || status == kOCEMoreData) {
				/*
				 * We have data to parse:
				 * this will call our callback function.
				 */
				PARSE.eachEnumSpec = MyForEachEnumSpec;
				status = DirEnumerateParse(&PB, SYNC);
				if (status != noErr && status != kOCEMoreData)
					LOG(status, "\pDirEnumerateDirectoriesParse");
			}
		} while (status == kOCEMoreData);
#undef GET
#undef PARSE
}

/*
 * MyForEachEnumSpec
 * This is called by the DirEnumerateDirectoriesParse function.
 */
static pascal Boolean
MyForEachEnumSpec(
		long							clientData,
		const DirEnumSpec				*enumSpec
	)
{
		register CatalogBrowserPtr		dbp;
		Boolean							stopParse;
		Handle							dirEnumSpecHdl;

		dbp = (CatalogBrowserPtr) clientData;
		stopParse = FALSE;
		TRY {
			/*
			 * Grab the spec. Note that this will hang
			 * around and be used as the starting point
			 * if we have to re-call DirEnumerateGet
			 */
			DBR.currentEnumSpec = *enumSpec;
			/*
			 * Now, make a copy so the data hangs around for a while.
			 * By the way, this might be a good place to ignore
			 * invisible items.
			 */
			dirEnumSpecHdl = MakeHandleFromDirEnumSpec(enumSpec);
			FailNIL(dirEnumSpecHdl);
			AddSetupCatalogEntry(dbp, dirEnumSpecHdl);
		}
		CATCH {
			/*
			 * Memory trouble
			 */
			ErrorAlert(ALRT_Continue, STATUS, "\pCan't add catalog information.");
			NO_PROPAGATE;
			stopParse = TRUE;
		}
		ENDTRY;
		return (stopParse);
}

