/*										Failure.h								*/
/*
 * Failure.h
 * Copyright  1992-93 Apple Computer Inc. All rights reserved.
 * See Failure.c for documentation.
 *
 * This is a slightly simplified implementation of the Think/MPW/whatever
 * "Failure" mechanism. To use, define error handlers using the following macros:
 *	TRY {						initialize a failure unit.
 *	} CATCH {					end of the "normal" sequence, begin error handler.
 *	} ENDTRY;					end of the error unit.
 *	NO_PROPAGATE				(Inside the CATCH segment: do not pass the error
 *								to the higher-level).
 *	Failure(status)				Initiate error recovery.
 *	FailureDebug(status)		DebugStr trap, then call Failure.
 *	FailureNeverDebug(status)	Don't call DebugStr trap, but fail nonetheless.
 *	FailNIL(ptr/handle)			Error if thing is NULL
 *	FailOSErr(status)			Error if status != noErr
 *	FailOSErrNeverDebug(status)	As with FailOSErr, but never DebugStr trap.
 *	FailMemError()				Fail if MemError is set.
 * If an error is not caught (in a sequence with NO_PROPAGATE), it will eventually
 * force an error alert with the indicated message.
 */

#ifndef _H_Failure
#define _H_Failure
#include <setjmp.h>
#ifndef THINK_C				/* MPW includes			*/
#include <Processes.h>
#include <Errors.h>
#include <Finder.h>
#include <Folders.h>
#include <Script.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Packages.h>
#include <Traps.h>
#include <Lists.h>
#endif

typedef struct FailureRecord {
	jmp_buf							jmpBuf;
	OSErr							status;
} FailureRecord, *FailureRecordPtr;

#define FATAL		(TRUE)
#define NON_FATAL	(FALSE)

#define TRY	{														\
		FailureRecordPtr	__oldFailurePtr;						\
		FailureRecord		__failureRecord;						\
		__oldFailurePtr = gFailurePtr;								\
		gFailurePtr = &__failureRecord;								\
		if ((__failureRecord.status									\
			= setjmp(__failureRecord.jmpBuf)) == noErr) {
			/* Normal code here */

#define CATCH														\
		} else {
			/* Error handler here */
#define ENDTRY														\
			}														\
			gFailurePtr = __oldFailurePtr;							\
			if (__failureRecord.status != noErr) {					\
				__FailureNeverDebug(__failureRecord.status);		\
			}														\
		}

#define STATUS			(gFailurePtr->status)
#define NO_PROPAGATE do {											\
		gFailurePtr->status = noErr;								\
	} while (0)
	

#define FailureDebug(status) do {									\
	DebugStr("\p" __FILE__);										\
		__Failure(status);											\
	} while (0)
#define Failure(status) do {										\
		__Failure(status);											\
	} while (0)
#define FailOSErr(status) do {										\
		OSErr			_status = (status);							\
		if (_status != noErr) {										\
			Failure(_status);										\
		}															\
	} while (0)
#define FailOSErrNeverDebug(status) do {							\
		OSErr			_status = (status);							\
		if (_status != noErr) {										\
			FailureNeverDebug(_status);								\
		}															\
	} while (0)
#define FailNIL(what) do {											\
		if ((what) == NULL) {										\
			__FailNIL();											\
		}															\
	} while (0)
#define FailMemError()		FailOSErr(MemError())

void							__Failure(
		OSErr						errorStatus
	);
void							FailureNeverDebug(
		OSErr						errorStatus
	);
void							__FailureNeverDebug(
		OSErr						errorStatus
	);
void							__FailNIL(void);
void							__FormatCallerName(
		unsigned long				*a6,
		StringPtr					callerName
	);

extern Str255				gFailureLocation;
extern FailureRecordPtr		gFailurePtr;
extern Boolean				gDebugOnError;

#endif
