/*									GetSDPIcon.c								*/
/*
 * DTS Catalog Peek
 * GetSDPIcon.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * The user has selected a new directory specification.
 * Grab the icon 
 *
 * Globals
 *	DBR.selectedDSSpec
 *			The currently-selected directory specification.
 *	DBR.isValidDSSpec
 *			TRUE if the directory spec is correctly
 *			formatted.
 *	DBR.SDPIconSuite
 *			The icon suite associated with this directory.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

/*
 * GetSDPIcon
 * This is called after unpacking the record selection
 * to extract the SDP icon, or to choose a generic icon
 * if no SDP icon is present.
 */
void
GetSDPIcon(
		register CatalogBrowserPtr		dbp
	)
{
		OSErr						status;
		short						genericIconSuiteID;
		
		DisposeIconSuiteHandle(dbp, &DBR.SDPIconSuite);	/* Redundant?	*/
		RefreshCatalogIcons(dbp);
		if (DBR.selectedDSSpec != NULL && DBR.isValidDSSpec) {
			status = SDPGetDSSpecIcon(
					DBR.selectedDSSpec,				/* This directory	*/
					svAllAvailableData,				/* All icons		*/
					&DBR.SDPIconSuite				/* Store it here	*/
				);
			LOG(status, "\pSDPGetDSSpecIcon");
			if (status != noErr) {
				/*
				 * If there is an error, use one of the generic icon suites.
				 * *TBS* -- select the correct suite: currently we can't
				 * display genericLockedCatalogIconResource.
				 */
				genericIconSuiteID = 0;
				switch (DBR.specType) {
				case kOCEDirsRootDSSpec:
				case kOCEDirectoryDSSpec:
					genericIconSuiteID = genericDirectoryIconResource;
					break;
				case kOCEDNodeDSSpec:
					/* TBS: what suite goes here? */
					break;
				case kOCERecordDSSpec:
					genericIconSuiteID = genericRecordIconResource;
					break;
				case kOCEentnDSSpec:
				case kOCENOTentnDSSpec:
					genericIconSuiteID = genericAttributeIconResource;
					break;
				default:
					break;
				}
				if (genericIconSuiteID != 0) {
					status = GetIconSuite(
							&DBR.SDPIconSuite,
							genericIconSuiteID,
							svAllAvailableData
						);
					LOG(status, "\pGetIconSuite");
				}
			}
		}
}

