/*										Main.c									*/
/*
 * DTS Catalog Peek
 * Main.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * Globals (application-wide)
 *	gAppleMenu
 *	gFileMenu
 *	gEditMenu
 *			The Menu handles
 *	gQuitNow
 *			This Boolean is set when the user wants to quit.
 *	gUpdateMenusNeeded
 *			This Boolean is set when menu options need to be enabled or disabled.
 *	gEventRecord (or EVENT)
 *			The current Event Record.
 *	gOpenWindowCount
 *			The number of Catalog Browser windows that are opened. The first is
 *			opened when the application is launched. When the last is closed
 *			(gOpenWindowCount goes to zero), gQuitNow will be set to exit the
 *			application.
 *	gNullAddrBlock
 *			An empty AddrBlock structure. Needed for some Catalog Manager calls.
 *	gSystemHasBalloonHelp
 *	gHasColorQuickDraw
 *	gHasPopupCDEF
 *			These Booleans are set by Gestalt if the corresponding capability is
 *			present.
 *	gDebugOnError
 *			This will be set if we're running under the Think Debugger.
 *	gInForeground
 *			This local Boolean is set if this is the current application. It is
 *			only used to adjust the WaitNextEvent timer.
 */
#define EXTERN			/* */
#include "DTSCatalogPeek.h"
#include <Balloons.h>
#pragma segment MainCode


/*
 * These enum's define items in the various menus.
 */
enum AppleMenu {
	kAppleAbout				= 1
};
enum FileMenu {
	kFileNewBrowser			= 1,
	kFileCloseBrowser,
	kFileUnused1,
	kFileSaveAs,
	kFileUnused2,
	kFileSetIdentity,
	kFileUnused3,
	kFileQuit
};
enum EditMenu {
	kEditUndo				= 1,
	kEditUnused,
	kEditCut,
	kEditCopy,
	kEditPaste,
	kEditClear
};

Boolean					gInForeground;

/*
 * Local function prototypes.
 */
void					main(void);
void					EventLoop(void);
void					ApplicationEventLoop(void);
void					DoMouseEvent(void);
void					ProcessActivateEvent(
		WindowPtr				activeWindow,
		Boolean					isActivate
	);
void					DoCommand(
		WindowPtr				activeWindow,
		long					menuChoice
	);
void					AdjustMenus(void);
void					DeleteAllWindows(void);
void					InitMacintosh(void);
void					InitApplication(void);
void					AdjustEditMenu(
		Boolean					isDeskAcc
	);

/*
 * main
 * The application main program.
 */
void
main()
{
		OSErr						status;
		
		InitMacintosh();
		/*
		 * This trick jumps to the Think Source debugger
		 * if it is running. I.e., if we're running from
		 * the project, rather from a compiled application.
		 */
		gDebugOnError = (Get1Resource('CODE', 0) == NULL);
		gInForeground = TRUE;
		TRY {
			InitApplication();
			if ((status = SystemSupportsAOCE()) != noErr) {
				ErrorAlert(ALRT_Fatal, status, "\pAOCE is unavailable");
				ExitToShell();
			}
			if (gHasPopupCDEF == FALSE) {
				ErrorAlert(
					ALRT_Fatal,
					gestaltUndefSelectorErr,
					"\pPopup Menu Control support is required"
				);
				ExitToShell();
			}
			MakeCatalogBrowser();
			InitCursor();
			gUpdateMenusNeeded = TRUE;
			while (gQuitNow == FALSE) {
				TRY {
					EventLoop();
				}
				CATCH {
					if (STATUS != noErr && STATUS != userCanceledErr)
						ErrorAlert(ALRT_NonFatal, STATUS, "\p");
					gUpdateMenusNeeded = TRUE;
					InitCursor();
					NO_PROPAGATE;
				}
				ENDTRY;
			}
		}
		CATCH {
			ErrorAlert(ALRT_Fatal, STATUS, "\pCan't continue");
			NO_PROPAGATE;					/* Nothing above us		*/
		}
		ENDTRY;
		TRY {
			DeleteAllWindows();
		}
		CATCH {
			ErrorAlert(ALRT_Fatal, STATUS, "\pFatal error during exit phase");
			NO_PROPAGATE;					/* Nothing above us		*/
		}
		ENDTRY;
		ExitToShell();
}

/*
 * Application event loop: process one event each time.
 */	
void
EventLoop(void)
{
		long							menuChoice;
		register WindowPtr				theWindow;
		GrafPtr							savePort;
		Boolean							isActivating;
		
		if (gUpdateMenusNeeded) {
			gUpdateMenusNeeded = FALSE;
			AdjustMenus();
		}
		WaitNextEvent(
			everyEvent,
			&EVENT,
			(gInForeground) ? 10L : 60L,
			NULL
		);
		theWindow = FrontWindow();
		switch (EVENT.what) {
		case nullEvent:
			break;
		case keyDown:
		case autoKey:
			if ((EVENT.message & charCodeMask) == '.'
			 && (EVENT.modifiers & cmdKey) != 0) {
				FlushEvents(keyDown | autoKey, 0);
				gQuitNow = TRUE;
			}
			else if ((EVENT.modifiers & cmdKey) != 0) {
				if (EVENT.what == keyDown) {
					menuChoice = MenuKey(EVENT.message & charCodeMask);
					if (HiWord(menuChoice) != 0 && IsOurWindow(theWindow))
						DoCommand(theWindow, menuChoice);
					else if (IsOurWindow(theWindow))
						DoWindowKeyDown((CatalogBrowserPtr) theWindow);
					else {
						SysBeep(10);
					}
				}
			}
			else if (IsOurWindow(theWindow))
				DoWindowKeyDown((CatalogBrowserPtr) theWindow);
			else {
				SysBeep(10);
			}
			break;
		case mouseDown:
			DoMouseEvent();
			break;
		case updateEvt:
			theWindow = (WindowPtr) EVENT.message;
			GetPort(&savePort);
			SetPort(theWindow);
			BeginUpdate(theWindow);
			EraseRect(&theWindow->portRect);
			if (IsOurWindow(theWindow) == FALSE)
				DrawControls(theWindow);
			else {
				UpdateBrowserWindow((CatalogBrowserPtr) theWindow);
			}
			EndUpdate(theWindow);
			SetPort(savePort);
			break;
		case activateEvt:
			theWindow = (WindowPtr) EVENT.message;
			isActivating = ((EVENT.modifiers & activeFlag) != 0);
			goto activateEvent;
			break;
		case osEvt:
			switch (((unsigned long) EVENT.message) >> 24) {
			case mouseMovedMessage:
				break;
			case suspendResumeMessage:
				isActivating = ((EVENT.message & 0x01) != 0);
activateEvent:		if (isActivating) {
					/*
					 * Activate this window. Activate events
					 * define theWindow from the event record,
					 * while suspend/resume uses the pre-set
					 * FrontWindow value.
					 */
					SelectWindow(theWindow);
					(void) TEFromScrap();
				}
				if (IsOurWindow(theWindow)) {
					ActivateCatalogBrowser(
						(CatalogBrowserPtr) theWindow,
						isActivating
					);
					/*
					 * Globalize it so we can find the current
					 * window in the debugger.
					 */
					if (isActivating) {
						gCurrentCatalogBrowserPtr =
							(CatalogBrowserPtr) theWindow;
					}
				}
				else {
					/* Desk accessory or what? */
				}
				gInForeground = isActivating;
				gUpdateMenusNeeded = TRUE;
				break;
			}
			break;
		}
		theWindow = FrontWindow();	/* Can be changed by events	*/
		if (IsOurWindow(theWindow)) {
			SetPort(theWindow);
			BalloonHelp((CatalogBrowserPtr) theWindow);
			DoSDPPanelEvent((CatalogBrowserPtr) theWindow);
		}
}

/*
 * DoMouseEvent
 * The user clicked on something. Handle application-wide
 * processing here, or call a Catalog Browser function
 * for specific action.
 */
void
DoMouseEvent(void)
{
		WindowPtr		theWindow;
		short			whichPart;
		
		whichPart = FindWindow(EVENT.where, &theWindow);
		if (theWindow == NULL)
			theWindow = FrontWindow();
		if (whichPart == inMenuBar && IsOurWindow(theWindow) == FALSE)
			theWindow = FrontWindow();
		switch (whichPart) {
		case inDesk:
			break;
		case inMenuBar:
			InitCursor();
			DoCommand(theWindow, MenuSelect(EVENT.where));
			break;
		case inDrag:
			DragWindow(theWindow, EVENT.where, &qd.screenBits.bounds);
			break;
		case inGoAway:
			if (TrackGoAway(theWindow, EVENT.where)) {
				if (IsOurWindow(theWindow)) {
					DisposeCatalogBrowser((CatalogBrowserPtr) theWindow);
					if (gOpenWindowCount <= 0)
						gQuitNow = TRUE;
				}
				else {
					SysBeep(10);
				}
			}
			break;
		case inZoomIn:
		case inZoomOut:
			if (IsOurWindow(theWindow)
			 && TrackBox(theWindow, EVENT.where, whichPart)) {
				DoZoomWindow(theWindow, whichPart);
				goto resizeWindow;
			}
			break;
		case inGrow:
			if (IsOurWindow(theWindow)) {
				if (DoGrowWindow(theWindow, kMinWindowWidth, kMinWindowHeight)) {
resizeWindow:		DecorateDisplay((CatalogBrowserPtr) theWindow, TRUE);
				}
			}
			break;
		case inContent:
			if (theWindow != FrontWindow())
				SelectWindow(theWindow);
			else if (IsOurWindow(theWindow)) {
				DoContentClick((CatalogBrowserPtr) theWindow);
			}
			else {
				/* Nothing happens here		*/
			}
			break;
		default:
			break;						/* Might be for the SDP Panel	*/
		}
}

/*
 * DoCommand
 * Process a menu or keystroke command.
 */
void
DoCommand(
		WindowPtr				theWindow,
		long					menuChoice
	)
{
		short				menuItem;
		Str255				menuText;
		GrafPtr				savePort;
		
		menuItem = LoWord(menuChoice);
		switch (HiWord(menuChoice)) {
		case MENU_Apple:
			if (menuItem == kAppleAbout)
				DoHelp(DLOG_Help, STRn_Help);
			else {
				GetItem(gAppleMenu, menuItem, menuText);
				AdjustEditMenu(TRUE);
				GetPort(&savePort);
				OpenDeskAcc(menuText);
				SetPort(savePort);
				AdjustEditMenu(IsOurWindow(theWindow) == FALSE);
			}
			break;
		case MENU_File:
			switch (menuItem) {
			case kFileNewBrowser:
				MakeCatalogBrowser();
				break;
			case kFileCloseBrowser:
				if (IsOurWindow(theWindow)) {
					DisposeCatalogBrowser((CatalogBrowserPtr) theWindow);
					if (gOpenWindowCount <= 0)
						gQuitNow = TRUE;
				}
				break;
			case kFileSaveAs:
				if (IsOurWindow(theWindow))
					SaveCatalogBrowserInfo((CatalogBrowserPtr) theWindow);
				break;

			case kFileSetIdentity:
				if (IsOurWindow(theWindow))
					SetSpecificIdentity((CatalogBrowserPtr) theWindow);
				break;			
			case kFileQuit:
				gQuitNow = TRUE;
				break;
			}
			break;
		case MENU_Edit:
			if (SystemEdit(menuItem - 1) == FALSE)
				SysBeep(10);
			break;
		case kHMHelpMenuID:
			DoHelp(DLOG_Help, STRn_Help);
			break;
		}
		HiliteMenu(0);
}		

/*
 * AdjustMenus
 * Enable/disable menu options.
 */
void
AdjustMenus(void)
{
		Boolean				isOurWindow;
		
		isOurWindow = IsOurWindow(FrontWindow());
		EnableItem(gFileMenu, kFileQuit);
		EnableItem(gFileMenu, kFileNewBrowser);
		if (isOurWindow) {
			EnableItem(gFileMenu, kFileCloseBrowser);
// TBS		EnableItem(gFileMenu, kFileSaveAs);
			EnableItem(gFileMenu, kFileSetIdentity);
		}
		else {
			DisableItem(gFileMenu, kFileCloseBrowser);
			DisableItem(gFileMenu, kFileSaveAs);
			DisableItem(gFileMenu, kFileSetIdentity);
		}
		AdjustEditMenu(isOurWindow == FALSE);
}

/*
 * AdjustEditMenu
 * Enable/disable Edit Menu options.
 */
void
AdjustEditMenu(
		Boolean				isDeskAcc
	)
{
		if (isDeskAcc) {
			EnableItem(gEditMenu, kEditUndo);
			EnableItem(gEditMenu, kEditCut);
			EnableItem(gEditMenu, kEditCopy);
			EnableItem(gEditMenu, kEditPaste);
			EnableItem(gEditMenu, kEditClear);
		}
		else {
			DisableItem(gEditMenu, kEditUndo);
			DisableItem(gEditMenu, kEditCut);
			DisableItem(gEditMenu, kEditCopy);
			DisableItem(gEditMenu, kEditPaste);
			DisableItem(gEditMenu, kEditClear);
		}
}

/*
 * DeleteAllWindows
 * This is called when the application quits to make
 * sure that all windows are deleted. It isn't really
 * needed until we have to handle open files.
 */
void
DeleteAllWindows(void)
{
		WindowPtr			thisWindow;
		WindowPeek			nextWindow;
		
		thisWindow = FrontWindow();
		while (thisWindow != NULL) {
			nextWindow = ((WindowPeek) thisWindow)->nextWindow;
			if (IsOurWindow(thisWindow)) {
				DisposeCatalogBrowser(
					(CatalogBrowserPtr) thisWindow
				);
			}
			thisWindow = (WindowPtr) nextWindow;
		}
}

/*
 * ClearMemory
 * This is a utility function that clears a block
 * of memory. It isn't very efficient. It us
 * normally called by executing the CLEAR macro
 * which expands CLEAR(thing) to
 *		ClearMemory(&thing, sizeof thing);
 */
void
ClearMemory(
		void				*recordPtr,
		register Size		recordSize
	)
{
		register char		*rp;
		
		rp = (char *) recordPtr;
		while (recordSize > 0) {
			*rp++ = 0;
			--recordSize;
		}
}

/*
 * InitMacintosh
 * Perform the normal application initialization.
 * This should probably be extended to add a
 * "rainy day" memory fund. The only thing
 * this module does is initialize the managers.
 */
void
InitMacintosh(void)
{
		int				i;
		
		MaxApplZone();		
		InitGraf(&qd.thePort);
		InitFonts();
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(NULL);
		HNoPurge((Handle) GetCursor(watchCursor));
		SetCursor(*GetCursor(watchCursor));
		for (i = 0; i < 3; i++)
			EventAvail(everyEvent, &EVENT);
}

/*
 * InitApplication
 * Continue initialization. Failure is possible from
 * this module -- it results in a fatal error message.
 */
void
InitApplication(void)
{
		Handle			menuBarHdl;
		long			response;

		(void) TEFromScrap();
		if (Gestalt(gestaltHelpMgrAttr, &response) == noErr
		 && (response & (1 << gestaltHelpMgrPresent)) != 0)
		 	gSystemHasBalloonHelp = TRUE;
		if (Gestalt(gestaltQuickdrawVersion, &response) == noErr
		 && response >= gestalt8BitQD)
		 	gHasColorQuickDraw = TRUE;
		if (Gestalt(gestaltPopupAttr, &response) == noErr
		 && (response & (1 << gestaltPopupPresent)) != 0)
		 	gHasPopupCDEF = TRUE;
		menuBarHdl = GetNewMBar(MBAR_MenuBar);
		FailNIL(menuBarHdl);
		SetMenuBar(menuBarHdl);
		gAppleMenu = GetMHandle(MENU_Apple);
		FailNIL(gAppleMenu);
		gFileMenu = GetMHandle(MENU_File);
		FailNIL(gFileMenu);
		gEditMenu = GetMHandle(MENU_Edit);
		FailNIL(gEditMenu);
		AddResMenu(GetMHandle(MENU_Apple), 'DRVR');
		InitializeBalloonHelp();
		DrawMenuBar();
}

/*
 * IsOurWindow
 * Return TRUE if this is a directory browser window.
 */
Boolean
IsOurWindow(
		WindowPtr			theWindow
	)
{
		if (theWindow == NULL
		 || ((WindowPeek) theWindow)->windowKind != userKind)
			return (FALSE);
		else {
			return (TRUE);
		}
}
