/*							ManagePersonalAddrBook.c							*/
/*
 * DTS Catalog Peek
 * ManagePersonalAddrBook.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * These functions open and close the personal address book.
 *
 * Globals
 *	SDP_INFO_PDRefNum
 *			The current personal directory reference number.
 *			Most Catalog Manager calls need this. It is
 *			non-zero if a personal directory is open.
 *	SDP_INFO_RecordID.rli
 *			The record location identifier of the current
 *			directory browser record. OpenPersonalAddressBook
 *			checks whether this is a personal directory and.
 *			if so, opens it.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

/*
 * OpenPersonalAddressBook
 * If the current selection is an alias, open the personal
 * address book that is related to that alias. Note: this call
 * is valid only if the selection state is kSDPRecordSelected
 * or kSDPRecordAliasSelected.
 */
void
OpenPersonalAddressBook(
		register CatalogBrowserPtr		dbp
	)
{
		AliasPtr				aliasPtr;
		AliasHandle				aliasHandle;
		Boolean					wasChanged;
		FSSpec					theFSSpec;
		OSErr					status;
#define OPEN (PB.openPersonalDirectoryPB)

		aliasHandle = NULL;
		SDP_INFO_PDRefNum = 0;
		if (OCEValidPackedRLI(SDP_INFO_RecordID.rli)) {
			aliasPtr = OCEExtractAlias(SDP_INFO_RecordID.rli);
			if (aliasPtr != NULL) {
				FailOSErr(
					PtrToHand(
						(Ptr) aliasPtr,
						(Handle *) &aliasHandle,
						aliasPtr->aliasSize
					)
				);
				status = ResolveAlias(
						NULL,
						aliasHandle,
						&theFSSpec,
						&wasChanged
					);
				if (status != noErr)
					LOG(status, "\pOpen PAB: ResolveAlias");
				ForgetHandle(aliasHandle);
				if (status == noErr) {
					OPEN.fsSpec = &theFSSpec;
					OPEN.accessRequested = fsRdPerm;
					status = DirOpenPersonalDirectory(&PB);
					if (status == noErr)
						SDP_INFO_PDRefNum = OPEN.dsRefNum;
					else {
						LOG(status, "\pOpen PAB: DirOpenPersonalDirectory");
						NOTE(theFSSpec.name);
					}
				}
			}
		}
#undef OPEN
}


/*
 * ClosePersonalAddressBook
 * Close the current personal address book, if there is one.
 */
void
ClosePersonalAddressBook(
		register CatalogBrowserPtr		dbp
	)
{
		if (SDP_INFO_PDRefNum != 0) {
			PB.closePersonalDirectoryPB.dsRefNum = SDP_INFO_PDRefNum;
			SDP_INFO_PDRefNum = 0;
			FailOSErr(DirClosePersonalDirectory(&PB));
		}
}

