/*								SetupCatalogLDEF.c							*/
/*
 * DTS Catalog Peek
 * SetupCatalogLDEF.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * This is a List display function for lists whose
 * contents are DirectoryEnumSpec's. (This could
 * be better.)
 * The text is drawn using a style to indicate the contents.
 *	Plain			Records
 *	Italic			Alias records
 *	Underline		Pseudonym
 *	Shadow			Unknown (bug?)
 *	Bold			DNode
 * In addition, "invisible" records are outlined.
 */
#include "DTSCatalogPeek.h"
#ifndef THINK_C
#include <SysEqu.h>
#endif
#pragma segment SupportCode

/*
 * SetupCatalogLDEF
 * Draw the directory name. The cell contains a
 * DirEnumSpec
 */
pascal void
SetupCatalogLDEF(
		short							listMessage,
		Boolean							listSelect,
		Rect							*listRect,
		Cell							listCell,
		short							listDataOffset,
		short							listDataLen,
		ListHandle						listHandle
	)
{
#pragma unused (listCell)
#define LIST	(**listHandle)
#define SPEC	(dirEnumSpec)
#define LRID	(SPEC.u.recordIdentifier)
#define DNODE	(SPEC.u.dNodeIdentifier)

		Boolean							cellLockState;
		register CatalogBrowserPtr		dbp;
		Rect							viewRect;
		DirEnumSpec						dirEnumSpec;
		Handle							dirEnumSpecHdl;
		unsigned short					handleLockState;
		Style							recordStyle;

		dbp = (CatalogBrowserPtr) LIST.refCon;
		switch (listMessage) {
		case lInitMsg:
			break;
		case lDrawMsg:
			EraseRect(listRect);
			if (listDataLen > 0) {
				TextFont(DBR.typeFont.fontNumber);
				TextSize(DBR.typeFont.fontSize);
				viewRect = *listRect;
				viewRect.left += LIST.indent.h;
				cellLockState = HGetState(LIST.cells);
				HLock(LIST.cells);
				dirEnumSpecHdl =
					(* ((Handle *) ((*LIST.cells) + listDataOffset)));
				handleLockState = GetDirEnumSpecFromHandle(
							dirEnumSpecHdl,
							&dirEnumSpec
						);
				switch (SPEC.enumFlag & ~kEnumInvisibleMask) {
				default:
					recordStyle = shadow;		/* Can't happen		*/
					goto drawRecord;
				case kEnumDistinguishedNameMask:
					recordStyle = 0;			/* Plain			*/
					goto drawRecord;
				case kEnumAliasMask:
					recordStyle = italic;
					goto drawRecord;
				case kEnumPseudonymMask:
					recordStyle = underline;
drawRecord:			if ((SPEC.enumFlag & kEnumInvisibleMask) != 0)
						recordStyle |= outline;
					/*
					 * It's a Local Record ID. Draw the record name
					 * and type (assuming there's room)
					 */
					DrawRString(LRID.recordName, &viewRect, recordStyle);
					viewRect.left += RStringWidth(LRID.recordName);
					if (viewRect.left < viewRect.right) {
						DrawWideString("\p (", &viewRect, 0);
						viewRect.left += StringWidth("\p (");
					}
					if (viewRect.left < viewRect.right) {
						DrawRString(LRID.recordType, &viewRect, 0);
						viewRect.left += RStringWidth(LRID.recordType);
					}
					if (viewRect.left < viewRect.right) {
						DrawWideString("\p)", &viewRect, 0);
					}
					break;
				case kEnumDNodeMask:
					/*
					 * It's a DNode. Draw the name. (We should
					 * draw the icon, too).
					 */
					recordStyle = bold;
					if ((SPEC.enumFlag & kEnumInvisibleMask) != 0)
						recordStyle |= outline;
					DrawRString(DNODE.name, &viewRect, recordStyle);
					break;
				}
				TextFace(0);			/* Plain		*/
				HSetState(dirEnumSpecHdl, handleLockState);
				HSetState(LIST.cells, cellLockState);
			}
			if (listSelect == FALSE)
				break;
			/* Continue to do hilite */
		case lHiliteMsg:
#ifdef THINK_C
			HiliteMode &= ~(1 << hiliteBit);
#else /* MPW */
			*((char *) HiliteMode) &= ~(1 << hiliteBit);	/* Inside Mac V-61	*/
#endif
			InvertRect(listRect);
			break;
		}
#undef LIST
}



