/*							UnpackSDPPanelSelection.c							*/
/*
 * DTS Catalog Peek
 * UnpackSDPPanelSelection.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * Completely unpack the contents of the current DSSpec.
 * Globals (these define the currently-selected directory):
 *	DBR.selectedDSSpecSize
 *			The size of the current directory specification.
 *	DBR.selectedDSSpec
 *			The current packed directory specification. This
 *			is extracted from the directory panel (here)
 *	DBR.theDSSpec
 *			The unpacked directory specification (points to
 *			DBR.selectedDSSpec)
 *	SDP_INFO_RecordID
 *			The record id that is associated with the current
 *			directory specification.
 *	DBR.theDSSpecRLI
 *			The record location identifier associated with
 *			the current directory specification.
 *	DBR.specType
 *			The type of the current directory specification.
 *	DBR.isValidDSSpec
 *	DBR.isNonNullRLI
 *	DBR.isValidPackedRLI
 *	DBR.isValidPackedPathName
 *	DBR.isValidNodeNameCount
 *			These Booleans are TRUE if the associated test
 *			succeeds. Note that tests are done in sequence:
 *			if DBR.isNonNullRLI is false, there is no test
 *			for DBR.isValidPackedRLI.
 *	DBR.nodeNameCount
 *			The number of node names defined for this RLI.
 *			This is the maximum index into DBR.partsPtr.
 *	DBR.unpackedNodeNameCount
 *			This should equal DBR.nodeNameCount: this is
 *			the actual number of RStrings unpacked into
 *			DBR.partsPtr
 *	DBR.partsPtr
 *			The RStrings that define the path to this RLI.
 *	DBR.SDPIconSuite
 *			The icon suite associated with this directory.
 *			(Extracted in GetSDPIcon.c)
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

/*
 * UnpackSDPPanelSelection
 * This function performs a complete "unpacking" of
 * the currently-selected DSSpec. Flags are set
 * along the way if any structure fails to unpack.
 */
void
UnpackSDPPanelSelection(
		register CatalogBrowserPtr		dbp,
		SDPPanelHandle					thePanel
	)
{
		OSErr						status;
		
		DBR.isValidDSSpec = FALSE;
		DBR.isValidPackedRLI = FALSE;
		DBR.isValidPackedPathName = FALSE;
		DBR.isValidNodeNameCount = FALSE;
		DBR.isValidUnpackedCount = FALSE;
		DBR.specType = kOCEInvalidDSSpec;
		DBR.nodeNameCount = 0;
		ForgetPtr(DBR.partsPtr);
		ForgetPtr(DBR.selectedDSSpec);
		status = SDPGetPanelSelectionSize(
				thePanel,
				&DBR.selectedDSSpecSize
			);
		if (status != noErr) {
			LOG(status, "\pSDPGetPanelSelectionSize");
			return;
		}
		DBR.selectedDSSpec = (PackedDSSpecPtr)
			NewPtrClear(DBR.selectedDSSpecSize);
		FailNIL(DBR.selectedDSSpec);
		status = SDPGetPanelSelection(
				thePanel,
				DBR.selectedDSSpec
			);
		if (status != noErr) {
			LOG(status, "\pSDPGetPanelSelection");
			return;
		}
		/*
		 * Make sure our record id is initialized in case
		 * OCEUnpackDSSpec doesn't set something in it.
		 */
		SDP_INFO_RecordID = gNullRecordID;
		DBR.isValidDSSpec = OCEValidPackedDSSpec(DBR.selectedDSSpec);
		if (DBR.isValidDSSpec) {
			OCEUnpackDSSpec(
				DBR.selectedDSSpec,
				&DBR.theDSSpec,
				&SDP_INFO_RecordID
			);
			/*
			 * Write the DSSpec record name into the error
			 * log to separate error sequences.
			 */
			{
				Str255				work;
				RecordIDPtr			thisEntity;

				thisEntity = DBR.theDSSpec.entitySpecifier;
				pstrcpy(work, "\p ");
				pstrcat(
					work,
					OCERToPString(thisEntity->local.recordName)
				);
				NOTE(work);
			}
			DBR.specType = OCEGetDSSpecInfo(&DBR.theDSSpec);
			DBR.isNonNullRLI = (SDP_INFO_RecordID.rli != NULL);
			if (DBR.isNonNullRLI) {
				DBR.isValidPackedRLI = OCEValidPackedRLI(SDP_INFO_RecordID.rli);
				if (DBR.isValidPackedRLI) {
					OCEUnpackRLI(SDP_INFO_RecordID.rli, &DBR.theDSSpecRLI);
					DBR.isValidPackedPathName = OCEValidPackedPathName(DBR.theDSSpecRLI.path);
					/***
					 *** DBR.isValidPackedPathName is FALSE if you click
					 *** on a printer or CPU in the AppleTalk directory.
					 ***/
					if (DBR.isValidPackedPathName) {
						DBR.nodeNameCount = OCEDNodeNameCount(DBR.theDSSpecRLI.path);
						DBR.isValidNodeNameCount = (DBR.nodeNameCount < 64);
						if (DBR.isValidNodeNameCount) {
							DBR.partsPtr = (RStringPtr *) NewPtrClear(
									sizeof (RStringPtr) * DBR.nodeNameCount
								);
							FailNIL(DBR.partsPtr);
							DBR.unpackedNodeNameCount = OCEUnpackPathName(
									DBR.theDSSpecRLI.path,
									DBR.partsPtr,
									DBR.nodeNameCount
								);
							DBR.isValidUnpackedCount =
								(DBR.nodeNameCount == DBR.unpackedNodeNameCount);
						}
						else {
							NOTE("\pInvalid node name count");
						}
					}
					else {
						NOTE("\pInvalid PackedPathName");
					}
				}
				else {
					NOTE("\pInvalid Packed RLI");
				}
			}
			else {
				NOTE("\pValid DSSpec but NULL RLI");
			}
		}
		else {
			NOTE("\pInvalid Packed DSSpec");
		}
}
