/*									VampDisplay.c								*/
/*
 * DTS Catalog Peek
 * VampDisplay.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 *
 * VampDisplayIdle is called by SDPDirectoryPanel when it
 * is actively looking for something on the network.
 *
 * Globals
 *	DBR.vampSICNIndex
 *			The current icon to draw. This ranges from
 *			kFirstSpinnerIcon to kLastSpinnerIcon inclusive.
 *	DBR.nextAnimation
 *			The next TickCount to animate the icon.
 * TBS: This all works, but the icon callback won't be
 * in AOCE until Beta 2.
 */
#include "DTSCatalogPeek.h"
#pragma segment MainCode

#define kVampDisplayDelay		(6)				/* Ticks		*/

/*
 * InitializeVampDisplay is called when the window is created to
 * initialize the hourglass buffer.
 */
void
InitializeVampDisplay(
		register CatalogBrowserPtr		dbp
	)
{
#ifdef kFirstSpinnerIcon	/* TBS: remove at beta 2 release	*/
		DBR.vampSICNIndex = kFirstSpinnerIcon;
#endif
		DBR.nextAnimation = TickCount() + kVampDisplayDelay;
}


/*
 * AnimateVampDisplay does all the work. It is called repeatedly to
 * update the icon display.
 */
pascal void AnimateVampDisplay(
		SDPPanelHandle					panelHandle,
		Boolean							busy
	)
{
		register CatalogBrowserPtr		dbp;
		unsigned long					now;
		
		dbp = (CatalogBrowserPtr) (**panelHandle).refCon;
		now = TickCount();
		if (busy && now >= DBR.nextAnimation) {
			DBR.nextAnimation = now + kVampDisplayDelay;
			++DBR.vampSICNIndex;
#ifdef kFirstSpinnerIcon
			if (DBR.vampSICNIndex > kLastSpinnerIcon)
				DBR.vampSICNIndex = kFirstSpinnerIcon;
#endif
			UpdateVampDisplay(dbp);
		}			
}

/*
 * UpdateVampDisplay is called by update events. It just blasts the
 * icon to the window.
 */
void
UpdateVampDisplay(
		register CatalogBrowserPtr		dbp
	)
{
#ifdef kFirstSpinnerIcon
		(void) PlotIconID(
					&RECT(kInSpinnerRect),
					atNone,
					ttNone,
					DBR.vampSICNIndex
				);
#else
#pragma unused (dbp)
#endif
}

