/*
 * AuditFileDialog.c
 * Copyright  1992-93, Apple Computer Inc. All Rights Reserved.
 * Programmed by Martin Minow,
 *	Internet:	minow@apple.com
 *	AppleLink:	MINOW
 * Version of January 22, 1993
 *
 * File Dialog for Audit display. This replaces the
 * SFPutFile dialog, adding a TextEdit field for the
 * Audit Selector and a checkbox for "initially
 * enabled." See Inside Mac Files (3-21) for details.
 *
 * Edit History
 *	93.01.14 MM		First public release
 */
#include "DisplayAudit.h"
#include <StandardFile.h>

/*
 * Standard File dialog items. This follows the layout
 * of the old Standard File (-3999) DITL.
 */
enum {
	kSFSaveButton = 1,
	kSFCancelButton,
	kSFSaveAsStaticText,
	kSFVolumeUser,
	kSFEjectButton,
	kSFDriveButton,
	kSFFileNameTextEcit,
	kSFFileListUser,
	kSFItemAuditSelectorTextEdit,
	kSFItemAuditSelectorStaticText,
	kSFItemAuditEnabledCheckBox
};
enum {
	kFirstTime			= -1	/* the first time our hook it's passed a -1		*/
};


static pascal short		PutDialogFilter(
		short					itemHit,
		DialogPtr				theDialog
	);


void						AuditPutFile(
		short					dialogID,
		const StringPtr			promptString,
		const StringPtr			originalName,
		SFReply					*reply,
		OSType					*auditSelector,
		Boolean					*enableAudit
	);
static OSType					sAuditSelector;
static Boolean					sEnableAudit;

void						AuditPutFile(
		short					dialogID,
		const StringPtr			promptString,
		const StringPtr			originalName,
		SFReply					*reply,
		OSType					*auditSelector,
		Boolean					*enableAudit
	)
{
		Point					where;
		DialogTHndl				dialogHdl;
		Rect					box;

		sAuditSelector = *auditSelector;
		sEnableAudit = *enableAudit;
		/*
		 * Center the dialog
		 */
		dialogHdl = (DialogTHndl) GetResource('DLOG', dialogID);
		if (dialogHdl == NULL)							/* No resource!			*/
		    SetPt(&where, 80, 80);
		else {
		    box = (**dialogHdl).boundsRect;				/* Dialog shape			*/
		    ReleaseResource((Handle) dialogHdl);
			where.h =
				(width(qd.screenBits.bounds) - width(box)) / 2;
			where.v = 
				((height(qd.screenBits.bounds) - GetMBarHeight()) / 3)
				+ GetMBarHeight();
		}
		SFPPutFile(
			where,
			promptString,
			originalName,
			(DlgHookProcPtr) PutDialogFilter,	/* Dialog item filter			*/
			reply,
			dialogID,
			NULL								/* No Modal Dialog Filter		*/
		);
		*auditSelector = sAuditSelector;
		*enableAudit = sEnableAudit;
}

static pascal short
PutDialogFilter(
		short					itemHit,
		DialogPtr				theDialog
	)
{
		short			itemType;
		Handle			itemHandle;
		Rect			itemRect;
		short			i;
		Str255			auditSelector;	
#define DIALOG	(* ((DialogPeek) theDialog))

		switch (itemHit) {
		case kFirstTime:
			/*
			 * Copy the audit selector from the global
			 * storage to the dialog item.
			 */
			auditSelector[0] = 4;
			BlockMove(&sAuditSelector, &auditSelector[1], 4);
			for (i = 1; i <= 4; i++) {
				if (auditSelector[i] < ' ') {
					auditSelector[0] = i - 1;
					break;
				}
			}
			GetDItem(
				theDialog,
				kSFItemAuditSelectorTextEdit,
				&itemType,
				&itemHandle,
				&itemRect
			);
			SetIText(itemHandle, auditSelector);
			/*
			 * Set the checkbox status.
			 */
			GetDItem(
				theDialog,
				kSFItemAuditEnabledCheckBox,
				&itemType,
				&itemHandle,
				&itemRect
			);
			SetCtlValue((ControlHandle) itemHandle, sEnableAudit);
			break;
		case kSFItemAuditEnabledCheckBox:
			GetDItem(
				theDialog,
				kSFItemAuditEnabledCheckBox,
				&itemType,
				&itemHandle,
				&itemRect
			);
			SetCtlValue(
				(ControlHandle) itemHandle,
				1 - GetCtlValue((ControlHandle) itemHandle)
			);
			break;
		case kSFSaveButton:
		case kSFCancelButton:
			/*
			 * Exit: get the audit selector, even if the
			 * user hit the "Cancel" button. SFPPutFile
			 * will still return reply->good == FALSE
			 * if the file isn't to be created.
			 */
			GetDItem(
				theDialog,
				kSFItemAuditSelectorTextEdit,
				&itemType,
				&itemHandle,
				&itemRect
			);
			GetIText(itemHandle, auditSelector);
			sAuditSelector = 0;
			for (i = auditSelector[0] + 1; i <= 4; i++)
				auditSelector[i] = 0;
			if (auditSelector[0] != 4)
				auditSelector[0] = 4;
			BlockMove(&auditSelector[1], &sAuditSelector, 4);
			/*
			 * Get the checkbox status.
			 */
			GetDItem(
				theDialog,
				kSFItemAuditEnabledCheckBox,
				&itemType,
				&itemHandle,
				&itemRect
			);
			sEnableAudit = GetCtlValue((ControlHandle) itemHandle);
		}
		return (itemHit);
}		

