/*
 * DisplayAudit.c
 * Copyright  1992-93, Apple Computer Inc. All Rights Reserved.
 * Programmed by Martin Minow,
 *	Internet:	minow@apple.com
 *	AppleLink:	MINOW
 * Version of January 14, 1993
 *
 * Edit History
 *	93.01.09 MM		First public release
 *	93.01.14 MM		Removed Think C stdio dependencies. Added
 *					Standard File dialog with additional dialog
 *					items to allow defining the audit selector.
 *	93.01.19 MM		MPW \r is the wrong character. Force <return>
 *					on both MPW and Think, because this is what
 *					the Mac file system requires for end of line.
 *					(Change is in DisplayAuditFile.c). #if'ed out
 *					some test calls. Also added error status logging
 *					for file I/O errors.
 *	93.01.22 MM		Added a preference file that remembers the last
 *					audit selector parameter and window sizes. Also
 *					added error alerts. Moved global parameters to
 *					DisplayAudit.h
 *
 * This stand-alone application displays the data from a
 * AuditRecord, writing it to a file or the screen. It is not
 * very intelligent.
 */

#include "DisplayAudit.h"
#include "SimpleTextWindow.h"
#define kAuditCycles			100

#ifndef TESTING
#define TESTING					0
#endif

WindowPtr			gTextWindow;
EventRecord			gEventRecord;
#define	EVENT		(gEventRecord)
Parameters			gParameterRecord;
#define PARM		(gParameterRecord)
short				gParameterFileRefNum = kNoParamFile;
Boolean				gParameterUpdateNeeded = FALSE;
AuditPtr			gAuditPtr = NULL;
ProcessSerialNumber	gOldPSN;
unsigned long		gTotalMissedCount;
unsigned long		gLogIndex;
AuditEntry			gCurrentLogEntry;
#define ENTRY		(gCurrentLogEntry)
Boolean				gQuitNow = FALSE;
SFReply				gSFReply;
short				gFileRefNum = 0;	/* Non-zero if output on	*/
Boolean				gEnableAudit = TRUE;
StringPtr			gErrorMessage[] = {	/* Should be a resource		*/
	"\pDummy test message",				/* No messageIndex zero		*/
	"\pCan't initialize application - fatal",
	"\pNo memory",
	"\pCan't create audit record - fatal",
	"\pCan't create display window",
	/*
	 * Preference file errors
	 */
	"\pWrong system version: cannot create preference file",
	"\pCan't find Preference folder in System folder",
	"\pCan't find Preference file",
	"\pCan't open Preference file",
	"\pCan't create Preference file",
	"\pCan't write Preference information",
	"\pCan't read Preference information",
	/*
	 * Log file errors
	 */
	"\pCan't create output file",
	"\pCan't write ouput file",
	"\pCan't close output file"
};

void					ProcessAudit(void);
void					DisplayLogEntry(void);
void					DumpAuditEntry(
		register AuditPtr			auditPtr,
		register AuditEntryPtr		entryPtr
	);

void				InitializeApplication(void);
void				InitializeTextWindow(void);
void				InitializeAudit(void);
void				EventLoop(void);
void				DoMouseEvent(void);
void				DoCommand(
		long				menuChoice
	);
void				AdjustMenus(
		WindowPtr			theWindow
	);
void				AdjustEditMenu(
		Boolean				isDeskAcc
	);
#if TESTING
void				TestAuditStatusLocationUsingAnExtremelyLongFunctionName(void);
pascal void			TestPascalVoidFunction(void);
pascal short		TestPascalShortFunction(void);
pascal OSErr		TestPascalFunctionWithParameters(Ptr, Ptr);
#endif


enum AppleMenu {
	kAppleAbout			= 1
};
enum FileMenu {
	kFileSaveAs			= 1,
	kFileClose,
	kFileUnused,
	kFileQuit
};
#define kFileMenuString						\
	"\pSave As\xC9/S;"						\
	"Close File;"							\
	"(-;"									\
	"Quit/Q"
enum EditMenu {
	kEditUndo				= 1,
	kEditUnused,
	kEditCut,
	kEditCopy,
	kEditPaste,
	kEditClear
};
#define kEditMenuString		\
	"\pUndo/Z;(-;Cut/X;Copy/C;Paste/V;Clear"

MenuHandle				gAppleMenu;
MenuHandle				gFileMenu;
MenuHandle				gEditMenu;


		
void
main(void)
{
		TRY {
			InitializeApplication();
		 	InitializeTextWindow();
			InitializeAudit();
			gAuditPtr = GetAuditPtr(PARM.auditIdent);
			if (gAuditPtr == NULL) {
				DisplayString(gTextWindow, "\pLog creation failed");
				Failure(MemError(), kErrCreateAuditRecord);
			}
			while (gQuitNow == FALSE) {
				EventLoop();
				ProcessAudit();
				if (gParameterUpdateNeeded)
					WriteAuditParameters();
			}
		}
		CATCH {
			ErrorAlert(STATUS, MESSAGE, TRUE);	/* Fatal	*/
			NO_PROPAGATE;
		}
		ENDTRY;
		if (gParameterFileRefNum != kNoParamFile) {
			CloseResFile(gParameterFileRefNum);
			gParameterFileRefNum = kNoParamFile;
		}
		WakeUpAudit(gAuditPtr, &gOldPSN);
		ExitToShell();
}

void
EventLoop(void)
{
		long				menuChoice;
		TextWindowState		textWindowState;
		
		WaitNextEvent(everyEvent, &EVENT, 60L, 0);
		textWindowState = TextWindowEvent(gTextWindow, &EVENT);
		if ((textWindowState & kTextWindowMoved) != 0) {
			/*
			 * Remember the window location for next time.
			 */
			PARM.topLeft.h = gTextWindow->portRect.left;
			PARM.topLeft.v = gTextWindow->portRect.top;
			LocalToGlobal(&PARM.topLeft);
			PARM.windowSize.h = width(gTextWindow->portRect);
			PARM.windowSize.v = height(gTextWindow->portRect);
			gParameterUpdateNeeded = TRUE;
		}
		if ((textWindowState & kTextWindowEvent) == 0) {
			switch (EVENT.what) {
			case nullEvent:
				break;
			case keyDown:
			case autoKey:
				if ((EVENT.message & charCodeMask) == '.'
				 && (EVENT.modifiers & cmdKey) != 0) {
					FlushEvents(keyDown | autoKey, 0);
					gQuitNow = TRUE;
				}
				else if ((EVENT.modifiers & cmdKey) != 0) {
					if (EVENT.what == keyDown) {
						menuChoice = MenuKey(EVENT.message & charCodeMask);
						if (HiWord(menuChoice) != 0)
							DoCommand(menuChoice);
						else {
							SysBeep(10);
						}
					}
				}
				else {
					SysBeep(10);
				}
				break;
			case mouseDown:
				DoMouseEvent();
				break;
			case activateEvt:		/* Hook for later development	*/
				break;
			case osEvt:				/* Hook for later development	*/
				switch (((unsigned long) EVENT.message) >> 24) {
				case mouseMovedMessage:
					break;
				case suspendResumeMessage:
					break;
				}
				break;
			}
		}
}

void
DoMouseEvent(void)
{
		WindowPtr		theWindow;
		short			whichPart;
		
		whichPart = FindWindow(EVENT.where, &theWindow);
		if (whichPart == inMenuBar && theWindow != gTextWindow)
			theWindow = FrontWindow();
		AdjustMenus(theWindow);
		switch (whichPart) {
		case inDesk:
			SysBeep(10);
			break;
		case inMenuBar:
			InitCursor();
			DoCommand(MenuSelect(EVENT.where));
			break;
		case inDrag:			/* TextWindow handled these	*/
		case inContent:
		case inGoAway:
			SysBeep(10);
			break;
		}
}

void
DoCommand(
		long					menuChoice
	)
{
		short				menuItem;
		Str255				menuText;
		GrafPtr				savePort;
		
		menuItem = LoWord(menuChoice);
		switch (HiWord(menuChoice)) {
		case MENU_Apple:
			if (menuItem != kAppleAbout) {
				GetItem(gAppleMenu, menuItem, menuText);
				AdjustEditMenu(TRUE);
				GetPort(&savePort);
				OpenDeskAcc(menuText);
				SetPort(savePort);
				AdjustEditMenu(FrontWindow() != gTextWindow);
			}
			break;
		case MENU_File:
			/*
			 * Note that File/Measure and File/Adjust are enabled
			 * only if the application is in the idle state.
			 */
			switch (menuItem) {
			case kFileSaveAs:
				if (gFileRefNum == 0) {
					TRY {
						PromptAndCreateAuditOutputFile(
							"\pAudit Output File",
							"\pAudit Log",
							'ttxt',
							&gSFReply,
							&gFileRefNum
						);
					}
					CATCH {
						ErrorAlert(STATUS, MESSAGE, FALSE);
						AuditStatusString(
							gAuditPtr,
							'APPL',
							STATUS,
							gErrorMessage[MESSAGE]
						);
						gFileRefNum = 0;
						NO_PROPAGATE;
					}
					ENDTRY;
				}
				AdjustMenus(FrontWindow());
				break;
			case kFileClose:
				if (gFileRefNum != 0) {
					TRY {
						CloseAuditOutputFile(
							noErr,
							&gSFReply,
							gFileRefNum
						);
					}
					CATCH {
						ErrorAlert(STATUS, MESSAGE, FALSE);
						AuditStatusString(
							gAuditPtr,
							'APPL',
							STATUS,
							gErrorMessage[MESSAGE]
						);
						NO_PROPAGATE;
					}
					ENDTRY;
					gFileRefNum = 0;
				}
				AdjustMenus(FrontWindow());
				break;
			case kFileQuit:
				gQuitNow = TRUE;
				break;
			default:
				SysBeep(10);
				break;
			}
			break;
		case MENU_Edit:
			if (SystemEdit(menuItem - 1) == FALSE)
				SysBeep(10);
			break;
		}
		HiliteMenu(0);
}		

void
AdjustMenus(
		WindowPtr				theWindow
	)
{
		EnableItem(gFileMenu, kFileQuit);
		if (gFileRefNum == 0) {
			EnableItem(gFileMenu, kFileSaveAs);
			DisableItem(gFileMenu, kFileClose);
		}
		else {
			EnableItem(gFileMenu, kFileClose);
			DisableItem(gFileMenu, kFileSaveAs);
		}
		CheckItem(gFileMenu, kFileSaveAs, gFileRefNum != 0);
		AdjustEditMenu(theWindow != gTextWindow);
}

void
AdjustEditMenu(
		Boolean				isDeskAcc
	)
{
		if (isDeskAcc) {
			EnableItem(gEditMenu, kEditUndo);
			EnableItem(gEditMenu, kEditCut);
			EnableItem(gEditMenu, kEditCopy);
			EnableItem(gEditMenu, kEditPaste);
			EnableItem(gEditMenu, kEditClear);
		}
		else {
			DisableItem(gEditMenu, kEditUndo);
			DisableItem(gEditMenu, kEditCut);
			DisableItem(gEditMenu, kEditCopy);
			DisableItem(gEditMenu, kEditPaste);
			DisableItem(gEditMenu, kEditClear);
		}
}

void
InitializeApplication(void)
{
		int				i;
		Boolean			result;
		
		result = FALSE;
		InitGraf(&qd.thePort);
		InitFonts();
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(NULL);
		SetCursor(*GetCursor(watchCursor));
		for (i = 0; i < 3; i++)
			EventAvail(everyEvent, &EVENT);
		gAppleMenu = NewMenu(MENU_Apple, "\p\024");
		FailNIL(gAppleMenu, kErrInitialization);
		AppendMenu(gAppleMenu, "\p( 1992-93 Apple Computer Inc.;(-");
		AddResMenu(gAppleMenu, 'DRVR');
		InsertMenu(gAppleMenu, 0);
		gFileMenu = NewMenu(MENU_File, "\pFile");
		FailNIL(gFileMenu, kErrInitialization);
		AppendMenu(gFileMenu, kFileMenuString);
		InsertMenu(gFileMenu, 0);
		gEditMenu = NewMenu(MENU_Edit, "\pEdit");
		FailNIL(gEditMenu, kErrInitialization);
		AppendMenu(gEditMenu, kEditMenuString);
		InsertMenu(gEditMenu, 0);
		DrawMenuBar();
		OpenAuditParameterFile();
		ReadAuditParameters();
		InitCursor();
}

void
InitializeTextWindow(void)
{
		Str255			appName;
		short			resRefNum;
		Handle			hParms;
		Rect			boundsRect;
		
		SetRect(
			&boundsRect,
			PARM.topLeft.h,
			PARM.topLeft.v,
			PARM.topLeft.h + PARM.windowSize.h,
			PARM.topLeft.v + PARM.windowSize.v
		);
		GetAppParms(appName, &resRefNum, &hParms);
		gTextWindow = NewTextWindow(
					appName,
					&boundsRect,
					FALSE,				/* No goAway	*/
					applFont,			/* Font			*/
					9					/* Font size	*/
				);
		FailNIL(gTextWindow, kErrCreateTextWindow);
}

void
InitializeAudit(void)
{
		Boolean					saveAuditEnable;
		Boolean					alreadyCreated;
		StringPtr				message;
		OSType					newAuditIdent;
		
		newAuditIdent = PARM.auditIdent;
		AuditPutFile(
			DLOG_SFPutFile,
			"\pAudit Log File",
			"\pAudit Log",
			&gSFReply,
			&newAuditIdent,
			&gEnableAudit
		);
		if (newAuditIdent != PARM.auditIdent) {
			PARM.auditIdent = newAuditIdent;
			gParameterUpdateNeeded = TRUE;
		}
		if (gSFReply.good == FALSE)
			gFileRefNum = 0;
		else {
			(void) CreateOutputFile(
					'ttxt',
					gSFReply.fName,
					gSFReply.vRefNum,
					&gFileRefNum
				);
		}
		gAuditPtr = GetAuditPtr(PARM.auditIdent);
		alreadyCreated = (gAuditPtr != NULL);
		if (alreadyCreated)
			message = "\pgAuditPtr was previously created";
		else {
			gAuditPtr = InitAudit(PARM.auditIdent, 64, gEnableAudit, FALSE);
			message = "\pgAuditPtr initialized";
		}
		GetCurrentProcess(&gOldPSN);
		WakeUpAudit(gAuditPtr, &gOldPSN);
		saveAuditEnable = EnableAudit(gAuditPtr, TRUE);
		Audit(
			gAuditPtr,
			'APPL',
			AuditFormat3(
				kAuditFormatHex,
				kAuditFormatAddress,
				kAuditFormatString
			),
			PARM.auditIdent,
			gAuditPtr,
			message			
		);
		EnableAudit(gAuditPtr, saveAuditEnable);
#if TESTING
		AuditString(
			gAuditPtr,
			'Moof',
			"\p1234567890123456789012345678901234567890"
		);
		AuditStatusString(gAuditPtr, 'Moof', 1, "\pHello world");
		AuditStatusLocation(gAuditPtr, 'Moof', 2);
		TestAuditStatusLocationUsingAnExtremelyLongFunctionName();
		TestPascalVoidFunction();
		TestPascalShortFunction();
		TestPascalFunctionWithParameters(NULL, NULL);
#endif
}

/*
 * This dequeues a bunch of events from the audit log.
 */
void
ProcessAudit(void)
{
		register int			i;
		
		for (i = 0; i < kAuditCycles; i++) {
			if (ReadAudit(gAuditPtr, &gCurrentLogEntry) == FALSE)
				break;
			else {
				DisplayLogEntry();
			}
		}
}

void
DisplayLogEntry(void)
{
		Str255					message;
		Str255					work;
		short					i;
		
		if (ENTRY.lostData > 0) {
			gTotalMissedCount += ENTRY.lostData;
			NumToString(ENTRY.lostData, message);
			pstrcat(message, "\p log entries(");
			NumToString(gTotalMissedCount, work);
			pstrcat(message, work);
			pstrcat(message, "\p total) missed...");
			DisplayString(gTextWindow, message);
			gLogIndex += ENTRY.lostData;
		}
		gLogIndex += 1;
		message[0] = 0;
		NumToString(gLogIndex, work);
		for (i = work[0]; i < 3; i++)
			pstrcat(message, "\p ");
		pstrcat(message, work);
		pstrcat(message, "\p ");
		FormatAuditEntryTimestamp(gAuditPtr, &ENTRY, work);
		/*
		 * Since we know that the timestamp has a fixed-length
		 * format, we can chop out the date to gain some space
		 * in the text window. Note the format:
		 *		0        1         2
		 *		12345678901234567890123
		 *		1993.01.14 22:36:57.123
		 */
		work[11] = work[0] - 11;
		pstrcat(message, &work[11]);
		pstrcat(message, "\p ");
		FormatAuditEntryData(&ENTRY, work);
		pstrcat(message, work);
		DisplayString(gTextWindow, message);
		if (gFileRefNum != 0) {
			TRY {
				WriteAuditOutputLine(message, gFileRefNum);
			}
			CATCH {
				ErrorAlert(STATUS, MESSAGE, FALSE);
				AuditStatusString(
					gAuditPtr,
					'APPL',
					STATUS,
					gErrorMessage[MESSAGE]
				);
				CloseAuditOutputFile(STATUS, &gSFReply, gFileRefNum);
				gFileRefNum = 0;
				NO_PROPAGATE;
			}
			ENDTRY;
		}
}

/*
 * This displays an error message and its accompaning text.
 * It should be redone to use a message index (to an STR#
 * resource.
 */
void	
ErrorAlert(
		OSErr			status,
		short			messageIndex,
		Boolean			fatal
	)
{
		Str15			statusText;
		short			alertID;
		
		NumToString(status, statusText);
		alertID = (fatal) ? ALRT_Fatal : ALRT_NonFatal;
		ParamText(statusText, gErrorMessage[messageIndex], "\p", "\p");
		StopAlert(alertID, NULL);
}

#if TESTING
#define AuditLocation(auditPtr, ident) (		\
		Audit(									\
			(auditPtr),							\
			(ident),							\
			AuditFormat1(kAuditFormatLocation)	\
		)										\
	)

void
TestAuditStatusLocationUsingAnExtremelyLongFunctionName(void) {
		AuditLocation(gAuditPtr, 'Moof');
}

pascal void
TestPascalVoidFunction(void)
{
		AuditLocation(gAuditPtr, 'Moof');
}

pascal short
TestPascalShortFunction(void)
{
		AuditLocation(gAuditPtr, 'Moof');
		return (123);
}
pascal OSErr
TestPascalFunctionWithParameters(
		Ptr						fooPtr,
		Ptr						barPtr
	)
{
		AuditLocation(gAuditPtr, 'Moof');
		ErrorAlert(noErr, kNoMessage, FALSE);
		ErrorAlert(noErr, kNoMessage, TRUE);
		return ((fooPtr == barPtr) ? noErr : paramErr);
}
#endif

