/*
 * DisplayAudit.c
 * Copyright  1992-93, Apple Computer Inc. All Rights Reserved.
 * Programmed by Martin Minow,
 *	Internet:	minow@apple.com
 *	AppleLink:	MINOW
 * Version of January 22, 1993
 *
 * Edit History
 *	93.01.09 MM		First public release
 *	93.01.22 MM		Broke out from DisplayAudit.c
 */

#include "Audit.h"
#include "Failure.h"
#include <Processes.h>
#include <Errors.h>
#include <Finder.h>
#include <Folders.h>
#include <GestaltEqu.h>
#include <Script.h>
#include <Types.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Packages.h>
#include <Traps.h>

#ifndef TRUE
#define TRUE					1
#define FALSE					0
#endif

/*
 * Resource (etc.) definitions
 */
#define ALRT_NonFatal		9000
#define ALRT_Fatal			9001
#define MENU_Apple			1
#define MENU_File			256
#define	MENU_Edit			257
#define DLOG_SFPutFile		4000

/*
 * Error alert messages. These should be moved to
 * an STR# resource.
 */
enum {
	kNoMessage					= 0,
	kErrInitialization,
	kErrNoMemory,
	kErrCreateAuditRecord,
	kErrCreateTextWindow,
	kErrSystemVersion,
	kErrFindPreferenceFolder,
	kErrFindPreferenceFile,
	kErrOpenPreferenceFile,
	kErrCreatePreferenceFile,
	kErrWritePreferenceResource,
	kErrReadPreferenceResource,
	kErrCreateOutputFile,
	kErrWriteOutputFile,
	kErrCloseOutputFile,

	kDummyLastMessage
};
	

/*
 * Preference resource file information.
 * kApplicationCreator	is the registered Application Creator ID.
 * kDefaultAuditIdent	is the registered Application Creator ID
 *						for the sample DSAM - I used this because
 *						it's convenient.
 */
#define kApplicationCreator		'MAM1'	/* DTS-registered		*/
#define kDefaultAuditIdent		'Pdsm'	/* kDSAMCreatorID		*/
#define kPreferenceFileType		'pref'	/* Preference fileType	*/
#define PARM_ResourceID			1024
#define STR_FinderIDString		(-16397)
#define kNoParamFile			(-1)

typedef struct {
	OSType			auditIdent;
	Point			topLeft;
	Point			windowSize;
} Parameters;

extern WindowPtr	gTextWindow;
extern EventRecord	gEventRecord;
#define	EVENT		(gEventRecord)
extern Parameters	gParameterRecord;
#define PARM		(gParameterRecord)
extern short		gParameterFileRefNum;
extern Boolean		gParameterUpdateNeeded;
extern SFReply		gSFReply;
extern short		gFileRefNum;				/* Non-zero if output on	*/

/*
 * AuditFileDialog.c
 */
void						AuditPutFile(
		short					dialogID,
		const StringPtr			promptString,
		const StringPtr			originalName,
		SFReply					*reply,
		OSType					*auditSelector,
		Boolean					*enableAudit
	);

/*
 * DisplayAuditFile.c
 */
/*
 * File functions
 */
void				PromptAndCreateAuditOutputFile(
		StringPtr				promptString,
		StringPtr				defaultFileName,
		OSType					creator,
		SFReply					*reply,
		short					*fileRefNum
	);
OSErr				CreateOutputFile(
		OSType					creator,
		StringPtr				fileName,
		short					vRefNum,
		short					*fileRefNum
	);
void				WriteAuditOutputLine(
		const StringPtr			theText,
		short					fileRefNum
	);
void				CloseAuditOutputFile(
		OSErr				status,
		SFReply				*reply,
		short				fileRefNum
	);


/*
 * DisplayAuditPreferences.c
 */
void				OpenAuditParameterFile(void);
void				ReadAuditParameters(void);
void				WriteAuditParameters(void);
void				ParameterFactorySettings(void);
OSErr				SearchForCreatorType(
		short			prefVRefNum,
		long			prefDirID,
		Str63			trueFileName
	);

void				ErrorAlert(
		OSErr			status,
		short			messageIndex,
		Boolean			fatal
	);

#define height(box)				(box.bottom - box.top)
#define width(box)				(box.right - box.left)
/*
 * Cheap 'n dirty pascal string copy routine.
 */
#define pstrcpy(dst, src) do {							\
		StringPtr	_src = (src);						\
		BlockMove(_src, dst, _src[0] + 1);				\
	} while (0)
/*
 * Cheap 'n dirty pascal string concat.
 */
#define pstrcat(dst, src) do {							\
		StringPtr		_dst = (dst);					\
		StringPtr		_src = (src);					\
		short			_len;							\
		_len = 255 - _dst[0];							\
		if (_len > _src[0]) _len = _src[0];				\
		BlockMove(&_src[1], &_dst[1] + _dst[0], _len);	\
		_dst[0] += _len;								\
	} while (0)
/*
 * Cheap 'n dirty memory clear routine.
 */
#define CLEAR(record) do {								\
		register char	*ptr = (char *) &record;		\
		register long	size;							\
		for (size = sizeof record; size > 0; --size)	\
			*ptr++ = 0;									\
	} while (0)

