/*
 * DisplayAuditFile.c
 * Copyright  1992-93, Apple Computer Inc. All Rights Reserved.
 * Output file writer for the Audit library.
 * Edit History
 *	93.01.14 MM		Removed Think C stdio dependencies. Added
 *					Standard File dialog with additional dialog
 *					items to allow defining the audit selector.
 *	93.01.19 MM		MPW \r is the wrong character. Force <return>
 *					on both MPW and Think, because this is what
 *					the Mac file system requires for end of line.
 *					(Change is in DisplayAuditFile.c)
 *
 */
#include "DisplayAudit.h"
#define kEndOfLine				0x0D	/* Return			*/


/*
 * File Output
 * Open/create a file to write the audit information.
 * Errors fail via the Failure mechanism. Otherwise,
 * reply.good is TRUE on success, FALSE if the user
 * cancels the SFPutFile.
 * Serious errors fail via the failure mechanism.
 * The file is TEXT type.
 * reply.vRefNum is the volume id.
 * fileRefNum gets the file id on success.
 */
void
PromptAndCreateAuditOutputFile(
		StringPtr				promptString,
		StringPtr				defaultFileName,
		OSType					creator,
		SFReply					*reply,
		short					*fileRefNum
	)
{
		Point					where;
		DialogTHndl				dialog;
		Rect					box;
		
		/*
		 * Center the dialog
		 */
		dialog = (DialogTHndl) GetResource('DLOG', putDlgID);
		if (dialog == NULL)								/* No resource!			*/
		    SetPt(&where, 80, 80);
		else {
		    box = (**dialog).boundsRect;				/* Dialog shape			*/
		    ReleaseResource((Handle) dialog);
			where.h =
				(width(qd.thePort->portRect) - width(box)) / 2;
			where.v = 
				((height(qd.thePort->portRect) - GetMBarHeight()) / 3)
				+ GetMBarHeight();
		}
		SFPutFile(where, promptString, defaultFileName, NULL, reply);
		if (reply->good == FALSE)
			*fileRefNum = 0;
		else {
			FailOSErr(
				CreateOutputFile(creator, reply->fName, reply->vRefNum, fileRefNum),
				kErrCreateOutputFile
			);
		}
}

/*
 * Create an output file. Return noErr on success.
 * On success, fileRefNum has the refNum. On Failure, it has zero.
 */
OSErr
CreateOutputFile(
		OSType					creator,
		StringPtr				fileName,
		short					vRefNum,
		short					*fileRefNum
	)
{
		OSErr			status;
		
		/*
		 * Create the file, elmininating any duplicate.
		 */
		SetCursor(*GetCursor(watchCursor));
		status = Create(fileName, vRefNum, creator, 'TEXT');
		if (status == dupFNErr)	 {					/* Exists already?		*/
			status = FSDelete(fileName, vRefNum);
			if (status == noErr)
				status = Create(fileName, vRefNum, creator, 'TEXT');
		}
		if (status == noErr)
			status = FSOpen(fileName, vRefNum, fileRefNum);
		if (status != noErr)
			*fileRefNum = 0;
		InitCursor();
		return (status);
}

void
WriteAuditOutputLine(
		const StringPtr			theText,
		short					fileRefNum
	)
{
		long				textLength;
		static char			gEndOfLine[1] = { kEndOfLine };

		textLength = theText[0];
		if (textLength > 0)
			FailOSErr(FSWrite(fileRefNum, &textLength, &theText[1]), kErrWriteOutputFile);
		textLength = 1;
		FailOSErr(FSWrite(fileRefNum, &textLength, gEndOfLine), kErrWriteOutputFile);
}

/*
 * Close the Audit output file. If status == noErr, the file is closed
 * normally, Otherwise, the program cleans up after an error. Errors
 * return via the Failure mechanism. (Note: this includes a call with
 * status != noErr.) The I/O model is as follows:
 *		TRY {
 *			PromptAndCreateAuditOutputFile(...);
 *			WriteAuditOutputLine(...);
 *			CloseAuditOutputFile(noErr, ...);
 *		}
 *		CATCH {
 *			CloseAuditOutputFile(STATUS, ...);
 *		}
 *		ENDTRY;
 */
void
CloseAuditOutputFile(
		OSErr				status,
		SFReply				*reply,
		short				fileRefNum
	)
{
		if (status == noErr) {
			TRY {
				FailOSErr(FSClose(fileRefNum), kNoMessage);
				FailOSErr(FlushVol(NULL, reply->vRefNum), kNoMessage);
			}
			CATCH {
				status = STATUS;
				NO_PROPAGATE;
			}
			ENDTRY;
		}
		if (status != noErr)
			(void) FSDelete(reply->fName, reply->vRefNum);
		FailOSErr(status, kErrCloseOutputFile);
}
