/*
 * DisplayAudit.c
 * Copyright  1992-93, Apple Computer Inc. All Rights Reserved.
 * Programmed by Martin Minow,
 *	Internet:	minow@apple.com
 *	AppleLink:	MINOW
 * Version of January 14, 1993
 */
#include "DisplayAudit.h"
/*
 * Yes, I know, these should be in the application resource file.
 */
#define kAuditParameterFileName	"\pDisplayAudit Preferences"
static unsigned char	*gAuditParameterFinderString =
	"\pThis document describes user preferences for the DisplayAudit"
	" application. It cannot be opened or printed."
	" To be effective, this document must be stored in the Preferences"
	" folder inside the System Folder.";

/*
 * Open an existing AuditParameter file, or create a new one.
 */
void
OpenAuditParameterFile(void)
{
		OSErr			status;
		long			gestaltResult;
		short			prefVRefNum;
		long			prefDirID;
		FSSpec			prefFSSpec;			/* IM-VI 25-15, 25-30		*/
		Str63			trueFileName;
		Boolean			newResourceFile;
		Handle			handle;				/* For finder info string	*/
		
		newResourceFile = FALSE;
		handle = NULL;
		ParameterFactorySettings();
		TRY {
			FailOSErr(
				Gestalt(gestaltFindFolderAttr, &gestaltResult),
				kErrSystemVersion
			);
			FailOSErr(
				FindFolder(
					kOnSystemDisk,
					kPreferencesFolderType,
					kCreateFolder,
					&prefVRefNum,
					&prefDirID
				),
				kErrFindPreferenceFolder
			);
			status = SearchForCreatorType(
						prefVRefNum,
						prefDirID,
						trueFileName
					);
			switch (status) {
			case noErr:
				/*
				 * The preference file already exists.
				 */
				status = FSMakeFSSpec(
							prefVRefNum,
							prefDirID,
							trueFileName,
							&prefFSSpec
						);
				FailOSErr(status, kErrOpenPreferenceFile);
				break;
			case fnfErr:
				/*
				 * The preferences file did not exist. Create it.
				 * FSMakeFSSpec will fail with a fnfErr. We're in
				 * trouble if it succeeds, as someone else created
				 * a file with the same name and a different owner type.
				 */
				status = FSMakeFSSpec(
							prefVRefNum,
							prefDirID,
							kAuditParameterFileName,
							&prefFSSpec
						);
				if (status == noErr)
					status = dupFNErr;
				else if (status == fnfErr) {
					FSpCreateResFile(
						&prefFSSpec,
						kApplicationCreator,
						kPreferenceFileType,
						smRoman
					);
					status = ResError();
					newResourceFile = TRUE;
				}
				FailOSErr(status, kErrCreatePreferenceFile);
				break;
			default:
				FailOSErr(status, kErrFindPreferenceFile);
				break;
			}
			/*
			 * The preference file existed, or could be created.
			 * This returns (-1) == kNoParamFile on error.
			 */
			gParameterFileRefNum = FSpOpenResFile(&prefFSSpec, fsRdWrPerm);
			FailOSErr(ResError(), kErrOpenPreferenceFile);
			if (newResourceFile) {
				handle = NewHandle(gAuditParameterFinderString[0] + 1);
				FailNIL(handle, kErrNoMemory);
				BlockMove(
					gAuditParameterFinderString,
					*handle,
					gAuditParameterFinderString[0] + 1
				);
				AddResource(handle, 'STR ', STR_FinderIDString, "\p");
				FailOSErr(ResError(), kErrWritePreferenceResource);
				WriteResource(handle);
				FailOSErr(ResError(), kErrWritePreferenceResource);
				ReleaseResource(handle);
				FailOSErr(ResError(), kErrWritePreferenceResource);
				handle = NULL;
				UpdateResFile(gParameterFileRefNum);
				FailOSErr(ResError(), kErrWritePreferenceResource);
			}
		}
		CATCH {
			ErrorAlert(STATUS, MESSAGE, FALSE);
			gParameterFileRefNum = kNoParamFile;
			if (handle != NULL)
				DisposHandle(handle);
			NO_PROPAGATE;
		}
		ENDTRY;
}

void
ReadAuditParameters(void)
{
		short			oldResFile;
		Handle			parmHandle;
		Size			parmSize;
		
		if (gParameterFileRefNum != kNoParamFile) {
			oldResFile = CurResFile();
			TRY {
				UseResFile(gParameterFileRefNum);
				FailOSErr(ResError(), kErrReadPreferenceResource);
				parmHandle = GetResource(kApplicationCreator, PARM_ResourceID);
				if (parmHandle == NULL) {
					/*
					 * Write the factory settings.
					 */
					gParameterUpdateNeeded = TRUE;
				}
				else {
					CLEAR(PARM);
					parmSize = GetHandleSize(parmHandle);
					if (parmSize > sizeof (PARM))
						parmSize = sizeof (PARM);
					BlockMove(*parmHandle, &PARM, parmSize);
					if (parmSize != sizeof (PARM))
						gParameterUpdateNeeded = TRUE;
					ReleaseResource(parmHandle);
					FailOSErr(ResError(), kErrReadPreferenceResource);
				}
			}
			CATCH {
				ErrorAlert(STATUS, MESSAGE, FALSE);
				CloseResFile(gParameterFileRefNum);
				gParameterFileRefNum = kNoParamFile;
				NO_PROPAGATE;
			}
			ENDTRY;
		}
}

void
WriteAuditParameters(void)
{
		short				oldResFile;
		Handle				parmHandle;
		
		if (gParameterFileRefNum != kNoParamFile) {
			oldResFile = CurResFile();
			parmHandle = NULL;
			TRY {
				UseResFile(gParameterFileRefNum);
				FailOSErr(ResError(), kErrWritePreferenceResource);
				parmHandle = GetResource(
						kApplicationCreator,
						PARM_ResourceID
					);
				if (parmHandle != NULL) {
					/*
					 * Update an existing parameter resource
					 */
					FailOSErr(MemError(), kErrWritePreferenceResource);
					SetHandleSize(parmHandle, sizeof PARM);
					FailOSErr(MemError(), kErrWritePreferenceResource);
					BlockMove(&PARM, *parmHandle, sizeof PARM);
					ChangedResource(parmHandle);
					FailOSErr(ResError(), kErrWritePreferenceResource);
				}
				else {
					/*
					 * Create a new parameter resource
					 */
					parmHandle = NewHandle(sizeof PARM);
					FailNIL(parmHandle, kErrWritePreferenceResource);
					BlockMove(&PARM, *parmHandle, sizeof PARM);
					AddResource(
						parmHandle,
						kApplicationCreator,
						PARM_ResourceID,
						"\pParameters"
					);
					FailOSErr(ResError(), kErrWritePreferenceResource);
				}
				WriteResource(parmHandle);
				FailOSErr(ResError(), kErrWritePreferenceResource);
				ReleaseResource(parmHandle);
				parmHandle = NULL;
				FailOSErr(ResError(), kErrWritePreferenceResource);
				UpdateResFile(gParameterFileRefNum);
				FailOSErr(ResError(), kErrWritePreferenceResource);
			}
			CATCH {
				ErrorAlert(STATUS, MESSAGE, FALSE);
				if (parmHandle != NULL)
					ReleaseResource(parmHandle);
				CloseResFile(gParameterFileRefNum);
				gParameterFileRefNum = kNoParamFile;
				NO_PROPAGATE;
			}
			ENDTRY;
			UseResFile(oldResFile);
			FailOSErr(ResError(), kErrWritePreferenceResource);
		}
		gParameterUpdateNeeded = FALSE;
}

/*
 * Initialize the parameter array. The values will be replaced
 * by the current preferences values, if any are read. If we
 * can't read a preferences resource, these values (or whatever
 * the user chooses) will be written.
 */
void
ParameterFactorySettings(void)
{
		PARM.auditIdent = kDefaultAuditIdent;
		PARM.topLeft.v =
			qd.screenBits.bounds.top + (GetMBarHeight() * 2) + 2;
		PARM.windowSize.v =
			qd.screenBits.bounds.bottom - PARM.topLeft.v - 2;
		PARM.windowSize.h = width(qd.screenBits.bounds) / 2;
		PARM.topLeft.h =
			qd.screenBits.bounds.right - PARM.windowSize.h - 2;
}

/*
 * This searches the preference file folder for files
 * with a specific creator and file type. This lets
 * the user rename our preference file without harm.
 */
OSErr
SearchForCreatorType(
		short			prefVRefNum,
		long			prefDirID,
		StringPtr		trueFileName
	)
{
		CInfoPBRec			specPBRec;
		OSErr				status;
		register short		i;
#define SPEC	(specPBRec.hFileInfo)

		CLEAR(specPBRec);
		SPEC.ioNamePtr = trueFileName;
		SPEC.ioVRefNum = prefVRefNum;
		for (i = 1;; i++) {
			SPEC.ioDirID = prefDirID;
			SPEC.ioFDirIndex = i;
			status = PBGetCatInfo(&specPBRec, FALSE);
			if (status != noErr)
				break;
			if (SPEC.ioFlFndrInfo.fdType == kPreferenceFileType
			 && SPEC.ioFlFndrInfo.fdCreator == kApplicationCreator)
				break;
		}
		return (status);
#undef SPEC
}

