/*
 * DTSSampleCSAMDriver.r
 * Copyright  1992-93 Apple Computer Inc. All Rights Reserved.
 *
 * This file contains the driver-specific resources for the
 * DTS Sample CSAM. It is included by DTSSampleCSAM.r
 * It is only needed to build the MPW version of the dsam.
 */

/*
 * These parameters configure the driver:
 *	kDriverNameString		The Device Manager driver name
 *	kDriverCodeFile			Where link (or Think C) stores the driver code resource
 */
#define kDriverNameString			".Sample Catalog"
#define kDriverCodeFile				"::Obj:DTSSampleDRVR.code"	/* MPW only	*/

#if Think_Rez == 0
/*
 * MPWTypes.r contains the hack DRVR resource.
 */
#include "MPWTypes.r"

type 'DRVR' as 'DRVW';			/* Map 'DRVW' => 'DRVR' */
/*
 * This will produce a DRVR resource from the special DRVW type.
 *
 * The leading NUL in the resource name is required to
 * conform to the desk accessory naming convention.
 *
 * The resource is declared purgeable.	If the code were to
 * do funky things like SetTrapAddress calls (requiring the code to
 * be around at all times), we would have to set it nonpurgeable.
 */

#define DriverID	0x0b		/* DriverID is a dummy, unused, value			*/
resource 'DRVR' (DriverID, kDriverNameString, sysheap, locked) 
{
	/*
	 * DRVR flags
	 */
	needLock, dontNeedTime, dontNeedGoodbye, noStatusEnable,
	ctlEnable, noWriteEnable, noReadEnable,
	0,					/* drvrDelay in ticks				*/
	0,					/* drvrEMask						*/
	0,					/* drvrMenu							*/
	kDriverNameString,	/* drvrName (fixed limit 31 chars)	*/
	/*
	 * This directive inserts the contents of the DRVW resource
	 * produced by linking DRVRRuntime.o with our Driver code.
	 * Note: either the Think or MPW CSAM will be created.
	 * This shouldn't result in any difference.
	 */
	$$resource(kDriverCodeFile, 'DRVW', 0)
};
#endif

/*
 * The Driver's name must be in the resource named DashName."
 */
resource 'STR ' (kDriverName, "DashName", purgeable ) {
	kDriverNameString
};

/*
 * The 'sysz' resource extends the system heap at startup, leaving
 * room for the data needed by this System Extension.
 */
data 'sysz' (0, "8K") {
	$"00 00 20 00" 				/* 8K of system heap */
};

