/*----------------------------------------------------------

AOCE Post Office Protocol (3)
Personal Gateway

written by Steven Falkenburg-- MacDTS
1991 Apple Computer, Inc.

----------------------------------------------------------*/

#pragma once
#define __MYTYPES__

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __EPPC_
#include <EPPC.h>
#endif

#ifndef __OCE__
#include <OCE.h>
#endif

#ifndef __OCEMAIL__
#include <OCEMail.h>
#endif

#ifndef __OCEAUTHDIR__
#include <OCEAuthDir.h>
#endif

#ifndef __GW_SPECSLOTS__
#include "pop.slots.h"
#endif

#ifndef __RETRACE__
#include <Retrace.h>
#endif

/*************** vbl helpers ***********************************************/

typedef struct MyVBLTask {
	long myA5;
	VBLTask vbl;
} MyVBLTask;

/*************** high-level event helper-structures ************************/

// HLEventQ:
//
// queue element for high level events that we want to defer until we get back to
// our main event loop

typedef struct HLEventQ {
	struct HLEventQ *next;
	struct HLEventQ *prev;
	long messageID;
	MailEPPCMsg *mailEPPC;
	short slotID;
} HLEventQ;

/*************** message identifier helper-structures ************************/

// MsgIDBlock:
//
// convenience structure for uniquely identifying a message in an MSAM queue

typedef struct MsgIDBlock {
	long			seqNum;					// message sequence #
	MailMsgRef		mnmRefNum;				// id's the gateway's copy of the message
	MSAMQueueRef	qid;					// slot's queue id
} MsgIDBlock, *MsgIDBlockPtr;


/*************** MSAMEnumerate helper-structures ************************/

// EnumInQReply:
//
// this structure is returned by the MSAMEnumerate call (in the data buffer).  It's the
// number of messages enumerated (2 bytes) followed by each message.

typedef struct EnumInQReply {
	MailReply				reply;			// # of messages returned
	MSAMEnumerateInQReply	message;		// message storage
} EnumInQReply, *EnumInQReplyPtr;


// EnumOutQReply:
//
// this structure is returned by the MSAMEnumerate call (in the data buffer).  It's the
// number of messages enumerated (2 bytes) followed by each message.

typedef struct EnumOutQReply {
	MailReply				reply;			// # of messages returned
	MSAMEnumerateOutQReply	message;		// message storage
} EnumOutQReply, *EnumOutQReplyPtr;


/*************** MSAMGetRecipients helper-structures ************************/

// MailOriginalRecipientExt:
//
// each entry in the recipient array returned by MSAMGetRecipients is of the below
// form when getting original recipients.

typedef struct MailOriginalRecipientExt {
	MailOriginalRecipient	ext;
	OCEPackedRecipient		packedRecip;
} MailOriginalRecipientExt;


// MailResolvedRecipientExt:
//
// each entry in the recipient array returned by MSAMGetRecipients is of the below
// form when getting resolved recipients.

typedef struct MailResolvedRecipientExt {
	MailResolvedRecipient	ext;
	OCEPackedRecipient		packedRecip;
} MailResolvedRecipientExt;


// OriginalRecipientReply:
//
// returned by MSAMGetRecipients (in the data buffer).  It's the number of recipients
// returned followed by the array of the receipients.  This structure is for original
// (non-responsible) receipients.

typedef struct OriginalRecipientReply {
	MailReply					reply;			// # of recipients returned
	MailOriginalRecipientExt	recipient[];	// array of recipients
} OriginalRecipientReply, *OriginalRecipientReplyPtr;


// ResolvedRecipientReply:
//
// returned by MSAMGetRecipients (in the data buffer).  It's the number of recipients
// returned followed by the array of the receipients.  This structure is for resolved
// (responsible) receipients.

typedef struct ResolvedRecipientReply {
	MailReply					reply;			// # of recipients returned
	MailResolvedRecipientExt	recipient[];	// array of recipients
} ResolvedRecipientReply, *ResolvedRecipientReplyPtr;


/**************** MSAMGetAttributes helper-structures ***********************/

// MaximumLetterAttributes:
//
// this structure is here to calculate the maximum buffer size needed when
// calling MSAMGetAttributes.  This structure isn't actually used to read the
// attributes, since if one is missing, they will not be aligned in this way.

typedef struct MaximumLetterAttributes {
	MailIndications		indications;
	OCECreatorType		msgType;
	MailLetterID		letterID;
	MailTime			sendTimeStamp;
	MailNestingLevel	nestingLevel;
	OSType				messageFamily;
	MailLetterID		replyID;
	MailLetterID		conversationID;
	RString				subject;
} MaximumLetterAttributes, *MaximumLetterAttributesPtr;


/**************** slot database helper-structures ***********************/

// DirIdentity
//
// this stucture is used to store the name and password for the user's account on
// the external system.  it's filled in only when the local identity is unlocked

typedef struct DirIdentity {
	char userName[64];
	char password[64];
	Boolean valid;
} DirIdentity;


// SlotSpec:
//
// storage area for all information relevant to a particular slot.
// one of these structures is allocated for each slot in use by the system.

typedef struct SlotSpec {
	Boolean							active;			// true if this slot space is in use
	CreationID						slotCID;		// CreationID of slot
	RString							directoryName;	// name of directory
	DirDiscriminator				discriminator;	// directory discriminator
	MailStandardSlotInfoAttribute	stdInfo;		// standard slot info
	SpecificSlotInfo				specInfo;		// specific slot info
	DirIdentity						dirIdentity;	// the external identity from the directory
	MSAMSlotID						slotID;			// slot ID (index)
	MSAMQueueRef					inQueue;		// in queue ref for slot
	MSAMQueueRef					outQueue;		// out queue ref for slot
	unsigned long					lastCheckGet;	// time of last put check (seconds)
	unsigned long					lastCheckPut;	// time of last get check (seconds)
	short							retryPut;		// how many retries have we done?
	short							retryGet;		// how many retries have we done?
	OSErr							retryPutError;	// what error have we been retrying for?
	OSErr							retryGetError;	// what error have we been retrying for?
	Boolean							enabled;		// true if slot is not suspended
} SlotSpec,*SlotSpecPtr;


/*************** directory manager helper-structures **********************/

// AttributeCopyInfo:
//
// this structure is used by the slot database lookup calls to store client
// data for the LookupParse attribute value callback identifying how to copy
// the data out of the attribute
//

typedef struct AttributeCopyInfo {
	Ptr buffer;
	Size bufferSize;
} AttributeCopyInfo,*AttributeCopyInfoPtr;