/*								AOMTypeInManager.c								*/
/*
 * AddressOMatic Sample
 * AOMTypeInManager.c
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 */
#include "AddressOMaticPrivate.h"
#include <OCEAuthDir.h>
#include <OCEMessaging.h>
#include <Script.h>
#include <Packages.h>

#ifdef ENABLE_TYPEIN
/*
 * This module is compiled only if ENABLE_TYPEIN is defined. It does not solve the
 * "entire" type-in problem. In particular, it has only a default, minimal,
 * function that converts a type-in address to a DSSpec; special-case routines,
 * not included, are needed for mail (server, and serverless) and for dial-up.
 * However, this may be sufficient for applications that need to access private
 * directories. It's at least a start.
 */
#define SYNC			0

#define TEXT	(**AOM.typeInAddress)
#define ForgetPtr(ptr) do {						\
		if ((ptr) != NULL)						\
			DisposePtr((Ptr) (ptr));			\
	} while (0)
#define ForgetHandle(handle) do {				\
		if ((handle) != NULL)					\
			DisposeHandle((Handle) (handle));	\
	} while (0)

enum {										/* Keyboard commands	*/
	NUL					= 0,				/* NUL == EOS			*/
	charEnter			= 0x03,				/* Enter key			*/
	charTab				= '\t',				/* Tab key				*/
	charBackspace		= 0x08,				/* Backspace key		*/
	charNewLine			= 0x0A,				/* LineFeed key			*/
	charReturn			= 0x0D,				/* Return key			*/
	charEscape			= 0x1B				/* Stop key				*/
};

OSErr							_AOMMakeTypeInPopup(
		register AddressOMaticPtr	aomPtr
	);
#if 1
OSErr
_AOMConstructDefaultTypeInAddress(
		register AddressOMaticPtr	aomPtr,
		unsigned short				*selectionSize,
		PackedDSSpecPtr				*dsSpec,
		const _AOMDirectoryInfoPtr	directoryInfoPtr,
		const RStringPtr			addressRString
	);
OSErr							_AOMGetExtendedDirectoriesInfo(
		register AddressOMaticPtr	aomPtr
	);
#else
OSErr							_AOMEnumerateDirectories(
		register AddressOMaticPtr	aomPtr
	);
pascal Boolean					_AOMForEachDirectory(
		long						clientData,
		const DirectoryName			*dirName,
		const DirDiscriminator		*discriminator,
		DirGestalt					features
	);
#endif

void							_AOMCleanOutDirInfoHandle(
		register AddressOMaticPtr	aomPtr
	);

RStringPtr						_AOMMakeRStringFromHandle(
		Handle						textHandle,
		short						scriptCode
	);

/*
 * Create the type-in panel. This contains two active elements: the address
 * text edit record and a "Directory" popup control.
 */
OSErr
_AOMTypeInCreate(
		register AddressOMaticPtr	aomPtr
	)
{
		OSErr						status;
		Rect						destRect;
		Rect						viewRect;
		
		status = noErr;
		/*
		 * Make the Text Edit record.
		 */
		_AOMSetFont(aomPtr, kAOMTextFontStyle);		
		viewRect = RECT(kAOMTypeInAddressItem);
		InsetRect(&viewRect, 2, 2);
		destRect = viewRect;
		destRect.bottom = 30000;
		/*
		 * Move the viewRect offscreen.
		 */
		OffsetRect(&viewRect, 4096, 4096);
		AOM.typeInAddress = TENew(&destRect, &viewRect);
		status = _AOMMakeTypeInPopup(aomPtr);
		AOM.typeInVisible = FALSE;
		return (status);
}

OSErr
_AOMTypeInDispose(
		register AddressOMaticPtr	aomPtr
	)
{
		OSErr						status;

		status = noErr;
		if (AOM.typeInAddress != NULL) {
			TEDispose(AOM.typeInAddress);
			AOM.typeInAddress = NULL;
		}
		if (AOM.typeInPopup != NULL) {
			DisposeControl(AOM.typeInPopup);
			AOM.typeInPopup = NULL;
		}
		if (AOM.popupMenu != NULL) {
			DeleteMenu(AOM.popupMenuID);
			DisposeMenu(AOM.popupMenu);
			AOM.popupMenu = NULL;
		}
		if (AOM.dirInfoHdl != NULL) {
			_AOMCleanOutDirInfoHandle(aomPtr);
			DisposeHandle((Handle) AOM.dirInfoHdl);
			AOM.dirInfoHdl = NULL;
		}
		return (status);
}

OSErr
_AOMTypeInEvent(
		register AddressOMaticPtr	aomPtr,
		const EventRecord			*theEvent,
		register AOMPanelState		*whatHappened
	)
{
		OSErr						status;
		unsigned short				theChar;
		Point						mousePt;
		Boolean						isActivating;
		WindowPtr					theWindow;
		ControlHandle				theControl;
		short						partCode;
		AOMSaveState				saveState;
#define EVENT	(*theEvent)

		status = noErr;
		switch (EVENT.what) {
		case nullEvent:
			TEIdle(AOM.typeInAddress);
			break;
		case keyDown:
		case autoKey:
			/*
			 * TBS: handle cut/copy/paste
			 */
			if ((EVENT.modifiers & cmdKey) == 0) {
				theChar = EVENT.message & charCodeMask;
#if 0
				/*
				 * I don't like this. Instead, all control characters
				 * will be passed to the application, except for backspace.
				 */
				switch (theChar) {
				case charTab:
				case charEnter:
				case charNewLine:
				case charReturn:
				case charEscape:
					break;
				default:
					if (AOM.isTarget) {
						_AOMSaveState(aomPtr, kAOMTextFontStyle, &saveState);
						TEKey(theChar, AOM.typeInAddress);
						*whatHappened &= ~kAOMAppMustHandleEventMask;
						AOM.buttonUpdateNeeded = TRUE;
						_AOMRestoreState(&saveState);
					}
					break;
				}
#else
				if (AOM.isTarget
				 && (theChar >= ' '  || theChar == charBackspace)) {
					_AOMSaveState(aomPtr, kAOMTextFontStyle, &saveState);
					TEKey(theChar, AOM.typeInAddress);
					*whatHappened &= ~kAOMAppMustHandleEventMask;
					AOM.buttonUpdateNeeded = TRUE;
					_AOMRestoreState(&saveState);
				}
#endif
			}
			break;
		case mouseDown:
			/*
			 * Note: AOMPanelEvent will call us only if the mouse-down
			 * is in our window.
			 */
			partCode = FindWindow(EVENT.where, &theWindow);
			if (theWindow == AOM.window) {
				_AOMSaveState(aomPtr, kAOMTextFontStyle, &saveState);
				mousePt = EVENT.where;
				GlobalToLocal(&mousePt);
				if (PtInRect(mousePt, &RECT(kAOMTypeInAddressItem))) {
					TEClick(
						mousePt,
						(EVENT.modifiers & shiftKey) != 0,
						AOM.typeInAddress
					);
					AOM.buttonUpdateNeeded = TRUE;
				}
				else {
					partCode = FindControl(mousePt, AOM.window, &theControl);
					if (partCode != 0 && theControl == AOM.typeInPopup) {
						partCode = TrackControl(theControl, mousePt, (ProcPtr) -1);
						/*
						 * We don't care what the value is until the user tries
						 * to get the selection.
						 */
					}
				}
				_AOMRestoreState(&saveState);
			}	
			break;
		case updateEvt:
			_AOMTypeInUpdate(aomPtr, AOM.window->visRgn);
			break;
		case activateEvt:
			theWindow = (WindowPtr) EVENT.message;
			isActivating = (EVENT.modifiers & activeFlag) != 0;
			goto activateEvent;
		case osEvt:
			switch (((unsigned long) EVENT.message) >> 24) {
			case mouseMovedMessage:
				break;
			case suspendResumeMessage:
				isActivating = ((EVENT.message & 0x01) != 0);
				theWindow = FrontWindow();
activateEvent:	if (theWindow == AOM.window) {
					_AOMSaveState(aomPtr, kAOMTextFontStyle, &saveState);
					if (isActivating) {
						TEActivate(AOM.typeInAddress);
						HiliteControl(AOM.typeInPopup, kActiveControl);
					}
					else {
						TEDeactivate(AOM.typeInAddress);
						HiliteControl(AOM.typeInPopup, kInactiveControl);
					}
					_AOMRestoreState(&saveState);
				}
				break;
			}
			break;
		}
		return (status);
#undef EVENT
}

OSErr
_AOMTypeInShow(
		register AddressOMaticPtr	aomPtr,
		Boolean						isShow
	)
{
		OSErr						status;
		AOMSaveState				saveState;
		
		status = noErr;
		if (isShow != AOM.typeInVisible) {
			if (AOM.typeInVisible) {
				OffsetRect(&TEXT.viewRect, 4096, 4096);
				TEDeactivate(AOM.typeInAddress);
				HideControl(AOM.typeInPopup);
			}
			else {
				_AOMSaveState(aomPtr, kAOMTextFontStyle, &saveState);
				OffsetRect(&TEXT.viewRect, -4096, -4096);
				TEActivate(AOM.typeInAddress);
				HiliteControl(AOM.typeInPopup, kActiveControl);
				ShowControl(AOM.typeInPopup);
				_AOMRestoreState(&saveState);
			}
			AOM.typeInVisible = isShow;
			InvalRect(&RECT(kAOMTypeInAddressItem));
		}
		return (status);
}

OSErr
_AOMTypeInUpdate(
		register AddressOMaticPtr	aomPtr,
		RgnHandle					drawRgn
	)
{
		OSErr						status;
		Rect						viewRect;
		RGBColor					saveBackColor;
		Str255						work;
		FontInfo					fontInfo;
#pragma unused (drawRgn)

		status = noErr;
		if (AOM.typeInVisible) {
			GetFontInfo(&fontInfo);
			GetIndString(work, AOM.stringsResID, kAOMTypeInAddrLabelString);
			MoveTo(
				RECT(kAOMTypeInAddrLabelItem).left,
				RECT(kAOMTypeInAddrLabelItem).top + fontInfo.ascent
			);
			DrawString(work);
			GetIndString(work, AOM.stringsResID, kAOMTypeInDirLabelString);
			MoveTo(
				RECT(kAOMTypeInDirLabelItem).left,
				RECT(kAOMTypeInDirLabelItem).top + fontInfo.ascent
			);
			DrawString(work);
			viewRect = RECT(kAOMTypeInAddressItem);
			FrameRect(&viewRect);
			if (IS_COLOR(AOM.window)) {
				GetBackColor(&saveBackColor);
				RGBBackColor(&_AOMWhiteColor);
			}
			InsetRect(&viewRect, 1, 1);
			EraseRect(&viewRect);
			_AOMSetFont(aomPtr, kAOMTextFontStyle);
			TEUpdate(&viewRect, AOM.typeInAddress);
			Draw1Control(AOM.typeInPopup);
			if (IS_COLOR(AOM.window))
				RGBBackColor(&saveBackColor);
		}
		return (status);
}

/*
 * This handles all of the selection extraction in one big lump.
 * Returns kOCENoSuchRecord if there is no selection.
 */
OSErr
_AOMTypeInExtractSelection(
		register AddressOMaticPtr	aomPtr,
		unsigned short				*selectionSize,
		PackedDSSpecPtr				*dsSpec
	)
{
		OSErr						status;
		short						menuID;
		_AOMDirectoryInfoPtr		directoryInfoPtr;
		RStringPtr					addressRString;
#define DIR		(*directoryInfoPtr)

		status = noErr;
		addressRString = NULL;
		/*
		 * Make sure there's a record type.
		 */
		if (AOM.typeInRecordType == NULL
		 || OCEValidRString(AOM.typeInRecordType, kOCERecordType) == FALSE) {
			status = paramErr;
			goto exit;
		}
		menuID = GetCtlValue(AOM.typeInPopup) - 1;
		if (menuID < 0 || menuID >= CountMItems(AOM.popupMenu)) {
			status = paramErr;			/* Bug in program!		*/
			goto exit;
		}
		HLock((Handle) AOM.dirInfoHdl);
		directoryInfoPtr = &(*AOM.dirInfoHdl)[menuID];
		addressRString = _AOMMakeRStringFromHandle(
					TEXT.hText,
					Font2Script(TEXT.txFont)
				);
		if ((status = MemError()) != noErr)
			goto exit;
		/*
		 * Here's where we look at the directory kind to handle special
		 * cases (server-based mail, serverless mail, dialup, possibly
		 * others). The signatures are defined in OCEMessaging.h
		 */
		switch (DIR.entnType) {
		case kOCEalanXtn:		/* Serverless mail		*/
			NOTE("\pCan't handle serverless mail");
			goto doItAnyway;
		case kOCEentnXtn:		/* Server-based mail	*/
			NOTE("\pCan't handle server-based mail");
			goto doItAnyway;
		case kOCEaphnXtn:		/* Dial-up addressing	*/
			NOTE("\pCan't handle dial-up addressing");
			goto doItAnyway;
		default:
doItAnyway:
			status = _AOMConstructDefaultTypeInAddress(
						aomPtr,
						selectionSize,
						dsSpec,
						directoryInfoPtr,
						addressRString
					);
			/*
			 * Debugging:
			 */
			if (status == noErr) {
				if (OCEValidPackedDSSpec(*dsSpec) == FALSE)
					NOTE("\pConstructed an invalid DSSpec");
			}
			break;
 		}
exit:	if (status != noErr) {
			*selectionSize = 0;
			*dsSpec = NULL;
		}
		ForgetPtr(addressRString);
		HUnlock((Handle) AOM.dirInfoHdl);
		return (status);
#undef DIR
}

/*
 * This is the constructor for default (unknown) addressing types.
 */
OSErr
_AOMConstructDefaultTypeInAddress(
		register AddressOMaticPtr	aomPtr,
		unsigned short				*selectionSize,
		PackedDSSpecPtr				*dsSpec,
		const _AOMDirectoryInfoPtr	directoryInfoPtr,
		const RStringPtr			addressRString
	)
{
		OSErr						status;
		OCERecipient				oceRecipient;
		RecordID					recordID;
		unsigned short				length;
		OCEPackedRecipient			*packedRecip;
		RStringPtr					extensionValue;
#define DIR		(*directoryInfoPtr)
#define RECIP	(oceRecipient)
		
		extensionValue = NULL;
		packedRecip = NULL;
		OCESetCreationIDtoNull(&recordID.local.cid);
		recordID.local.recordType = AOM.typeInRecordType;
		recordID.local.recordName = addressRString;
		/*
		 * I'm confused: this appears to duplicate addressRString.
		 */
		extensionValue = _AOMMakeRStringFromHandle(
					TEXT.hText,
					Font2Script(TEXT.txFont)
				);
		if ((status = MemError()) != noErr)
			goto exit;
		/*
		 * Construct the recipient
		 */
		RECIP.extensionType = DIR.entnType;
		RECIP.extensionSize = extensionValue->dataLength + sizeof (ProtoRString);
		RECIP.extensionValue = (Ptr) extensionValue;
		RECIP.entitySpecifier = &recordID;
		HLock((Handle) DIR.packedRLIHandle);
		recordID.rli = *(DIR.packedRLIHandle);
		length = OCESizePackedRecipient(&oceRecipient);
		packedRecip = (OCEPackedRecipient *) NewPtr(length);
		if ((status = MemError()) != noErr)
			goto exit;
		length = OCEPackRecipient(&oceRecipient, packedRecip);
		HUnlock((Handle) DIR.packedRLIHandle);
		*dsSpec = (PackedDSSpecPtr) packedRecip;
		*selectionSize = length;
		packedRecip = NULL;				/* We don't want to dispose it!		*/
		status = noErr;					/* Success, at long last			*/
exit:	ForgetPtr(packedRecip);			/* Disposes only on error			*/
		ForgetPtr(extensionValue);
		return (status);
#undef DIR
#undef RECIP
}

void
_AOMTypeInAdjust(
		register AddressOMaticPtr	aomPtr
	)
{
		Rect						viewRect;
		Rect						destRect;
		
		viewRect = RECT(kAOMTypeInAddressItem);
		InsetRect(&viewRect, 2, 2);
		destRect = viewRect;
		destRect.bottom = 30000;
		if (AOM.typeInAddress != NULL) {
			TEXT.destRect = destRect;
			TEXT.viewRect = viewRect;
			TECalText(AOM.typeInAddress);
		}
		if (AOM.typeInPopup != NULL) {
			if (AOM.typeInVisible)
				HideControl(AOM.typeInPopup);
			MoveControl(
				AOM.typeInPopup,
				RECT(kAOMTypeInDirectoryItem).left,
				RECT(kAOMTypeInDirectoryItem).top
			);
			SizeControl(
				AOM.typeInPopup,
				width(RECT(kAOMTypeInDirectoryItem)),
				height(RECT(kAOMTypeInDirectoryItem))
			);
			if (AOM.typeInVisible)
				ShowControl(AOM.typeInPopup);
		}
}

/*
 * In order to create a popup menu, we must create a new menu that contains the
 * catalogs that have been installed on this system.
 */
OSErr
_AOMMakeTypeInPopup(
		register AddressOMaticPtr	aomPtr
	)
{
		OSErr						status;
		static short				nextMenuID = kAOMPrivateMenuID;
		
		AOM.popupMenuID = nextMenuID++;
		AOM.popupMenu = NewMenu(AOM.popupMenuID, "\pCatalogs");
		if (AOM.popupMenu == NULL)
			status = MemError();
		else {
			InsertMenu(AOM.popupMenu, -1);			/* Insert this popup		*/
#if 1
			status = _AOMGetExtendedDirectoriesInfo(aomPtr);
#else
			status = _AOMEnumerateDirectories(aomPtr);
#endif
			if (status != noErr) {
				DeleteMenu(AOM.popupMenuID);
				DisposeMenu(AOM.popupMenu);
				AOM.popupMenu = NULL;
			}
		}
		LOG(status, "\p_AOMEnumerateDirectories");
		if (status == noErr) {
			_AOMSetFont(aomPtr, kAOMTextFontStyle);
			/*
			 * Note that we don't draw the popup title here. If we did so, it
			 * would be drawn with a white background, rather than the soothing
			 * gray that decorates the rest of the panel.
			 */
			AOM.typeInPopup = NewControl(
					AOM.window,						/* Display window			*/
					&RECT(kAOMTypeInDirectoryItem),	/* Where it goes 			*/
					"\p",							/* No popup title			*/
					FALSE,							/* Invisible				*/
					( popupTitleCenterJust			/* Value:	title adjust	*/
					 | popupTitleNoStyle			/*			title style		*/
					),
					AOM.popupMenuID,				/* Min:		menu id			*/			
					0,								/* Max:		popup width		*/
					( popupMenuProc					/* Popup control proc		*/
					 | popupFixedWidth				/* Force control width		*/
					 | popupUseWFont				/* Draw in window font		*/
					),
					0								/* RefCon (unused)			*/
				);
			if (AOM.typeInPopup == NULL)
				status = MemError();
			LOG(status, "\p_AOMMakeTypeInPopup NewControl");
		}
			
		return (status);
}

#if 1
/*
 * Get all of the directory information. This is a hack.
 *
 * The result of the GetExtendedDirectory call is a packed structure having,
 * roughly, the following form. We must extract elements individually, however.
 */

typedef struct EachDirectoryData {
	PackedRLI		pRLI;			/*  packed RLI for the catalog				*/
	OSType			entnType;		/*  Entn Type								*/
	long			hasMailSlot;	/*  1 if mail slot, zero if not				*/
	ProtoRString	RealName;		/*  Packed RString for Real Name, padded	*/ 
	ProtoRString	comment;		/*  Packed RString, padded					*/
	long            length;			/*  data length								*/
	char            data[1];		/*  data[length], padded to even boundary	*/
};

OSErr
_AOMGetExtendedDirectoriesInfo(
		register AddressOMaticPtr	aomPtr
	)
{
		OSErr							status;
		register _AOMDirectoryInfoPtr	dirInfoPtr;
		DirParamBlock					pb;
		Handle							buffer;
		register Ptr					bufPtr;
		unsigned long					length;
		unsigned long					entryCount;
		unsigned short					pRLILength;
		PackedRLIPtr					pRLIPtr;
		PackedRLIPtr					*packedRLIHandle;
		RLI								theRLI;
		OSType							entnType;
		
#define GET	(pb.getExtendedDirectoriesInfoPB)

		buffer = NULL;
		if (AOM.dirInfoHdl == NULL)
			AOM.dirInfoHdl = (_AOMDirectoryInfoHdl) NewHandle(0);
		else {
			_AOMCleanOutDirInfoHandle(aomPtr);
			SetHandleSize((Handle) AOM.dirInfoHdl, 0);
		}
		if ((status = MemError()) != noErr)
			goto exit;
		buffer = NewHandle(kWorkBufferSize);
		if ((status = MemError()) != noErr)
			goto exit;
		while (status == noErr) {
			MoveHHi(buffer);
			HLock(buffer);
			CLEAR(pb);
			GET.identity = AOM.identity;
			GET.buffer = *buffer;
			GET.bufferSize = GetHandleSize(buffer);
			status = DirGetExtendedDirectoriesInfo(&pb, SYNC);
			if (status != kOCEMoreData)
				break;
			HUnlock(buffer);
			SetHandleSize(buffer, GetHandleSize(buffer) + kWorkBufferSize);
			status = MemError();
		}
		LOG(status, "\pDirGetExtendedDirectoriesInfo final status");
		if (status != noErr)
			goto exit;
		/*
		 * Paw through the mess to extract the directory info. Note that the
		 * buffer is still locked.  INCR macro increments bufPtr by some amount,
		 * making sure that it is rounded up to a word boundary.
		 */
#define INCR(v)	(bufPtr += ((v) + ((v) & 0x01)))
		bufPtr = (*buffer);
		for (entryCount = 0; entryCount < GET.actualEntries; entryCount++) {
			pRLIPtr = (PackedRLIPtr) bufPtr;
			pRLILength = pRLIPtr->dataLength;
			INCR(pRLILength + sizeof (ProtoPackedRLI));
			entnType = *((OSType *) bufPtr);
			INCR(sizeof (OSType) + sizeof (long));		/* -> real name string	*/
			length = ((ProtoRString *) bufPtr)->dataLength;
			INCR(length + sizeof (ProtoRString));
			length = ((ProtoRString *) bufPtr)->dataLength;
			INCR(length + sizeof (ProtoRString));		/* -> comment text		*/
			length = *((long *) bufPtr);				/* Data length			*/
			INCR(length + sizeof (long));				/* Step over the rest	*/
			/*
			 * Whoof! Now all we need to do is to save a copy of the
			 * packed RLI and entity, and stuff the name into the menu.
			 */
			/*
			 * Append the directory name to the menu handle. This is a
			 * two-step process to ensure that metacharacters are not
			 * interpreted.
			 */
			OCEUnpackRLI(pRLIPtr, &theRLI);
			AppendMenu(AOM.popupMenu, "\pX");
			SetItem(
				AOM.popupMenu,
				CountMItems(AOM.popupMenu),
				OCERToPString((const RStringPtr) theRLI.directoryName)
			);
			/*
			 * Now, save the RLI and address type in our permanent store.
			 */
			status = PtrToHand(
						pRLIPtr,
						(Handle *) &packedRLIHandle,
						pRLIPtr->dataLength + sizeof (ProtoPackedRLI)
					);
			if (status != noErr)
				goto exit;
			SetHandleSize(
				(Handle) AOM.dirInfoHdl,
				GetHandleSize((Handle) AOM.dirInfoHdl)
					+ sizeof (_AOMDirectoryInfoRecord)
			);
			if ((status = MemError()) != noErr) {
				DisposeHandle((Handle) packedRLIHandle);
				goto exit;
			}
			dirInfoPtr = &(*AOM.dirInfoHdl)[DIR_INFO_SIZE - 1];
			dirInfoPtr->packedRLIHandle = packedRLIHandle;
			dirInfoPtr->entnType = entnType;
		}								/* Do for all directory entries			*/
exit:	if (buffer != NULL)
			DisposeHandle(buffer);
		if (status != noErr) {
			/* TBS: dispose of the contents of the directory stuff
			*/
		}
		return (status);
}				
#else
/*
 * Not used any more. Apparently it was too simple.
 * Keep the sources around for "enumeration" TechNote.
 */

/*
 * _AOMEnumerateDirectories looks for all directories. It builds the
 * directory menu popup and saves the directory info in a private
 * handle-based structure.
 */
OSErr
_AOMEnumerateDirectories(
		register AddressOMaticPtr	aomPtr
	)
{
		OSErr							status;
		DirParamBlock					pb;
		register _AOMDirectoryInfoPtr	dirInfoPtr;
		Boolean							first;
		Ptr								buffer;
		short							lastDirectoryIndex;
#define GET		(pb.enumerateDirectoriesGetPB)
#define PARSE	(pb.enumerateDirectoriesParsePB)
		
		if (AOM.dirInfoHdl == NULL)
			AOM.dirInfoHdl = (_AOMDirectoryInfoHdl) NewHandle(0);
		else {
			SetHandleSize((Handle) AOM.dirInfoHdl, 0);
		}
		status = MemError();
		if (status == noErr) {
			buffer = NewPtr(kWorkBufferSize);
			if (buffer == NULL)
				status = MemError();
		}
		if (status == noErr) {
			CLEAR(pb);
			first = TRUE;
			do {
				GET.identity = AOM.identity;
				GET.ioCompletion = NULL;
				GET.directoryKind = kDirAllKinds;	/* Is this correct?			*/
				MoveHHi((Handle) AOM.dirInfoHdl);
				HLock((Handle) AOM.dirInfoHdl);
				if (first)
					first = FALSE;
				else {
					dirInfoPtr = &(*AOM.dirInfoHdl)[DIR_INFO_SIZE - 1];
					GET.startingDirectoryName = &(*dirInfoPtr).directoryName;
					GET.startingDirDiscriminator = (*dirInfoPtr).discriminator;
				}
				GET.includeStartingPoint = FALSE;
				GET.getBuffer = buffer;
				GET.getBufferSize = kWorkBufferSize;
				status = DirEnumerateDirectoriesGet(&pb, SYNC);	/* Synchronous	*/
				HUnlock((Handle) AOM.dirInfoHdl);
				if (status != kOCEMoreData)
					LOG(status, "\pDirEnumerateDirectoriesGet");
				if (status == noErr || status == kOCEMoreData) {
					/*
					 * We have some data.
					 */
					PARSE.eachDirectory = _AOMForEachDirectory;
					PARSE.clientData = (long) aomPtr;
					status = DirEnumerateDirectoriesParse(&pb, SYNC);
					if (status != kOCEMoreData)
						LOG(status, "\pDirEnumerateDirectoriesParse");
				}
			} while (status == kOCEMoreData);
		}
		if (buffer != NULL)
			DisposePtr(buffer);
		return (status);
}

pascal Boolean
_AOMForEachDirectory(
		long						clientData,
		const DirectoryName			*dirName,
		const DirDiscriminator		*discriminator,
		DirGestalt					features
	)
{
		register AddressOMaticPtr		aomPtr;
		register _AOMDirectoryInfoPtr	dirInfoPtr;
		Boolean							result;

		aomPtr = (AddressOMaticPtr) clientData;
		SetHandleSize(
			(Handle) AOM.dirInfoHdl,
			GetHandleSize((Handle) AOM.dirInfoHdl)
				+ sizeof (_AOMDirectoryInfoRecord)
		);
		result = (MemError() != noErr);
		if (result == FALSE) {
			/*
			 * This sequence does not move memory.
			 */
			dirInfoPtr = &(*AOM.dirInfoHdl)[DIR_INFO_SIZE - 1];
			OCECopyRString(
				(const RStringPtr) dirName,
				(RStringPtr) &(*dirInfoPtr).directoryName,
				kDirectoryNameMaxBytes
			);
			(*dirInfoPtr).discriminator = *discriminator;
			(*dirInfoPtr).features = features;
			/*
			 * Append the directory name to the menu handle. This is a two-step
			 * process to ensure that metacharacters are not interpreted.
			 */
			AppendMenu(AOM.popupMenu, "\pX");
			SetItem(
				AOM.popupMenu,
				CountMItems(AOM.popupMenu),
				OCERToPString((const RStringPtr) dirName)
			);
		}
		return (result);
}
#endif /* Unused Directory Enumerator */

/*
 * This removes the contents of the DirInfoHandle without removing
 * the handle itself.
 */
void
_AOMCleanOutDirInfoHandle(
		register AddressOMaticPtr	aomPtr
	)
{
		short							i;
		register _AOMDirectoryInfoPtr	directoryInfoPtr;

		HLock((Handle) AOM.dirInfoHdl);
		directoryInfoPtr = *AOM.dirInfoHdl;
		for (i = 0; i < DIR_INFO_SIZE; i++)
			ForgetHandle(directoryInfoPtr[i].packedRLIHandle);
		HUnlock((Handle) AOM.dirInfoHdl);
}


/*
 * Make an RString (pointer) from a handle. On return, the caller must
 * check MemError for errors. Note: we should check for a handle larger
 * than 65536 bytes.
 */
RStringPtr
_AOMMakeRStringFromHandle(
		Handle						textHandle,
		short						scriptCode
	)
{
		RStringPtr					result;
		unsigned long				textLength;
		
		textLength = GetHandleSize(textHandle);			
		result = (RStringPtr) NewPtr(textLength + sizeof (ProtoRString));
		if (result != NULL && MemError() == noErr) {
			result->charSet = scriptCode;
			result->dataLength = textLength;
			BlockMove(*textHandle, (Ptr) result->body, textLength);
		}
		return (result);
}

#endif /* ENABLE_TYPEIN */