/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AYS Application Framework
 *
 * 1991 Apple Computer
 *
 -------------------------------------------------------------------------------------*/
/*
 * dialog.c -- handles dialog items
 *
 * change history:
 *
 * SRK		3/9/93		1.0d1		initial coding
 *
 */

#include "const.h"
#include "mymenus.h"
#include "globals.h"
#include "utils.h"
#include "commands.h"
#include "dialog.h"
#include "events.h"

#include <OCEStandardDirectory.h>



// *****************************************************************
// *	SetControlState
// *
// *	sets our connect button to the desired state (active or inactive)
// *****************************************************************
void SetControlState (DialogPtr dlg, short item, short mode)
{
	Rect r;
	short kind;
	Handle h;

		GetDItem(dlg, item, &kind, &h, &r);
		HiliteControl((ControlHandle)h, mode);
}


// *****************************************************************
// *	UpdateItemBorder
// *
// *	re-draws the borders around our items
// *****************************************************************
void UpdateItemBorder (short item, Rect r)
{
	FontInfo fInfo;
	Rect tRect;
	Str255 theBorderString;


		theBorderString[0] = 0;

		CtoPstr((char *)&theBorderString);

		GetFontInfo(&fInfo);
		FrameRect(&r);
		MoveTo(r.left + 5, r.top + fInfo.ascent / 2 - 1);
		tRect.left = r.left + 4;
		tRect.right = tRect.left + StringWidth(theBorderString) + 1;
		tRect.top = r.top;
		tRect.bottom = r.top + 1;
		EraseRect(&tRect);
		DrawString((StringPtr)&theBorderString);

}


// *****************************************************************
// *	UpdateUserItems
// *
// * 	update procedure for the user items in our dialog.
// *****************************************************************
pascal void UpdateUserItems (WindowPtr whichWindow, short theItem)
{
	GrafPtr savedPort;
	Rect r;
	short kind;
	Handle h;

		GetPort(&savedPort);
		SetPort(whichWindow);
		GetDItem(whichWindow, theItem, &kind, &h, &r);
		switch(theItem)
		{
			default: 
				UpdateItemBorder(theItem, r);
				break;

		}
		
		SetPort(savedPort);
}


// *****************************************************************
// *	setEachUserItem
// *
// *	sets the update routine for our dialogs user items
// *****************************************************************

void setEachUserItem (DialogPtr dlg,short item)
{
	Rect r;
	short kind;
	Handle h;
	
		GetDItem(dlg, item, &kind, &h, &r);
		SetDItem(dlg, item, userItem, (Handle)UpdateUserItems, &r);
}

// *****************************************************************
// *	MakeDialogWithSDPanel
// *
// *	create our dialog window
// *****************************************************************

void MakeOurDialog(DialogPtr *dlg)
{
	DlogDataPtr dlogData;
	
		dlogData = (DlogDataPtr) NewPtr(sizeof(DlogData));
		if (MemError()!=noErr) {
			DoError(MemError());
			ExitToShell();
		}
		
		*dlg = GetNewDialog(kStatusBoxID,nil,kInFront);
		SetPort(*dlg);
	
		setEachUserItem (*dlg,kRecordBox);
		setEachUserItem (*dlg,kDirBox);
	
		SetControlState (*dlg,kOpenButton, kItemDisable);
	
		SetWRefCon(*dlg,(long)dlogData);
}


// *****************************************************************
// *	CloseOurDialog
// *
// *	dispose of our Directory Browser and our dialog window
// *****************************************************************

void CloseOurDialog(DialogPtr dlg)
{
SDPPanelHandle panel;

	if (GetPanelForThisDlg(dlg,&panel) == true)
		SDPDisposePanel(panel);
		
	if (dlg != nil)
		DisposeDialog(dlg);
		
	if (sParts != nil)		
		DisposePtr((Ptr)sParts);
}



// *****************************************************************
// *	HandleDialogEvents
// *
// *	handles events important to our dialog
// *****************************************************************

Boolean HandleDialogEvents(EventRecord *ev)
{
	DialogPtr theDlg;
	short item;
	SDPPanelState whatHappened;
	OSErr err;
	SDPPanelHandle panel;

	
	if (!IsDialogEvent(ev))
		return false;
	
	if (DialogSelect(ev,&theDlg,&item)) 
	{
// ########
		if (GetPanelForThisDlg(theDlg,&panel) == false)
			return false;
	
		switch (item) 
		{
			case kSDPanel:
				whatHappened = DoPanelEvent(ev,panel);
				switch (whatHappened)
				{
					case kSDPSelectedAnItem:
#if 0
!!!!!!!!! we never get this event !!!!!!!!!!!!!!!!!

		DebugStr("\pkSDPSelectedAnItem");

						err = SDPOpenSelectedItem(panel,&whatHappened);
						HandleUserSelection(theDlg,whatHappened,panel);
#endif
						break;
					case kSDPChangedSelection:
						HandleUserSelection(theDlg,whatHappened,panel);
						break;
					case kSDPProcessed:
//		DebugStr("\pkSDPProcessed");
						break;
			
				}
				break;
			case kOpenButton:
					// disable open button before we open the current selection, since
					// there will be no selection initially for the opened item
				SetControlState (theDlg, kOpenButton, kItemDisable);
				err = SDPOpenSelectedItem(panel,&whatHappened);
				HandleUserSelection(theDlg,whatHappened,panel);

				break;
		}
		return true;
	}
	
	return false;
}

// *****************************************************************
// *	GetPanelForThisDlg
// *
// *	returns the panel handle for the desired dialog
// *****************************************************************

Boolean GetPanelForThisDlg(DialogPtr theDlg,SDPPanelHandle *panel)
{
DlogDataPtr dlogData;

	dlogData = (DlogDataPtr)GetWRefCon(theDlg);
	if (dlogData != nil)
	{
		*panel = dlogData->ThePanel;
		return (true);
	}
	else
	{
		*panel = nil;
		return (false);
	}

}

// *****************************************************************
// *	SetPanelForThisDlg
// *
// *	sets the panel handle for the desired dialog
// *****************************************************************

void SetPanelForThisDlg(DialogPtr theDlg,SDPPanelHandle panel)
{
DlogDataPtr dlogData;

	dlogData = (DlogDataPtr)GetWRefCon(theDlg);
	if (dlogData != nil)
	{
		dlogData->ThePanel = panel;
	}
}
