/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AOCE Application Framework
 *
 * 1991 Apple Computer
 *
 -------------------------------------------------------------------------------------*/
/*
 * events.c -- main event loop and basic event handling
 *
 * change history:
 *
 * SRK		3/9/93		1.0d1		initial coding
 *
 */

#include "const.h"
#include "mymenus.h"
#include "globals.h"
#include "utils.h"
#include "commands.h"
#include "dialog.h"
#include "events.h"
#include "SDP.h"
#include "stdio.h"

#include <OCEStandardDirectory.h>





// *****************************************************************
// *	MainLoop
// *
// *	main event loop
// *****************************************************************
void MainLoop(void)
{
	EventRecord ev;
	Boolean gotEvent;

	while (!gDone) {
		if (gHasWaitNextEvent)
			gotEvent = WaitNextEvent(everyEvent,&ev,SleepTime(),nil);
		else {
			gotEvent = GetNextEvent(everyEvent,&ev);
			SystemTask();
		}
		
		if (!(HandleDialogEvents(&ev)))
			ProcessEvent(&ev);
	}
}


// *****************************************************************
// *	SleepTime
// *
// *	calculate how long to "wait"
// *****************************************************************
long SleepTime(void)
{
	if (gInBackground)
		return kSleepBackground;
	else
		return kSleepForeground;
}


// *****************************************************************
// *	HandleIdle
// *
// *	idle time processing
// *****************************************************************
void HandleIdle(EventRecord *ev, WindowPtr window)
{
OSErr err;
SDPPanelState whatHappened;
SDPPanelHandle panel;

	if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
		err = SDPPanelEvent(panel,ev,&whatHappened);

}



// *****************************************************************
// *	ShowAbout
// *
// *	show about box
// *****************************************************************
void ShowAbout(void)
{
	DialogPtr theDlg;
	short item;
	
	theDlg = GetNewDialog(kAboutBoxID,nil,kInFront);
	do 
	{
		ModalDialog(nil,&item);
	} while (item!=ok);
	
	DisposDialog(theDlg);
}


// *****************************************************************
// *	ProcessEvent
// *
// *	process an event gotten by MainLoop()
// *****************************************************************
void ProcessEvent(EventRecord *ev)
{
SDPPanelHandle panel;
	
	switch (ev->what) {
			case mouseDown:
			case mouseUp:
				HandleMouseDowns(ev);
				break;
			case keyDown:
			case autoKey:
				if (GetPanelForThisDlg((DialogPtr)FrontWindow(),&panel) == true)
					HandleKeyDownsForPanel(FrontWindow(),ev,panel);
				break;
			case updateEvt:
				HandleUpdates((WindowPtr)ev->message);
				break;
			case activateEvt:
				HandleActivates(ev);
				break;
			case osEvt:
				HandleSREvt(ev->where,ev->message);
				break;
//			case kHighLevelEvent:
//				DoHighLevelEvent(ev);
//				break;
			case nullEvent:
				HandleIdle(ev,FrontWindow());
				break;
	}
	
}



// *****************************************************************
// *	HandleSREvt
// *
// *	Handles suspend and resume events
// *****************************************************************
void HandleSREvt(Point where,long message)
{
	extern NMRec *gNotify;
	extern Boolean gInBackground;
	unsigned long whatMessage;
	
	whatMessage = message >> 24;
	
	if (whatMessage==suspendResumeMessage) {
		if ((message & 1) != 0) {
			gInBackground = false;
			SetCursor(&qd.arrow);
			if (FrontWindow()) {
				HiliteWindow(FrontWindow(),true);
				DoActivate(FrontWindow(),true);
			}
		}
		else if (FrontWindow()) {
			gInBackground = true;
			HiliteWindow(FrontWindow(),false);
			DoDeActivate(FrontWindow(),true);
		}
	}
	else if ((whatMessage&mouseMovedMessage)==mouseMovedMessage)
		;
}


// *****************************************************************
// *	HandleActivates
// *
// *	Handles activate and deactivate events for a window
// *****************************************************************
void HandleActivates(EventRecord *ev)
{

	if ((ev->modifiers & activeFlag) != 0) {
		DoActivate((WindowPtr)ev->message,((ev->modifiers & 0x0002) != 0));
	}
	else {
		DoDeActivate((WindowPtr)ev->message,((ev->modifiers & 0x0002) != 0));
	}
}


// *****************************************************************
// *	DoActivate
// *
// *	Handles activate events for a window
// *****************************************************************
void DoActivate(WindowPtr window,Boolean chFlag)
{
SDPPanelHandle panel;

	if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
	{
		SDPEnablePanel(panel,true);
		SDPUpdatePanel(panel,nil);
	}
}


// *****************************************************************
// *	DoDeActivate
// *
// *	Handles deactivate events for a window
// *****************************************************************
void DoDeActivate(WindowPtr window,Boolean chFlag)
{
SDPPanelHandle panel;

	if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
		SDPEnablePanel(panel,false);
}


// *****************************************************************
// *	HandleUpdates
// *
// *	handles update events for a window
// *****************************************************************
void HandleUpdates(WindowPtr window)
{
GrafPtr savePort;
SDPPanelHandle panel;

	
	GetPort(&savePort);
	SetPort(window);
	BeginUpdate(window);
	EraseRect(&window->portRect);

	if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
		SDPUpdatePanel(panel,savePort->visRgn);

	EndUpdate(window);
	SetPort(savePort);
}


// *****************************************************************
// *	HandleKeyDownsForPanel
// *
// *	handles program keydowns for panel
// *****************************************************************
void HandleKeyDownsForPanel(WindowPtr window, EventRecord *ev,SDPPanelHandle panel)
{
	short theChar;
	SDPPanelState whatHappened;
	OSErr err;
	
	
	theChar = ev->message & charCodeMask;
	if ((ev->modifiers & cmdKey) != 0)
		DoMenuCommand(window,MenuKey(theChar));
	else
	{
			// did they press the <return> key? If so, open up the
			// currently selected item.
		if (theChar == 0x0D)
		{
			err = SDPOpenSelectedItem(panel,&whatHappened);
			HandleUserSelection((DialogPtr)window,whatHappened,panel);
		}
		else
		{
				// first check if last key press was less than kMaxTickDelay ticks ago - if so,
				// append the current character onto our current search string and continue 
				// the search.
			if (((ev->when - gLastTime) < kMaxTickDelay))
			{
					// append character only if it doesn't overflow our buffer
				if (sKeyDowns.dataLength < kRStringMaxBytes)
				{
						// append key onto our previously saved string
					sKeyDowns.body[sKeyDowns.dataLength] = theChar;
					++sKeyDowns.dataLength;
				}
			}
			else
				// last key press was more than kMaxTickDelay ticks ago, so treat it as a "new"
				// key to search on
			{
				sKeyDowns.body[0] = theChar;
				sKeyDowns.dataLength = 1;
			}
			
				// save time of last key press for our comparison above
			gLastTime = ev->when;
				// go select the appropriate item based on our search string
			SDPSelectString(panel,&sKeyDowns);

// #######
// #######
			HandleUserSelection((DialogPtr)window,kSDPChangedSelection,panel);
		}
	}
}

// *****************************************************************
// *	DisplayDataForSelection
// *
// *	displays record and directory information for the selected item
// *****************************************************************
void DisplayDataForSelection(SDPPanelHandle panel,DialogPtr dlg)
{
	Rect iRect;
	Handle iHndl;
	short iType;
	OSErr err;

	PackedDSSpec selection;
	DSSpec dss;
	RLI theRLI;
	RecordID rid;


		err = SDPGetPanelSelection(panel,&selection);
		if (err == noErr)
		{
	
// ########
			TextFont(geneva);
			TextSize(12);
			
			OCEUnpackDSSpec(&selection,&dss,&rid);
			OCEUnpackRLI(rid.rli,&theRLI);
			
				// record name
			GetDItem(dlg,kRecName,&iType,&iHndl,&iRect);
			OCECopyRString(rid.local.recordName,&sTempRStr,kRStringMaxBytes);
			SetIText(iHndl,OCERToPString(&sTempRStr));
			
				// directory name
			GetDItem(dlg,kDirNameText,&iType,&iHndl,&iRect);
			OCECopyRString((RStringPtr)theRLI.directoryName,&sTempRStr,kRStringMaxBytes);
			SetIText(iHndl,OCERToPString(&sTempRStr));
			
				// record type
			GetDItem(dlg,kRecType,&iType,&iHndl,&iRect);
			OCECopyRString(rid.local.recordType,&sTempRStr,kRStringMaxBytes);
			SetIText(iHndl,OCERToPString(&sTempRStr));
			
				// DirDiscriminator signature
			GetDItem(dlg,kSigValue,&iType,&iHndl,&iRect);
			BlockMove((Ptr)&theRLI.discriminator.signature,(Ptr)&sTempStr[1],4);
			sTempStr[0] = 4;
			SetIText(iHndl,sTempStr);
		
				// DirDiscriminator misc.
			GetDItem(dlg,kMiscValue,&iType,&iHndl,&iRect);
			sprintf((char *)&sTempStr,"%9.9lX",theRLI.discriminator.misc);
			SetIText(iHndl,sTempStr);
		
				// directory node number
			GetDItem(dlg,kDNodeNum,&iType,&iHndl,&iRect);
			NumToString(theRLI.dNodeNumber,(StringPtr)&sTempStr);
			SetIText(iHndl,sTempStr);
			
				// CreationID source
			GetDItem(dlg,kSource,&iType,&iHndl,&iRect);
			NumToString(rid.local.cid.source,(StringPtr)&sTempStr);
			SetIText(iHndl,sTempStr);
		
				// CreationID sequence number
			GetDItem(dlg,kSequence,&iType,&iHndl,&iRect);
			NumToString(rid.local.cid.seq,(StringPtr)&sTempStr);
			SetIText(iHndl,sTempStr);
		
		}

}


// *****************************************************************
// *	HandleUserSelection
// *
// *	determine what to do with the current selection
// *****************************************************************
void HandleUserSelection(DialogPtr dlg,SDPPanelState whatHappened,SDPPanelHandle panel)
{
OSErr err;
SDPSelectionState ItsState;


	err = SDPGetPanelSelectionState(panel,&ItsState);
	if (err == noErr)
	{
		switch (ItsState)
		{
			case kSDPContainerSelected:
				if (whatHappened == kSDPChangedSelection)
				{
					SetControlState (dlg, kOpenButton, kItemEnable);
					DisplayDataForSelection(panel,dlg);
				}
				break;
			case kSDPRecordSelected :
				if (whatHappened == kSDPChangedSelection)
				{
					SetControlState (dlg, kOpenButton, kItemDisable);
					DisplayDataForSelection(panel,dlg);
				}
				break;
			case kSDPRecordAliasSelected:
				SetControlState (dlg, kOpenButton, kItemEnable);
				break;
			case kSDPLockedContainerSelected:
				SetControlState (dlg, kOpenButton, kItemDisable);
				break;
			case kSDPContainerAliasSelected:
				SetControlState (dlg, kOpenButton, kItemEnable);
				break;
			case kSDPNothingSelected:
				SetControlState (dlg, kOpenButton, kItemDisable);
				break;
	
		}
	}

}

// *****************************************************************
// *	DoPanelEvent
// *
// *	
// *****************************************************************
SDPPanelState DoPanelEvent(EventRecord *ev,SDPPanelHandle panel)
{
OSErr err;
SDPPanelState whatHappened;


			// pass the event to the Standard Directory Package
		err = SDPPanelEvent(panel,ev,&whatHappened);
		if (err == noErr)
			return whatHappened;
		else
			return err;

}

// *****************************************************************
// *	ClickedInOurPanel
// *
// *	
// *****************************************************************
Boolean ClickedInOurPanel(EventRecord *ev,Rect PanelDataBounds)
{
Point temp;


	SetPt(&temp,ev->where.h,ev->where.v);
	GlobalToLocal(&temp);
	
		// did they click in the SDP panel?
	if (PtInRect(temp,&PanelDataBounds) == true)
		return true;
	else
		return false;
}

// *****************************************************************
// *	HandleMouseDowns
// *
// *	handles mouse down events for a window
// *****************************************************************
void HandleMouseDowns(EventRecord *ev)
{
	WindowPtr window;
	short part;
	Rect limit;
	SDPPanelState whatHappened;
	SDPPanelHandle panel;

	
	
		SetRect(&limit,-32000,-32000,32000,32000);
	
		part = FindWindow(ev->where,&window);
		
		switch (FindWindow(ev->where,&window)) 
		{
			case inDrag:
				DragWindow(window,ev->where,&limit);
				break;
			case inGoAway:
				if (TrackGoAway(window,ev->where))
					gDone = true;
				break;
			case inMenuBar:
				DoMenuCommand(FrontWindow(),MenuSelect(ev->where));
				break;
			case inSysWindow:
				SystemClick(ev,window);
				break;
			case inContent:
				if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
				{
					if (ClickedInOurPanel(ev,(*panel)->bounds) == true)
					{					
						whatHappened = DoPanelEvent(ev,panel);
						HandleUserSelection((DialogPtr)window,whatHappened,panel);
					}
				}
				break;
		}
}

// *****************************************************************
// *	ErasePreviousMark
// *
// *	erase the previous menu selection check mark
// *****************************************************************
void ErasePreviousMark(short selMenu,short selItem)
{
MenuHandle theMenu;

	theMenu = GetMHandle(selMenu);
		/* un-mark old setting */
	SetItemMark(theMenu,selItem,0);

}

// *****************************************************************
// *	MarkNewSelection
// *
// *	mark the users menu selection
// *****************************************************************
void MarkNewSelection(short selMenu,short selItem)
{
MenuHandle theMenu;

	theMenu = GetMHandle(selMenu);
		/* mark new setting */
	SetItemMark(theMenu,selItem,kcheckChar);

}

// *****************************************************************
// *	DoMenuCommand
// *
// *	handles menu commands
// *****************************************************************
void DoMenuCommand(WindowPtr window,long mResult)
{
	short selItem,selMenu,temp;
	Str255 name;
	Boolean flag;
	GrafPtr tempPort;
	MenuHandle theMenu;
	OSErr err;
	SDPPanelHandle panel;
	
	
	selItem = LoWord(mResult);
	selMenu = HiWord(mResult);
	switch (selMenu) {
		case kAppleMenu:
			if (selItem>2) {
				GetPort(&tempPort);
				SetCursor(&qd.arrow);
				theMenu = GetMHandle(kAppleMenu);
				GetItem(theMenu,selItem,name);
				temp = OpenDeskAcc((ConstStr255Param)&name);
				SetPort(tempPort);
			}
			else ShowAbout();
			break;
		case kFileMenu:
			switch (selItem) {
				case kQuitItem:
					gDone = true;
					break;
			}
			break;
		case kEditMenu:
			SystemEdit(selItem-1);
			break;
			
		case kCategoriesMenu:
					// erase previous mark
				ErasePreviousMark(gLastMenuSelected,gCategoriesMenuSelItem);
					// mark the user's new selection
				MarkNewSelection(selMenu,selItem);
				
					// save current menu selected, in case they choose a hierarchical menu
					// (we need to know this so we can erase the old check mark when a
					// new item is selected)
				gLastMenuSelected = selMenu;
				gCategoriesMenuSelItem = selItem;
	
				OCECopyRString(sParts[gCategoriesMenuSelItem-1],&sCategorySelection,kRStringMaxBytes);

				if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
					ReBuildSDPPanel((DialogPtr)window,&panel);
			break;
		
		case kEnumFlagsMenu:
					// erase previous mark
				ErasePreviousMark(kEnumFlagsMenu,gFilterSelection);
					// mark the user's new selection
				MarkNewSelection(kEnumFlagsMenu,selItem);
				
				gFilterSelection = selItem;
	
				if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
					ReBuildSDPPanel((DialogPtr)window,&panel);

			break;
		
		default:
					// was a hierarchical (sub-menu) category-types menu selected?
				if (selMenu > 0 && selMenu < 235)
				{
					ErasePreviousMark(gLastMenuSelected,gCategoriesMenuSelItem);
					MarkNewSelection(selMenu,selItem);
		
					gCategoriesMenuSelItem = selItem;
					gLastMenuSelected = selMenu;
		
					theMenu = GetMHandle(selMenu);
					GetItem(theMenu,selItem,name);
					OCEPToRString(name,smRoman,&sCategorySelection,name[0]+1);
		
					if (GetPanelForThisDlg((DialogPtr)window,&panel) == true)
						ReBuildSDPPanel((DialogPtr)window,&panel);
				}
			break;

 	}
	HiliteMenu(0);
}

// *****************************************************************
// *	ExitProgram
// *
// *	get rid of main dialog and leave
// *****************************************************************
void ExitProgram(void)
{
	CloseDialog(gMainDialog);
}
