/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AYS Application Framework
 *
 * 1991 Apple Computer
 *
 -------------------------------------------------------------------------------------*/
/*
 * main.c -- main entry point and initialization
 *
 * change history:
 *
 * SRK		3/9/93		1.0d1		initial coding
 *
 */

#include <Traps.h>
#include <GestaltEqu.h>
#include <menus.h>
#include <OCE.h>
#include <OCEStandardDirectory.h>

#include "const.h"
#include "mymenus.h"
#include "globals.h"
#include "utils.h"
#include "trapavailable.h"
#include "events.h"
#include "dialog.h"
#include "sdp.h"
#include "main.h"

// *****************************************************************
// *	main
// *
// *	main entry point
// *****************************************************************
void main(void)
{
DlogDataPtr dlogData;
SDPPanelHandle newPanel;
Rect panelBounds;

	if (DoError(InitToolboxes()) == noErr)
	{
		if (!HasAOCE())
			StopAlert(kNoAYSAlertID,nil);
		else if (DoError(InitApplication()) == noErr)
		{
			if (DoError(DoAuthentication())==noErr)
			{
					// make sure the panel fits into the space allocated to it in the dialog
				CheckPanelSize(gMainDialog,&panelBounds,kSDPanel);
				
					// create the panel
				if (DoError(CreatePanelForThisDialog(gMainDialog,&newPanel,&panelBounds))==noErr)
				{
					SetPanelForThisDlg(gMainDialog,newPanel);
					MainLoop();
				}
				
			}
		}
	
	}
	
	ExitProgram();
	ExitToShell();
}


// *****************************************************************
// *	InitToolboxes
// *
// *	initializes the standard Mac toolbox
// *****************************************************************
OSErr InitToolboxes(void)
{

	MaxApplZone();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	FlushEvents(everyEvent,0L);
		
	return noErr;
}


// *****************************************************************
// *	InitApplication
// *
// *	initializes the application globals/menus/windows/etc...
// *****************************************************************
OSErr InitApplication(void)
{
OSErr err;
	
	gHasWaitNextEvent = TrapAvailable(_WaitNextEvent);
		
	gDone = false;
	gInBackground = false;

	err = InitMyMenus();
	InitMyWindows();
	
	return err;
}

// *****************************************************************
// *	InitMyMenus
// *
// *	build the Categories and Filter Menus
// *****************************************************************
OSErr InitMyMenus(void)
{
Handle mbar;
MenuHandle appleMenu;
OSErr err;

	
	
	mbar = GetNewMBar(kMenuBarID);
	if (mbar != nil)
		SetMenuBar(mbar);

	appleMenu = GetMHandle(kAppleMenu);
	if (appleMenu != nil)
		AddResMenu(appleMenu,'DRVR');			// build apple menu

		// build our Categories menu
	err = BuildCategoriesMenu(kCategoriesMenu);
		// build our Filter menu
	BuildFilterMenu(kEnumFlagsMenu);
	
	DrawMenuBar();
	
	return err;
}


// *****************************************************************
// *	BuildCategoriesMenu
// *
// *	build the Categories Menu
// *****************************************************************
OSErr BuildCategoriesMenu(short menuID)
{
MenuHandle CategoriesMHdl,CategoryTypesMHdl;

unsigned short actuallyFound,actualnSubParts;
unsigned short nSubParts,nParts;

short i,k;
short len;

OSErr err;

StringPtr CategoryStrPtr;
StringPtr SubCategoryStrPtr;

PackedRStringListHandle categories=NULL;
PackedRStringListHandle displayNames=NULL;
PackedRStringListHandle testTypes = NULL;

	
	
		// get a list of the AOCE directory-item categories
		// know to the Directory Browser
	err = SDPGetCategories(&categories,&displayNames);
	if (err != noErr)
		return (err);
		
		// how may are there?
	nParts = OCEDNodeNameCount(*displayNames);

		/* we need to add a type for "DNode" (see below) so
			we'll increment this here */
	++nParts;
	
		// Allocate memory for our category types. We need to add an
		// additional "dummy" item on the end to handle the "Everything" selection.
		// If the user selects this type then we set the matchTypeHow parameter
		// to SDPNewPanel to "KMatchAll"
	sParts = (RStringPtr *)NewPtr((nParts+1) * sizeof(RStringPtr) );
	if (sParts == nil)
		return (MemError());

	actuallyFound = OCEUnpackPathName( *displayNames,sParts,nParts);

	if (actuallyFound != (nParts-1))
	{
		nParts = actuallyFound;
	}

		/* add record type of "DNode" so we can display aliases of dNodes */
	sParts[nParts-1] = OCEGetIndRecordType(kDNodeRecTypeNum);
	
	// for now let's allocate enough memory to hold 15 category types. If
	// we find out we need more, we'll allocate the additional space below
	sSubParts = (RStringPtr *)NewPtr(kCatTypesLimit * sizeof(RStringPtr) );
	if (sSubParts == nil)
		return (MemError());
	
	CategoriesMHdl = GetMenu(menuID);

	for (i=0;i<nParts; ++i)
	{
			// let's make a Pascal string out of the category RString
			// so we can easily add it to our Categories Menu
		OCECopyRString((sParts[i]),&sCategory,kRStringMaxBytes);
		CategoryStrPtr = OCERToPString(&sCategory);
	
			// get a list of types for this particular category
		err = SDPGetCategoryTypes(sParts[i],&testTypes);
		if (err != noErr)
			return (err);

		nSubParts = OCEDNodeNameCount(*testTypes);
		if (nSubParts > 0)
		{
				// create a sub-menu for the types for this particular category
				// the '/' char. followed by a 0x1b means that this item has a sub-menu
			len = CategoryStrPtr[0];
			CategoryStrPtr[len+1] = '/';++CategoryStrPtr[0];
			CategoryStrPtr[len+2] = 0x1b;++CategoryStrPtr[0];
			
				// specify the menu id of the sub-menu
			len = CategoryStrPtr[0];
			CategoryStrPtr[len+1] = '!';++CategoryStrPtr[0];
				// i+1 is the sub-menu id
			CategoryStrPtr[len+2] = i+1;++CategoryStrPtr[0];
			
				// did we allocate enough space to hold all the category types? If not
				// we must go get more memory.
			if (nSubParts > kCatTypesLimit)
			{
					// de-allocate current space
				DisposePtr((Ptr)sSubParts);
					// get enough memory to hold "nSubParts" types
				sSubParts = (RStringPtr *)NewPtr(nSubParts * sizeof(RStringPtr) );
				if (sSubParts == nil)
					return (MemError());
				
			}
				// how many types are there for this category?
			actualnSubParts = OCEUnpackPathName(*testTypes,sSubParts,nSubParts);
		}
		else
		{
			actualnSubParts = 0;
		}

			// add the category type as a menu item to the "Categories" Menu
		AppendMenu(CategoriesMHdl,CategoryStrPtr);
		
		if (actualnSubParts != nSubParts)
		{
			nSubParts = actualnSubParts;
		}
		
			// now we add the category types for this category as a sub-menu
		if (actualnSubParts > 0)
		{
				// create the category types sub-menu
			CategoryTypesMHdl = NewMenu(i+1,"\pSubParts");
			
			for (k=0;k<actualnSubParts;++k)
			{
				OCECopyRString((sSubParts[k]),&sSubCategory,kRStringMaxBytes);
				SubCategoryStrPtr = OCERToPString(&sSubCategory);
				
					// add the category types as a sub-menu for this particular category menu-item
				AppendMenu(CategoryTypesMHdl,SubCategoryStrPtr);
			}
			
			InsertMenu(CategoryTypesMHdl,-1);
		
		}
	
	}
	
// #################
// #################
		// Add a dummy item called "Everything", to handle the case
		// where the user wants to see all types.
	AppendMenu(CategoriesMHdl,"\pEverything");
	sParts[nParts] = &sDummyEverything;

	DisposePtr((Ptr)sSubParts);

		// set current menu selection to the Categories menu
	gLastMenuSelected = menuID;
		// set the initial Category types selection to "Everything", which we tack onto
		// the end of the Categories menu
	gCategoriesMenuSelItem = nParts+1;

	MarkNewSelection(menuID,gCategoriesMenuSelItem);		
	OCECopyRString(sParts[gCategoriesMenuSelItem-1],&sCategorySelection,kRStringMaxBytes);
	
	return noErr;
}


// *****************************************************************
// *	BuildFilterMenu
// *
// *	build the Filter menu
// *****************************************************************
void BuildFilterMenu(short menuID)
{
MenuHandle mHdl;

// #################
		// set filter menu selection to "all entities" initially
	gFilterSelection = 5;
	mHdl = GetMenu(menuID);
	SetItemMark(mHdl,gFilterSelection,kcheckChar);
}

// *****************************************************************
// *	InitMyWindows
// *
// *	build the main dialog
// *****************************************************************
void InitMyWindows(void)
{
	MakeOurDialog(&gMainDialog);
}


// *****************************************************************
// *	HasAOCE
// *
// *	returns true only of AOCE is available and running
// *****************************************************************
Boolean HasAOCE(void)
{
	OSErr err;
	long response;

	if (!TrapAvailable(_GestaltDispatch))
		return false;
	
	err = Gestalt(gestaltOCEToolboxAttr,&response);
	if (err!=noErr)
		return false;
		
	return (response && (response << gestaltOCETBAvailable));
}