/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AYS Application Framework
 *
 * 1991 Apple Computer
 *
 -------------------------------------------------------------------------------------*/

#include <string.h>
#include <Script.h>

#include <OCE.h>
#include <OCEAuthDir.h>
#include <OCEMessaging.h>
#include <OCEErrors.h>
#include <OCEStandardMail.h>
#include <OCEStandardDirectory.h>

#include "SDP.h"
#include "globals.h"
#include "mymenus.h"
#include "dialog.h"


#if 0
RString gAllTypes = {smRoman,1,{'='}};
RString *gTypesList[] = {&gAllTypes};
#endif

// *****************************************************************
// *	DoAuthentication
// *
// *	check for user authentication
// *****************************************************************
OSErr DoAuthentication(void)
{
OSErr err;
AuthGetLocalIdentityPB pb;

		// first check if the user has already been authenticated. If so
		// we dont want to bother with authenticating again.
	err = AuthGetLocalIdentity((AuthParamBlockPtr)&pb,false);

	if (err != noErr)
			// user has not been authenticated, so let's do it now
		err = GetIdentity();
	else
			// user has already been authenticated
		gIdentity = pb.theLocalIdentity;
	
	return err;
}

// *****************************************************************
// *	ReBuildSDPPanel
// *
// *	dispose of and then re-create the Directory Browser
// *****************************************************************
void ReBuildSDPPanel(DialogPtr dlg,SDPPanelHandle *panel)
{
OSErr err;
Rect panelBounds;

		// disable the "open" button
	SetControlState (dlg, kOpenButton, kItemDisable);
	
		// dispose of the panel
	err = SDPDisposePanel(*panel);
	
		// we already checked the panel bounds at startup, but we'll do
		// it again here anyway
	CheckPanelSize(dlg,&panelBounds,kSDPanel);
	
		// re-build it for this dialog
	CreatePanelForThisDialog(dlg,panel,&panelBounds);
	
		// save the panel handle for this dialog
	SetPanelForThisDlg(dlg,*panel);


}

// *****************************************************************
// *	GetIdentity
// *
// *	authenticate the user
// *****************************************************************
OSErr GetIdentity(void)
{
OSErr err;
SDPIdentityKind selectedKind;
	
	
	err = SDPPromptForID(&gIdentity,
							  nil,
							  nil,
							  nil,
							  OCEGetIndRecordType(kUserRecTypeNum),
							  kSDPLocalIdentityMask,
							  &selectedKind,
							  nil,
							  0);	/* ignored */
	return err;
}


// *****************************************************************
// *	CheckPanelSize
// *
// *	make sure the Directory Broswer "fits" into the space alloted to it
// *****************************************************************
void CheckPanelSize(DialogPtr dlg,Rect *panelRect,short panelItem)
{
	Handle iHndl;
	short iType;
	FontInfo fInfo;
	short minWidth,minHeight,fontHeight;


	GetDItem(dlg,panelItem,&iType,&iHndl,panelRect);
// #### 
	GetFontInfo(&fInfo);
	minWidth = 5*fInfo.widMax + 42;
	fontHeight = fInfo.ascent + fInfo.descent + fInfo.leading;
	minHeight = fontHeight + 18;
// ##### 	
	minHeight = (64 > 3*fontHeight) ? (minHeight + 64) : (minHeight + 3*fontHeight);
	if ((panelRect->bottom - panelRect->top) < minHeight)
	{
		panelRect->bottom = panelRect->top + minHeight;
		SetDItem(dlg,panelItem,iType,iHndl,panelRect);
	}
	if ((panelRect->right - panelRect->left) < minWidth)
	{
		panelRect->right = panelRect->left + minWidth;
		SetDItem(dlg,panelItem,iType,iHndl,panelRect);
	}

}

// *****************************************************************
// *	SetMatchFlag
// *
// *	set flag that specifies how SDPNewPanel interprets the types list
// *****************************************************************
void SetMatchFlag(DirMatchWith *matchFlag,RString *CategorySelection)
{


		// We added a "dummy" item to our category types menu called
		// "Everything", so the user could display all types. If this
		// type was selected, we need to pass "kMatchAll" for the matchTypeHow
		// parameter to SDPNewPanel
		
	if (OCEEqualRString(&sDummyEverything,CategorySelection,kOCEGenericInsensitive) == true)
		*matchFlag = kMatchAll;
			
		// otherwise, match on the particular type that was selected from
		// the Categories Menu.
	else
		*matchFlag = kExactMatch;
	
}

// *****************************************************************
// *	CreatePanelForThisDialog
// *
// *	create a Directory Browser panel for this dialog
// *****************************************************************
OSErr CreatePanelForThisDialog(DialogPtr dlg,SDPPanelHandle *panel,Rect *panelDataBounds)
{
OSErr err;
DirMatchWith matchFlag;
RString *spCategorySelection;

	spCategorySelection = &sCategorySelection;
	
	SetMatchFlag(&matchFlag,spCategorySelection);
	
	err = SDPNewPanel(panel,
					(WindowPtr)dlg,
					panelDataBounds,
					true,
					true,
					nil,
					&spCategorySelection,
					1,
					gIdentity,
					gmyEnumFlags[gFilterSelection-1],
					matchFlag,
					0L);

	return err;

}
