/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AYS Application Framework
 *
 * 1991 Apple Computer
 *
 -------------------------------------------------------------------------------------*/
/*
 * utils.c -- utility routines
 *
 * change history:
 *
 * SJF		11/6/91		1.0d1		initial coding
 *
 */

#include <GestaltEqu.h>
#include <Traps.h>
#include <Notification.h>

#include "const.h"
#include "globals.h"
#include "trapavailable.h"
#include "main.h"
#include "mymenus.h"
#include "AnimCursor.h"

#include "utils.h"

static AnimCursHnd gAnimCursor;

// *****************************************************************
// *	HasColorQD
// *
// *
// *****************************************************************	
Boolean HasColorQD(void)
{
	SysEnvRec theEnv;
	
	if (SysEnvirons(1,&theEnv) != noErr)
		return false;

	return (theEnv.hasColorQD);
}

// *****************************************************************
// *	DoError
// *
// *
// *****************************************************************	
OSErr DoError(OSErr err)
{
	Str255 errStr;
	Str255 errNumStr;
	
	if (err==noErr)
		return noErr;
	
	NumToString(err,errNumStr);
	pstrcpy(errStr,"\pAn error has occurred: ");
	pstrcat(errStr,errNumStr);
	
	Notify(errStr);
//	DebugStr(errStr);

	return err;
}

// *****************************************************************
// *	Notify
// *
// *	puts up error dialog
// *****************************************************************	
void Notify(StringPtr string)
{
	NMRecPtr nm;
	StringPtr strPtr;
	
	nm = (NMRecPtr)NewPtr(sizeof(NMRec));
	if (MemError()!=noErr)
		return;
	strPtr = (StringPtr)NewPtr(string[0]);
	if (MemError()!=noErr)
		return;
	BlockMove(string,strPtr,string[0]+1);
	
	nm->qType = nmType;
	nm->nmMark = 0;
	nm->nmIcon = nil;
	nm->nmSound = nil;
	nm->nmStr = strPtr;
	nm->nmResp = nil;
	NMInstall(nm);
}

// *****************************************************************
// *	pstrcpy
// *
// *	copies a pascal string
// *****************************************************************	
void pstrcpy(void *dest,void *src)
{
	unsigned char srcLen = ((unsigned char *)src)[0];
	
	BlockMove(src,dest,srcLen+1);
}

// *****************************************************************
// *	pstrcat
// *
// *	pascal string concatenate function
// *****************************************************************	
void pstrcat(void *original,void *catStr)
{
	short length;
	unsigned char originalLen = ((unsigned char *)original)[0];
	unsigned char catStrLen = ((unsigned char *)catStr)[0];
	
	length = (short) originalLen;
	length += (short) catStrLen;
	
	if (length > 255) {
		DebugStr("\pstring catenation overflow");
		ExitProc();
	}
	
	BlockMove((char *)catStr+1,(char *)original+originalLen+1,catStrLen);
	((unsigned char *)original)[0] = (unsigned char) length;
}

// *****************************************************************
// *	NewPtrChk
// *
// *	allocate a pointer to a chunk of memory and check for errors
// *****************************************************************	
void *NewPtrChk(Size ptrSize)
{
	Ptr thePtr;

	thePtr = NewPtr(ptrSize);
	if (MemError()!=noErr) {
		thePtr = nil;
		DoError(MemError());
	}
	
#if kDEBUG
	{
		long *longPtr = (long *)thePtr;
		*longPtr = kBetterBusErr;
	}
#endif
	return thePtr;
}

// *****************************************************************
// *	NewHandleChk
// *
// *	allocate a handle to a chunk of memory and check for errors
// *****************************************************************	
void *NewHandleChk(Size hndlSize)
{
	Handle theHndl;
	
	theHndl = NewHandle(hndlSize);
	if (MemError()!=noErr) {
		theHndl = nil;
		DoError(MemError());
	}
	
#if kDEBUG
	{
		long **longHndl = (long **)theHndl;
		**longHndl = kBetterBusErr;
	}
#endif
	return theHndl;
}

// *****************************************************************
// *	DisposPtrChk
// *
// *	deallocate a pointer
// *****************************************************************	
void DisposPtrChk(void *thePtr)
{
#if kDEBUG
	{
		long *longPtr = (long *)thePtr;
		*longPtr = kBetterBusErr;
	}
#endif

	DisposPtr(thePtr);
	if (MemError()!=noErr)
		DoError(MemError());
}

// *****************************************************************
// *	DisposHandleChk
// *
// *	deallocate a handle
// *****************************************************************	
void DisposHandleChk(void *theHndl)
{
#if kDEBUG
	{
		long **longHndl = (long **)theHndl;
		**longHndl = kBetterBusErr;
	}
#endif

	DisposHandle(theHndl);
	if (MemError()!=noErr)
		DoError(MemError());
}

// *****************************************************************
// *	WaitPBDone
// *
// *	wait for an asynchronous call to complete
// *****************************************************************	
OSErr WaitPBDone(void *voidBlock)
{
	IOParam *pBlock;
	
	pBlock = (IOParam *)voidBlock;
	while (pBlock->ioResult > 0)
		Yield(0);
	return pBlock->ioResult;
}

// *****************************************************************
// *	Yield
// *
// *	
// *****************************************************************	
void Yield(long sleepTime)
{
	EventRecord ev;
	
	EventAvail(everyEvent,&ev);
}

// *****************************************************************
// *	ExitProc
// *
// *	
// *****************************************************************	
void ExitProc(void)
{
	ExitToShell();
}

// *****************************************************************
// *	InitAnimCursor
// *
// *	
// *****************************************************************	
void InitAnimCursor(void)
{
	gAnimCursor = GetAnimCurs(128);
}

// *****************************************************************
// *	SpinAnimCursor
// *
// *	
// *****************************************************************	
void SpinAnimCursor(void)
{
	AnimateCurs(gAnimCursor);
}

// *****************************************************************
// *	ReleaseAnimCursor
// *
// *	
// *****************************************************************	
void ReleaseAnimCursor(void)
{
	DisposeAnimCurs(gAnimCursor);
	SetCursor(&qd.arrow);
}
