/*-------------------------------------------------------------------------------------
 *
 * Simple Sample AOCE Application Framework
 *
 * 1991-1993 Apple Computer
 *
 -------------------------------------------------------------------------------------*/
/*
 * base.window.c -- base class for windows
 *
 * change history:
 *
 * SJF		04/21/93		1.0b2		update to b2
 * SJF		03/01/93		1.0b1		added digital signatures
 * SJF		02/09/93		1.0b1		update to b1
 * SJF		10/13/92		1.0d4		update to a11
 * SJF		09/09/92		1.0d3		update to a9
 * SJF		05/07/92		1.0d2		update to a6
 * SJF		11/06/91		1.0d1		initial coding
 *
 */
 
#include "const.h"
#include "strconst.h"
#include "mytypes.h"
#include "globals.h"
#include "utils.h"
#include "windowstuff.h"

#include "base.window.h"


/* instantiate a new base window */

WindowPtr BaseMakeWindow(Rect *wRect,StringPtr title,Boolean visible,short wdefProc,Boolean goAwayFlag)
{
	WInfoHndl infoHndl;
	WInfoPtr infoPtr;
	WindowPtr theWindow;
	Str255 defaultTitle;
	

	if (title==nil) {
		GetResString(defaultTitle,kDefaultTitleID,kDefaultTitle);
		title = defaultTitle;
	}
	
	infoHndl = NewHandleChk(sizeof(WInfo));
	if (MemError()!=noErr)
		return nil;
	
	MoveHHi((Handle) infoHndl);
	HLock((Handle) infoHndl);
	infoPtr = *infoHndl;
	
	infoPtr->m_idle = BaseIdleWindow;
	infoPtr->m_fixCursor = BaseFixCursorWindow;
	infoPtr->m_activate = BaseActivateWindow;
	infoPtr->m_deactivate = BaseDeactivateWindow;
	infoPtr->m_update = BaseUpdateWindow;
	infoPtr->m_key = BaseKeyWindow;
	infoPtr->m_resize = BaseResizeWindow;
	infoPtr->m_click = BaseClickWindow;
	infoPtr->m_destroy = BaseDestroyWindow;
	infoPtr->m_undo = BaseUndoWindow;
	infoPtr->m_cut = BaseCutWindow;
	infoPtr->m_copy = BaseCopyWindow;
	infoPtr->m_paste = BasePasteWindow;
	infoPtr->m_clear = BaseClearWindow;
	infoPtr->m_print = BasePrintWindow;
	infoPtr->m_pageSetup = BasePageSetupWindow;
	infoPtr->m_save = BaseSaveWindow;
	infoPtr->m_load = BaseLoadWindow;
	infoPtr->m_event = BaseEventWindow;
	infoPtr->m_ctrlHit = BaseHitControlWindow;
	infoPtr->m_selectAll = BaseSelectAllWindow;
	infoPtr->m_group = BaseGroupWindow;
	infoPtr->m_unGroup = BaseUnGroupWindow;
	
	if (HasColorQD())
		theWindow = NewCWindow(nil,wRect,title,visible,wdefProc,(WindowPtr)-1L,goAwayFlag,(unsigned long)infoHndl);

	else
		theWindow = NewWindow(nil,wRect,title,visible,wdefProc,(WindowPtr)-1L,goAwayFlag,(unsigned long)infoHndl);
	
	SetWindowKind(theWindow,kBaseWindow);
	
	pstrcpy(infoPtr->fileSpec.name,defaultTitle);
	infoPtr->refCount = gNextWindowToMake++;
	infoPtr->window = theWindow;
	infoPtr->saved = false;
	infoPtr->changed = false;
	infoPtr->fRefNum = 0;
	infoPtr->resRefNum = 0;
	infoPtr->topIndent = 0;
	infoPtr->leftIndent = 0;
	
	infoPtr->printRecord = NewHandleChk(sizeof(TPrint));
	if (MemError()!=noErr)
		return nil;
		
	PrOpen();
	PrintDefault(infoPtr->printRecord);
	if (PrError()!=noErr)
		DoError(PrError());
	PrClose();
	
	return theWindow;
}


void *BaseDestroyWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{	
	#pragma unused (window,data)
	
	DisposHandleChk(infoPtr->printRecord);
	
	return nil;
}


void *BaseIdleWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseFixCursorWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseActivateWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (infoPtr,data)
	
	MyDrawGrowIcon(window);
	return nil;
}


void *BaseDeactivateWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (infoPtr,data)
	
	MyDrawGrowIcon(window);
	SetDefaultMenus();
	gMenusDirty = true;
	return nil;
}


void *BaseUpdateWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (infoPtr,data)
	
	MyDrawGrowIcon(window);
	return nil;
}


void *BaseResizeWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{	
	#pragma unused (infoPtr,data)
	Rect gbRect;
	
	gbRect = window->portRect;
	gbRect.top = gbRect.bottom - kGrowBoxWidth;
	gbRect.left = gbRect.right - kGrowBoxWidth;
	EraseRect(&gbRect);

	return nil;
}


void *BaseKeyWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseClickWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,data)
	
	infoPtr->changed = true;
	return nil;
}


void *BaseUndoWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseCutWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseCopyWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BasePasteWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseClearWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseSelectAllWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseGroupWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseUnGroupWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BasePrintWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BasePageSetupWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,data)
	
	infoPtr->changed = true;
	
	return nil;
}


void *BaseSaveWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,data)
	
	infoPtr->saved = true;
	infoPtr->changed = false;

	return nil;
}


void *BaseLoadWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,data)
	
	infoPtr->saved = true;
	infoPtr->changed = false;

	return nil;
}


void *BaseEventWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}


void *BaseHitControlWindow(WindowPtr window,WInfoPtr infoPtr,void *data)
{
	#pragma unused (window,infoPtr,data)
	
	return nil;
}

